/*
 ast.syn -- Create an Abstract Syntax Tree
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 The grammar contained in this file describes the same language as the
 grammar in pll.syn. The differences can be categorized as follows:
   . Rules rewritten to make computation easier. In pll.syn all of the
     arithmetic operators have been factored out. Here the arithmetic
     operators have been substituted back into the rules for expressions to
     make computation more straightforward.
   . Rules rewritten to handle looping. The rules for loop constructs have
     been rewritten to handle repetitive parsing of loops.
   . Semantically determined rules (true, false) added to support parse time
     implementation of loops and if-else statements.

 The actual parser modules, ast.h and ast.cpp are created from
 ast.syn by the AnaGram parser generator using the Build Parser command.

 To build a demonstration program that illustrates the use of this parser in
 an interpreter that executes from an abstract syntax tree, compile and link
 as a single single project:
   astxi.cpp               // Execution module
   demo.cpp                // Demo program
   ast.cpp                 // This parser, creates abstract syntax tree
   astdefs.cpp             // Node definitions
   comdefs.cpp             // Common script definitions
   agclib1\src\ag*.cpp     // Supporting class library
 Include files describing the class library are in agclib1\include

 To build a demonstration program that illustrates the use of this parser in
 an interpreter that compiles bytecode from an abstract syntax tree, compile
 and link as a single single project:
   astci.cpp               // Execution module
   demo.cpp                // Demo program
   ast.cpp                 // This parser, creates abstract syntax tree
   astdefs.cpp             // Node definitions
   bcidefs.cpp             // Bytecode and compiler definitions
   comdefs.cpp             // Common script definitions
   agclib1\src\ag*.cpp     // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is somewhat similar to Pascal.

 The language consists of Pascal like statements and expressions. The
 language also implements Fortran-style exponentiation and simple dump and
 print statements. The syntax below uses "open" and "closed" statements to
 eliminate the traditional "dangling else" or "if-else" ambiguity problem.
 See http://www.parsifalsoft.com/ifelse.html or doc/ifelse.htm for a more
 detailed discussion.

 Statement types supported are:
   assignment statements
   compound statements
   if/else statements
   while statements
   repeat/until statements
   for statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"
#include <math.h>
#include "astdefs.h"         // Defines node structure of abstract syntax tree
#include <ctype.h>


// Forward reference for parxer control block
struct parse_pcb_struct;

struct Tree : public AbstractSyntaxTree {
  parse_pcb_struct *pcbPointer;
  FileLocation context() const;
};

#define AST PCB.ast

/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 26 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef AST_H_1027348577
#include "ast.h"
#endif

#ifndef AST_H_1027348577
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE parse_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer
static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 470, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

#define GET_CONTEXT CONTEXT = FileLocation(PCB.pointer, PCB.line, PCB.column)

FileLocation Tree::context() const {
  return PCONTEXT(*pcbPointer);
}

// Function definitions for parser control block

// Constructor
parse_pcb_struct::parse_pcb_struct(const char *text)
  : pointer((const unsigned char *) text), loopDepth(0)
{
  ast.pcbPointer = this;
}

void parse_pcb_struct::checkLoop() {
  if (loopDepth) return;
  char buf[100];
  FileLocation &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): No loop active", context.line, context.column);
  throw ErrorDiagnostic(buf);
}

// Record syntax error
void parse_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

// External Interface
AbstractSyntaxTree buildTree(const char *text) {
  parse_pcb_type pcb(text);
  parse(&pcb);
  return pcb.ast;
}


#ifndef CONVERT_CASE

static const char agCaseTable[31] = {
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,    0,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20
};

static int agConvertCase(int c) {
  if (c >= 'a' && c <= 'z') return c ^= 0x20;
  if (c >= 0xe0 && c < 0xff) c ^= agCaseTable[c-0xe0];
  return c;
}

#define CONVERT_CASE(c) agConvertCase(c)

#endif


#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif

static inline void ag_rp_1(PCB_DECL, AgStack<AstNode *> &sl) {
/* Line 145, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  AST.root = AST.node(typeScript, AST.listNode(typeStatementBlock, sl));
}

static inline AgStack<AstNode *> ag_rp_2(PCB_DECL) {
/* Line 149, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AgStack<AstNode *>();
}

static inline AgStack<AstNode *> ag_rp_3(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 150, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return s.push(x);
}

static inline AstNode * ag_rp_4(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 168, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.node(typeIfStatement, x, s);
}

static inline AstNode * ag_rp_5(PCB_DECL, AstNode * x, AstNode * s1, AstNode * s2) {
/* Line 170, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.node(typeIfElseStatement, x, s1, s2);
}

static inline AstNode * ag_rp_6(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 172, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return PCB.loopDepth--, AST.node(typeWhileStatement, x, s);
}

static inline AstNode * ag_rp_7(PCB_DECL, AstNode * lv, AstNode * b, AstNode * i, AstNode * e, AstNode * s) {
/* Line 175, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.node(typeSimpleFor, lv, b, i, e, s);
}

static inline AstNode * ag_rp_8(PCB_DECL, AstNode * x, AstNode * s1, AstNode * s2) {
/* Line 179, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.node(typeIfElseStatement,x,s1,s2);
}

static inline AstNode * ag_rp_9(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 181, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.node(typeWhileStatement,x,s);
}

static inline AstNode * ag_rp_10(PCB_DECL, AstNode * lv, AstNode * b, AstNode * i, AstNode * e, AstNode * s) {
/* Line 184, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.node(typeSimpleFor, lv, b, i, e, s);
}

static inline AstNode * ag_rp_11(PCB_DECL, AstNode * x) {
/* Line 185, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return x;
}

static inline AstNode * ag_rp_12(PCB_DECL) {
/* Line 188, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.constantNode(1);
}

static inline AstNode * ag_rp_13(PCB_DECL, AstNode * x) {
/* Line 189, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return x;
}

static inline AstNode * ag_rp_14(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 198, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.node(typeRepeatStatement,AST.listNode(typeStatementBlock, s), x);
}

static inline AstNode * ag_rp_15(PCB_DECL, AstNode * lv, AstNode * x) {
/* Line 199, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.assignmentNode(lv,SAP,x);
}

static inline AstNode * ag_rp_16(PCB_DECL) {
/* Line 200, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.node(typeNull);
}

static inline AstNode * ag_rp_17(PCB_DECL, AstNode * x) {
/* Line 202, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.node(typeReturn, x);
}

static inline AstNode * ag_rp_18(PCB_DECL, AgStack<AstNode *> &ns) {
/* Line 203, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.listNode(typeDump, ns);
}

static inline AstNode * ag_rp_19(PCB_DECL, AgStack<AstNode *> &ns) {
/* Line 204, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.listNode(typePrint, ns);
}

static inline AstNode * ag_rp_20(PCB_DECL, AstNode * x) {
/* Line 219, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return x;
}

static inline AstNode * ag_rp_21(PCB_DECL, AgStack<AstNode *> &s) {
/* Line 222, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.listNode(typeStatementBlock, s);
}

static inline AgStack<AstNode *> ag_rp_22(PCB_DECL, AgString &k) {
/* Line 226, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AgStack<AstNode *>().push(AST.variableNode(k));
}

static inline AgStack<AstNode *> ag_rp_23(PCB_DECL, AgStack<AstNode *> &ns, AgString &k) {
/* Line 227, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return ns.push(AST.variableNode(k));
}

static inline AgStack<AstNode *> ag_rp_24(PCB_DECL, AstNode * x) {
/* Line 231, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AgStack<AstNode *>().push(x);
}

static inline AgStack<AstNode *> ag_rp_25(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 232, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return s.push(x);
}

static inline AstNode * ag_rp_26(PCB_DECL) {
/* Line 235, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.node(typeNull);
}

static inline AstNode * ag_rp_27(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 242, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.binaryNode(op,x,y);
}

static inline AstNode * ag_rp_28(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 246, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.binaryNode(op,x,y);
}

static inline AstNode * ag_rp_29(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 250, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.binaryNode(op,x,y);
}

static inline AstNode * ag_rp_30(PCB_DECL, AstNode * x) {
/* Line 254, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return x;
}

static inline AstNode * ag_rp_31(PCB_DECL, Opcode op, AstNode * x) {
/* Line 255, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.unaryNode(op, x);
}

static inline AstNode * ag_rp_32(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 263, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.binaryNode(POW, x, y);
}

static inline AstNode * ag_rp_33(PCB_DECL, AstNode * x) {
/* Line 271, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return x;
}

static inline AstNode * ag_rp_34(PCB_DECL, AstNode * x) {
/* Line 273, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.unaryNode(FETCH, x);
}

static inline AstNode * ag_rp_35(PCB_DECL, AstNode * x) {
/* Line 275, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.unaryNode(CAST_LONG, x);
}

static inline AstNode * ag_rp_36(PCB_DECL, AstNode * x) {
/* Line 276, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.unaryNode(CAST_DOUBLE, x);
}

static inline AstNode * ag_rp_37(PCB_DECL, AgString &n) {
/* Line 279, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.variableNode(n);
}

static inline AstNode * ag_rp_38(PCB_DECL, AgString &k, AgStack<AstNode *> &args) {
/* Line 282, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.functionCallNode(k, args);
}

static inline AgStack<AstNode *> ag_rp_39(PCB_DECL) {
/* Line 285, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AgStack<AstNode *>();
}

static inline AgStack<AstNode *> ag_rp_40(PCB_DECL, AstNode * x) {
/* Line 289, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AgStack<AstNode *>().push(x);
}

static inline AgStack<AstNode *> ag_rp_41(PCB_DECL, AgStack<AstNode *> &al, AstNode * x) {
/* Line 290, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return al.push(x);
}

static inline AstNode * ag_rp_42(PCB_DECL, long x) {
/* Line 293, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_43(PCB_DECL, double x) {
/* Line 294, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_44(PCB_DECL, Value &x) {
/* Line 295, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_45(PCB_DECL, int x) {
/* Line 296, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AST.constantNode(x);
}

static inline Opcode ag_rp_46(PCB_DECL) {
/* Line 300, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return EQ;
}

static inline Opcode ag_rp_47(PCB_DECL) {
/* Line 301, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return NE;
}

static inline Opcode ag_rp_48(PCB_DECL) {
/* Line 302, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return LT;
}

static inline Opcode ag_rp_49(PCB_DECL) {
/* Line 303, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return LE;
}

static inline Opcode ag_rp_50(PCB_DECL) {
/* Line 304, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return GT;
}

static inline Opcode ag_rp_51(PCB_DECL) {
/* Line 305, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return GE;
}

static inline Opcode ag_rp_52(PCB_DECL) {
/* Line 308, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return ADD;
}

static inline Opcode ag_rp_53(PCB_DECL) {
/* Line 309, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return SUB;
}

static inline Opcode ag_rp_54(PCB_DECL) {
/* Line 310, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return IOR;
}

static inline Opcode ag_rp_55(PCB_DECL) {
/* Line 311, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return XOR;
}

static inline Opcode ag_rp_56(PCB_DECL) {
/* Line 314, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return MUL;
}

static inline Opcode ag_rp_57(PCB_DECL) {
/* Line 315, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return RDIV;
}

static inline Opcode ag_rp_58(PCB_DECL) {
/* Line 316, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return IDIV;
}

static inline Opcode ag_rp_59(PCB_DECL) {
/* Line 317, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return MOD;
}

static inline Opcode ag_rp_60(PCB_DECL) {
/* Line 318, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AND;
}

static inline Opcode ag_rp_61(PCB_DECL) {
/* Line 319, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return LS;
}

static inline Opcode ag_rp_62(PCB_DECL) {
/* Line 320, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return RS;
}

static inline Opcode ag_rp_63(PCB_DECL) {
/* Line 323, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return NEG;
}

static inline Opcode ag_rp_64(PCB_DECL) {
/* Line 324, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return NOT;
}

static inline AgString ag_rp_65(PCB_DECL, int c) {
/* Line 342, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AgString().append(tolower(c));
}

static inline AgString ag_rp_66(PCB_DECL, AgString &ns, int c) {
/* Line 343, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return ns.append(tolower(c));
}

static inline double ag_rp_67(PCB_DECL, double x, long e) {
/* Line 349, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return x*pow(10,e);
}

static inline double ag_rp_68(PCB_DECL, double x, long e) {
/* Line 350, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return x*pow(10,e);
}

static inline double ag_rp_69(PCB_DECL, double i, double f) {
/* Line 353, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return i+f;
}

static inline double ag_rp_70(PCB_DECL, double i) {
/* Line 354, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return i;
}

static inline double ag_rp_71(PCB_DECL, double f) {
/* Line 355, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return f;
}

static inline double ag_rp_72(PCB_DECL, long x) {
/* Line 358, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return x;
}

static inline double ag_rp_73(PCB_DECL, long x) {
/* Line 359, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return x;
}

static inline double ag_rp_74(PCB_DECL, long x) {
/* Line 360, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return makeDecimal(x);
}

static inline long ag_rp_75(PCB_DECL, long x) {
/* Line 363, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return (long)x;
}

static inline long ag_rp_76(PCB_DECL, long x) {
/* Line 364, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return -(long)x;
}

static inline double ag_rp_77(PCB_DECL, int d) {
/* Line 367, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_78(PCB_DECL, int d, double f) {
/* Line 368, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_79(PCB_DECL, int d) {
/* Line 376, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return d-'0';
}

static inline long ag_rp_80(PCB_DECL, long x, int d) {
/* Line 377, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_81(PCB_DECL, int d) {
/* Line 380, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return d-'0';
}

static inline long ag_rp_82(PCB_DECL, long x, int d) {
/* Line 381, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_83(PCB_DECL, long x, int d) {
/* Line 384, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_84(PCB_DECL, long x, int d) {
/* Line 385, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_85(PCB_DECL) {
/* Line 388, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return 0;
}

static inline long ag_rp_86(PCB_DECL, long n, int d) {
/* Line 389, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_87(PCB_DECL) {
/* Line 392, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return 0;
}

static inline long ag_rp_88(PCB_DECL, long n, int d) {
/* Line 393, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return 16*n + d;
}

static inline int ag_rp_89(PCB_DECL, int x) {
/* Line 396, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return x - '0';
}

static inline int ag_rp_90(PCB_DECL, int d) {
/* Line 397, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return (d & 7) + 9;
}

static inline Value ag_rp_91(PCB_DECL, Value &s, Value &e) {
/* Line 403, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return s+=e;
}

static inline Value ag_rp_92(PCB_DECL, AgString &b) {
/* Line 406, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return Value(b);
}

static inline AgString ag_rp_93(PCB_DECL) {
/* Line 410, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return AgString();
}

static inline AgString ag_rp_94(PCB_DECL, AgString &s, int c) {
/* Line 411, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return s.append(c);
}

static inline int ag_rp_95(PCB_DECL) {
/* Line 423, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return '\'';
}

static inline int ag_rp_96(PCB_DECL) {
/* Line 424, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return '"';
}

static inline int ag_rp_97(PCB_DECL) {
/* Line 425, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return '\?';
}

static inline int ag_rp_98(PCB_DECL) {
/* Line 426, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return '\\';
}

static inline int ag_rp_99(PCB_DECL) {
/* Line 427, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return '\a';
}

static inline int ag_rp_100(PCB_DECL) {
/* Line 428, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return '\b';
}

static inline int ag_rp_101(PCB_DECL) {
/* Line 429, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return '\f';
}

static inline int ag_rp_102(PCB_DECL) {
/* Line 430, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return '\n';
}

static inline int ag_rp_103(PCB_DECL) {
/* Line 431, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return '\r';
}

static inline int ag_rp_104(PCB_DECL) {
/* Line 432, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return '\t';
}

static inline int ag_rp_105(PCB_DECL) {
/* Line 433, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return '\v';
}

static inline int ag_rp_106(PCB_DECL, int d) {
/* Line 439, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return d-'0';
}

static inline int ag_rp_107(PCB_DECL, int n, int d) {
/* Line 442, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_108(PCB_DECL, int n, int d) {
/* Line 445, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_109(PCB_DECL, int d) {
/* Line 448, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return d;
}

static inline int ag_rp_110(PCB_DECL, int n, int d) {
/* Line 449, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return 16*n + d;
}

static inline int ag_rp_111(PCB_DECL, int d) {
/* Line 452, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return d-'0';
}

static inline int ag_rp_112(PCB_DECL, int d) {
/* Line 453, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_113(PCB_DECL, int c) {
/* Line 462, W:/agdev/agex/packages/xidek/pll/base/ast.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 4, 5, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 4, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 6, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<AgStack<AstNode *> >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static parse_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  2,  3,  0,  0,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
   16,  0, 17, 18, 19,  0,  0,  0,  0, 20, 21, 22, 23, 24, 25, 26,  0,  0,
   27,  0, 28,  0, 29,  0, 30, 31,  0, 32, 33,  0, 34,  0, 35, 36, 37, 38,
   39,  0, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55,
   56, 57, 58, 59, 60, 61, 62, 63, 64,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,  0, 65, 66,  0, 67, 68, 69, 70, 71, 72, 73, 74,  0,  0, 75, 76, 77,
   78,  0,  0,  0, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90,  0, 91,
   92, 93, 94,  0,  0,  0,  0,  0, 95, 96, 97, 98, 99,100,101,102,103,104,
  105,  0,  0,  0,106,107,108,109,110,111,112,113
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,150,  0,151,  0,152,  0,154,  0,156,  0,157,  0,158,  0,159,
  0,160,  0,162,  0,161,  0,164,  0,163,  0,166,  0,168,  0,173,
  0,174,  0,185,  0,186,  0,189,  0,190,  0,191,  0,192,  0,193,
  0,194,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 80, 84,255, 69,255, 47, 66, 68, 70, 73, 80, 82, 87,255,
   42,255, 42, 47,255, 61, 62,255, 69, 89,255, 85,255, 73, 79, 85,255, 76,
   78,255, 80, 84,255, 69,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 48,
   58, 60, 62, 65, 66, 68, 69, 70, 73, 76, 77, 78, 79, 80, 82, 83, 84, 85,
   87, 88,255, 80, 84,255, 69,255, 66, 68, 70, 73, 80, 82, 87,255, 42, 47,
  255, 47, 48, 78,255, 42, 47,255, 47,255, 42, 47,255, 80, 84,255, 69,255,
   47, 66, 68, 69, 70, 73, 80, 82, 87,255, 42, 47,255, 61, 62,255, 42, 47,
   58, 60, 62,255, 42, 47,255, 80, 84,255, 69,255, 47, 66, 68, 70, 73, 80,
   82, 85, 87,255, 48, 78,255, 80, 84,255, 69,255, 66, 68, 69, 70, 73, 80,
   82, 87,255, 42, 47,255, 76, 78,255, 80, 84,255, 69,255, 47, 66, 68, 69,
   70, 73, 80, 82, 85, 87,255, 58,255, 80, 84,255, 69,255, 66, 68, 70, 73,
   80, 82, 85, 87,255, 34, 39, 63, 65, 66, 70, 78, 82, 84, 86, 88, 92,255,
   92,255, 42, 47,255, 61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,
  255, 42, 47, 60, 62, 65, 66, 68, 77, 79, 83, 84, 88,255, 42, 47,255, 61,
   62,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 60, 62, 77, 79, 83, 84,
   88,255, 61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 60,
   62, 65, 66, 68, 77, 79, 83, 84, 88,255, 42, 47,255, 47, 48, 68, 76, 78,
  255, 48, 68, 76, 78,255, 61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72,
   79,255, 60, 62, 65, 66, 68, 77, 79, 83, 84, 88,255, 61, 62,255, 72, 79,
  255, 60, 62, 66, 68, 79, 84, 88,255, 84,255, 68,255, 76, 78,255, 80, 84,
  255, 69,255, 66, 68, 69, 70, 73, 80, 82, 85, 87,255, 42, 47,255, 61, 62,
  255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 48, 60, 62, 65,
   66, 68, 77, 79, 83, 84, 88,255, 48,255, 72, 79,255, 66, 68, 79, 84, 88,
  255, 66, 84,255, 84,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,7,7,4,2,4,2,7,7,7,7,7,2,7,4,3,4,0,0,4,0,0,4,7,5,4,7,4,7,6,7,4,
  7,7,4,7,7,4,2,4,5,5,4,2,4,7,5,4,3,2,3,3,2,3,7,2,2,2,7,7,7,7,7,7,7,2,2,
  2,7,7,7,4,7,7,4,2,4,7,7,7,7,7,2,7,4,0,0,4,2,3,7,4,0,0,4,2,4,0,0,4,7,7,
  4,2,4,2,7,7,7,7,7,7,2,7,4,0,0,4,0,0,4,3,2,3,2,3,4,0,0,4,7,7,4,2,4,2,7,
  7,7,7,7,2,7,7,4,3,7,4,7,7,4,2,4,7,7,7,7,7,7,2,7,4,0,0,4,7,7,4,7,7,4,2,
  4,2,7,7,2,7,7,7,2,7,7,4,3,4,7,7,4,2,4,7,7,7,7,7,2,7,7,4,0,0,0,0,0,0,0,
  0,0,0,0,0,4,2,4,0,0,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,2,3,7,7,2,7,7,2,
  2,7,4,0,0,4,0,0,4,5,5,4,2,4,7,5,4,3,2,2,3,7,7,2,2,7,4,0,0,4,7,5,4,5,5,
  4,2,4,7,5,4,3,2,3,7,7,2,7,7,2,2,7,4,0,0,4,2,3,7,7,7,4,3,7,7,7,4,0,0,4,
  7,5,4,5,5,4,2,4,7,5,4,2,3,7,7,2,7,7,2,2,7,4,0,0,4,7,5,4,2,3,7,7,7,2,7,
  4,7,4,7,4,7,7,4,7,7,4,2,4,7,7,2,7,7,7,2,7,7,4,0,0,4,0,0,4,7,5,4,5,5,4,
  2,4,7,5,4,3,2,3,2,3,7,7,2,7,7,2,2,7,4,3,4,7,5,4,7,7,7,2,7,4,7,7,4,7,4
};

static const unsigned char ag_key_parm[] = {
    0, 82, 87,  0, 10,  8,  0,  0,  0,  0, 22, 26, 16, 18, 28,  0, 14,  0,
   86,  0, 82, 87,  0,181,179,  0, 22,  6,  0, 32,  0, 38,  2, 26,  0,  0,
   24,  0, 10,  8,  0,  0,  0, 44, 46,  0,  0,  0, 20,  4,  0,170,  0,112,
  153,  0,183, 42,  0,  0,  0, 16, 18, 30, 40, 48, 34, 28,  0,  0,  0, 12,
   14, 36,  0, 10,  8,  0,  0,  0, 22, 26, 16, 18, 28,  0, 14,  0, 82, 87,
    0,  0,112, 48,  0, 82, 87,  0,  0,  0, 82, 87,  0, 10,  8,  0,  0,  0,
    0, 22, 26, 24, 16, 18, 28,  0, 14,  0, 82, 87,  0,181,179,  0,170,  0,
  153,  0,183,  0, 82, 87,  0, 10,  8,  0,  0,  0,  0, 22, 26, 16, 18, 28,
    0, 12, 14,  0,112, 48,  0, 10,  8,  0,  0,  0, 22, 26, 24, 16, 18, 28,
    0, 14,  0, 82, 87,  0,  0, 24,  0, 10,  8,  0,  0,  0,  0, 22, 26,  0,
   16, 18, 28,  0, 12, 14,  0,153,  0, 10,  8,  0,  0,  0, 22, 26, 16, 18,
   28,  0, 12, 14,  0,124,123,125,127,128,129,130,131,132,133,138,126,  0,
    0,  0, 82, 87,  0,181,179,  0, 38,  2,  0, 44, 46,  0,  0,  0, 20,  4,
    0,170,  0,  0,183, 42,  6,  0, 40, 34,  0,  0, 36,  0, 82, 87,  0,181,
  179,  0, 44, 46,  0,  0,  0, 20,  4,  0,170,  0,  0,183, 40, 34,  0,  0,
   36,  0,181,179,  0, 38,  2,  0, 44, 46,  0,  0,  0, 20,  4,  0,170,  0,
  183, 42,  6,  0, 40, 34,  0,  0, 36,  0, 82, 87,  0,  0,112, 32, 30, 48,
    0,112, 32, 30, 48,  0,181,179,  0, 38,  2,  0, 44, 46,  0,  0,  0, 20,
    4,  0,  0,183, 42,  6,  0, 40, 34,  0,  0, 36,  0,181,179,  0, 20,  4,
    0,  0,183,  6,  2, 34,  0, 36,  0, 20,  0,  2,  0,  0, 24,  0, 10,  8,
    0,  0,  0, 22, 26,  0, 16, 18, 28,  0, 12, 14,  0, 82, 87,  0,181,179,
    0, 38,  2,  0, 44, 46,  0,  0,  0, 20,  4,  0,170,  0,112,  0,183, 42,
    6,  0, 40, 34,  0,  0, 36,  0,112,  0, 20,  4,  0,  6,  2, 34,  0, 36,
    0,  6,  4,  0,  4,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0, 19, 23,  0,  4,  0,  1,  0,  5,  9, 12, 14,  7, 27,  0,
   32,  0,  0,  0,  0,  0,  0,  0, 45,  0,  0, 51,  0, 49, 29, 55,  0, 58,
   61,  0, 85, 89,  0, 38,  0,  0,  0,  0, 43,  0, 93,  0,  0, 34, 20, 36,
   38, 23, 40, 42, 26, 31, 35, 63, 66, 68, 72, 75, 78, 80, 41, 46, 48, 96,
  101,106,  0,128,132,  0, 75,  0,109,114,118,121,123, 78,136,  0,  0,  0,
    0, 88,141,143,  0,  0,  0,  0, 95,  0,  0,  0,  0,168,172,  0,103,  0,
  100,146,151,155,158,161,163,106,176,  0,  0,  0,  0,  0,  0,  0,181,118,
  183,121,185,  0,  0,  0,  0,206,210,  0,133,  0,130,187,192,196,199,201,
  136,214,219,  0,224,226,  0,251,255,  0,151,  0,229,234,238,241,244,246,
  154,259,  0,  0,  0,  0,273,276,  0,288,292,  0,171,  0,165,264,269,168,
  278,281,283,174,296,301,  0,306,  0,327,331,  0,189,  0,308,313,317,320,
  322,192,335,340,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
  203,  0,  0,  0,  0,  0,  0,  0,354,  0,  0,  0,  0,  0,227,  0,361,  0,
    0,345,218,221,347,349,352,224,356,359,230,232,364,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,254,  0,376,  0,  0,367,248,251,369,371,374,257,259,
  379,  0,  0,  0,  0,391,  0,  0,  0,  0,  0,278,  0,398,  0,  0,382,272,
  384,386,389,275,393,396,281,283,401,  0,  0,  0,  0,298,404,406,412,416,
    0,419,421,427,431,  0,  0,  0,  0,441,  0,  0,  0,  0,  0,318,  0,448,
    0,  0,312,434,436,439,315,443,446,321,323,451,  0,  0,  0,  0,462,  0,
    0,337,454,456,458,460,340,465,  0,468,  0,472,  0,483,486,  0,498,502,
    0,358,  0,474,479,355,488,491,493,361,506,511,  0,  0,  0,  0,  0,  0,
    0,527,  0,  0,  0,  0,  0,382,  0,534,  0,  0,516,373,518,376,520,522,
  525,379,529,532,385,387,537,  0,540,  0,548,  0,  0,542,544,546,406,551,
    0,554,556,  0,558,  0
};

static const unsigned short ag_key_index[] = {
    9,  0, 18, 51, 80,  0,  0, 18, 18, 80, 91, 98,108,124,138,148,  0, 91,
  156,176, 98, 91, 91,  0,  0,148,187,194,148,  0,148, 80,216,  0,  0,216,
  235,  0,235,262,235,235,235,235,235,286,286,301, 91, 91,307, 91,286,148,
  148,326,343,156, 91,148,  0,  0, 91,148,194,351,187,353,363,216,235,  0,
    0,216,216,216,  0,235,  0,  0,148, 98,  0, 98,  0,  0, 91,148, 91, 91,
   91, 91, 91, 91, 91,148, 91, 91,148, 91, 91, 91, 91, 91, 91,148,176,  0,
   91,148,  9,148,  9, 80,  9, 80,  0,  0,  0,  0,390,404,404,326,409,  0,
  415,235,235,148, 91,148,418, 91,148,353, 80
};

static const unsigned char ag_key_ends[] = {
69,71,73,78,0, 85,77,80,0, 79,82,0, 70,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 72,73,76,69,0, 47,0, 42,0, 88,0, 61,0, 
61,0, 78,68,0, 71,73,78,0, 86,0, 66,76,69,0, 77,80,0, 83,69,0, 
68,0, 79,82,0, 70,0, 79,78,71,0, 79,68,0, 79,84,0, 82,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 69,78,0, 78,84,73,76,0, 
72,73,76,69,0, 79,82,0, 69,71,73,78,0, 85,77,80,0, 79,82,0, 
70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 72,73,76,69,0, 
88,0, 79,84,0, 69,71,73,78,0, 85,77,80,0, 78,68,0, 79,82,0, 
70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 72,73,76,69,0, 
42,0, 61,0, 61,0, 69,71,73,78,0, 85,77,80,0, 79,82,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 78,84,73,76,0, 
72,73,76,69,0, 88,0, 79,84,0, 69,71,73,78,0, 85,77,80,0, 
78,68,0, 79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
72,73,76,69,0, 69,71,73,78,0, 85,77,80,0, 83,69,0, 68,0, 
79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
78,84,73,76,0, 72,73,76,69,0, 61,0, 69,71,73,78,0, 85,77,80,0, 
79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
78,84,73,76,0, 72,73,76,69,0, 42,0, 61,0, 78,68,0, 89,0, 86,0, 
79,68,0, 82,0, 69,78,0, 79,82,0, 42,0, 61,0, 79,68,0, 82,0, 
69,78,0, 79,82,0, 42,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 
82,0, 69,78,0, 79,82,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 
79,84,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 79,84,0, 61,0, 
78,68,0, 89,0, 86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 61,0, 
89,0, 79,0, 82,0, 69,78,0, 79,82,0, 72,69,78,0, 79,0, 
69,71,73,78,0, 85,77,80,0, 83,69,0, 68,0, 79,82,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 78,84,73,76,0, 
72,73,76,69,0, 42,0, 88,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 
82,0, 69,78,0, 79,82,0, 88,0, 89,0, 79,0, 82,0, 69,78,0, 
79,82,0, 89,0, 79,0, 79,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
    9,144,144,144,144,144,144,144,144,143, 91,143,143,143,144,144,144,144,
  144,144,144,144,144,144,144,144,144,144,144,144,144,144,143,144,115,144,
  144,144,144,140,172,171,187,169,167,184, 98,188,110,145,145,145,145,145,
  145,145,109,109,144,155,180,178,182,144,144,146,146,146,146, 95,146,147,
  147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
  147,144,137,144,144,147,144,146,146,146,146, 95,146,147,147,147,147,147,
  147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,144,144,144,
  144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
  144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
  144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
  144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
  144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
  144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
  144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
  144,144,144,144
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (parse_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
168,166,164,162,160,159,157,156,155,147,146,143,95,91,87,82,9,0,1,148,149,
188,187,184,182,180,178,172,171,169,167,155,147,146,145,144,143,140,137,115,
  110,109,98,95,91,0,89,90,
188,187,184,182,180,178,172,171,169,167,155,147,146,145,144,143,140,137,115,
  110,109,98,95,91,86,0,84,85,
143,91,87,82,0,1,
168,166,164,162,160,159,157,156,155,147,146,95,9,0,7,8,
188,187,184,182,180,178,172,171,169,167,155,147,146,145,144,143,140,137,115,
  110,109,98,95,0,
91,0,
188,187,184,182,180,178,172,171,169,167,155,147,146,145,144,143,140,137,115,
  110,109,98,95,91,0,
86,0,
168,166,164,162,160,159,157,156,155,147,146,95,9,0,4,10,11,12,13,15,18,19,
  23,25,27,28,29,31,32,33,35,36,37,39,41,43,165,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
147,146,143,95,91,87,82,0,1,148,149,
168,166,164,163,162,160,159,157,156,155,147,146,143,95,91,87,82,0,1,148,149,
188,187,184,183,182,181,180,179,178,172,171,170,169,167,155,153,147,146,145,
  143,110,109,95,91,87,82,0,1,148,149,
168,166,164,162,160,159,158,157,156,155,147,146,143,95,91,87,82,0,1,148,149,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,16,19,44,
  46,48,50,51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,
  177,195,
147,146,95,0,4,165,
194,184,172,169,155,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,
  148,149,
168,166,164,163,162,160,159,157,156,155,147,146,95,0,8,
168,166,164,163,162,160,159,158,157,156,155,150,147,146,143,95,91,87,82,9,0,
  1,148,149,
147,146,143,95,91,87,82,0,1,148,149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
167,155,0,27,42,
167,155,0,27,42,
194,184,172,169,155,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,16,19,
  30,44,46,48,50,51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,
  176,177,195,
153,0,20,
168,166,164,162,160,159,158,157,156,155,147,146,95,0,8,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,16,19,44,
  46,48,50,51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,
  177,195,
147,146,95,0,4,19,165,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,16,19,44,
  46,48,50,51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,
  177,195,
168,166,164,162,160,159,157,156,155,147,146,95,0,4,10,11,12,13,15,18,19,23,
  25,27,28,29,31,32,33,35,36,37,39,41,43,165,
188,187,184,182,180,178,172,171,169,167,155,147,146,145,144,143,140,138,137,
  133,132,131,130,129,128,127,126,125,124,123,115,110,109,98,95,0,116,
145,110,109,0,105,
145,110,109,0,99,105,
188,187,184,182,180,178,172,171,169,167,155,147,146,145,144,143,138,137,133,
  132,131,130,129,128,127,126,125,124,123,115,110,109,98,95,0,119,120,121,
  122,134,135,136,141,
193,192,191,190,189,188,187,186,185,184,183,182,181,180,179,178,171,170,169,
  167,161,155,154,152,151,143,115,91,87,82,0,1,148,149,
98,95,0,
95,0,
146,145,110,109,95,0,113,
193,192,191,190,189,188,187,186,185,184,183,182,181,180,179,178,171,170,169,
  167,161,155,154,152,151,145,143,110,109,98,95,91,87,82,0,
193,192,191,190,189,188,187,186,185,184,183,182,181,180,179,178,171,170,169,
  167,161,155,154,152,151,145,143,110,109,98,95,91,87,82,0,105,
193,192,191,190,189,188,187,186,185,184,183,182,181,180,179,178,171,170,169,
  167,161,155,154,152,151,143,91,87,82,0,1,148,149,
193,192,191,190,189,188,187,186,185,184,183,182,181,180,179,178,171,170,169,
  167,161,155,154,152,151,143,91,87,82,0,1,148,149,
193,192,191,190,189,188,187,186,185,184,183,182,181,180,179,178,171,170,169,
  167,161,155,154,152,151,143,91,87,82,0,1,148,149,
172,0,55,
115,0,5,195,
194,184,174,173,172,171,169,147,146,145,143,140,115,112,110,109,98,95,91,87,
  82,0,1,148,149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,174,173,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,
  16,19,44,46,48,50,51,52,53,55,57,58,59,60,63,70,80,94,97,100,101,102,
  106,165,175,176,177,195,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
170,0,54,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,19,48,50,
  51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,177,195,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,19,48,50,
  51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,177,195,
193,192,191,190,189,188,187,0,49,73,74,75,76,77,78,79,
186,185,184,183,182,181,180,179,178,169,0,45,47,51,64,65,66,67,68,69,70,71,
  72,
168,166,164,163,162,160,159,157,156,155,147,146,95,0,4,10,11,12,13,15,18,19,
  23,25,27,28,29,31,32,33,35,36,37,39,40,41,43,165,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,16,19,44,
  46,48,50,51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,
  177,195,
147,146,95,0,4,165,
155,0,27,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,16,19,44,
  46,48,50,51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,
  177,195,
168,166,164,162,160,159,158,157,156,155,147,146,95,0,4,10,11,12,13,15,18,19,
  23,25,26,27,28,29,31,32,33,34,35,36,37,39,41,43,165,
161,0,38,
153,0,20,
151,0,17,
150,0,14,
188,187,184,182,180,178,172,171,169,167,155,147,146,145,144,143,140,138,137,
  133,132,131,130,129,128,127,126,125,124,123,115,110,109,98,95,0,117,119,
  120,121,122,134,135,136,
145,110,109,0,99,105,
145,110,0,
146,145,110,109,95,0,139,
145,110,0,
145,110,0,
146,145,110,109,95,0,139,
140,0,
145,110,109,0,99,105,
184,169,145,110,109,0,96,103,
184,169,145,110,109,0,96,103,
194,184,172,171,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,16,19,
  44,46,48,50,51,52,53,55,57,58,61,62,63,70,80,94,97,100,101,102,106,165,
  175,176,177,195,
171,143,91,87,82,0,1,148,149,
171,0,56,
171,143,91,87,82,0,1,148,149,
171,0,56,
171,0,56,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,19,48,50,
  51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,177,195,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,19,48,50,
  51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,177,195,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,19,46,48,
  50,51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,177,195,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,19,44,46,
  48,50,51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,177,
  195,
168,166,164,163,162,160,159,158,157,156,155,150,147,146,143,95,91,87,82,9,0,
  1,148,149,
155,0,27,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,16,19,44,
  46,48,50,51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,
  177,195,
168,166,164,162,160,159,157,156,155,147,146,143,95,91,87,82,0,1,148,149,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,16,19,44,
  46,48,50,51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,
  177,195,
168,166,164,162,160,159,157,156,155,147,146,143,95,91,87,82,0,1,148,149,
168,166,164,162,160,159,157,156,155,147,146,95,0,4,11,12,13,15,18,19,23,25,
  27,28,29,31,32,33,35,36,37,39,41,43,165,
168,166,164,162,160,159,157,156,155,147,146,143,95,91,87,82,0,1,148,149,
168,166,164,162,160,159,157,156,155,147,146,95,0,4,11,12,13,15,18,19,23,25,
  27,28,29,31,32,33,35,36,37,39,41,43,165,
145,110,109,0,104,
145,110,109,0,104,
167,0,42,
171,0,56,
193,192,191,190,189,188,187,186,185,184,183,182,181,180,179,178,172,171,170,
  169,167,161,155,154,152,151,147,146,145,143,140,115,112,110,109,98,95,
  91,87,82,0,1,148,149,
172,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,19,53,55,57,58,63,94,
  97,100,101,102,106,165,175,176,177,195,
172,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,19,53,55,57,58,63,94,
  97,100,101,102,106,165,175,176,177,195,
193,192,191,190,189,188,187,0,49,73,74,75,76,77,78,79,
186,185,184,169,0,47,51,70,71,72,
155,0,27,
154,152,0,21,24,
145,110,109,0,105,
145,110,109,0,105,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,16,19,44,
  46,48,50,51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,
  177,195,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,16,19,44,
  46,48,50,51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,
  177,195,
152,0,22,
194,184,172,169,147,146,145,143,140,115,112,110,109,98,95,91,87,82,0,1,148,
  149,
194,184,172,169,147,146,145,140,115,112,110,109,98,95,0,2,3,4,5,6,16,19,44,
  46,48,50,51,52,53,55,57,58,63,70,80,94,97,100,101,102,106,165,175,176,
  177,195,
151,0,17,
168,166,164,162,160,159,157,156,155,147,146,95,0,4,11,12,13,15,18,19,23,25,
  27,28,29,31,32,33,35,36,37,39,41,43,165,

};


static unsigned const char ag_astt[2856] = {
  8,8,8,8,8,8,8,8,8,8,8,1,8,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,7,
  1,1,9,9,1,1,5,3,4,4,4,4,4,4,4,4,4,4,4,4,4,7,0,1,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,9,9,9,9,9,9,5,3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  5,3,7,1,1,1,1,1,1,1,1,1,2,2,2,2,7,2,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,
  5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,
  10,10,1,10,10,10,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,
  1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,2,2,2,7,2,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,
  1,3,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,7,2,1,1,1,7,2,1,1,1,1,1,4,2,2,
  2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,7,1,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,
  2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,
  1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,1,2,1,2,2,1,1,
  1,1,1,1,1,1,1,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,7,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,1,5,2,2,2,2,2,5,2,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,2,4,4,5,5,5,7,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,10,4,4,5,5,5,7,2,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,1,4,1,1,4,2,1,5,5,5,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,5,1,1,1,1,1,2,2,2,1,1,2,2,
  2,1,2,7,2,2,1,1,2,2,2,2,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,2,2,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,5,1,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,5,1,1,2,2,2,2,2,2,2,
  2,2,2,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,
  2,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,
  2,1,2,7,2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,
  2,7,2,1,1,7,2,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,
  1,2,2,2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,2,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,7,2,1,7,1,1,7,1,1,5,1,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
  10,10,1,1,2,2,2,2,2,2,2,2,2,2,2,2,10,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,2,
  1,2,2,7,2,2,2,2,2,7,2,2,2,5,2,2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,8,
  7,2,1,1,1,8,8,8,7,2,1,1,1,1,4,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,1,1,1,1,7,1,1,3,1,7,1,5,1,
  1,1,1,7,1,1,3,1,7,1,1,7,2,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,
  1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,2,2,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,2,2,1,1,1,1,
  1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,
  7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,
  5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,
  1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,
  5,1,1,1,5,7,1,1,3,1,7,2,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,
  2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,
  5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,
  1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,
  1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,
  7,1,2,2,2,7,1,1,5,1,1,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,
  2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,2,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,2,2,2,2,2,2,2,1,1,1,1,4,1,2,
  2,2,2,1,7,2,1,4,7,1,1,10,10,10,4,2,10,10,10,4,2,1,1,1,1,2,2,2,1,1,2,2,2,1,
  2,7,2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,
  2,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,7,1,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,
  1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,7,1,1,1,1,1,1,1,1,1,1,2,
  2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
4,4,4,4,4,4,4,4,4,4,4,3,4,3,1,2,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
269,269,1,2,271,269,
2,2,2,2,2,2,2,2,2,2,2,2,2,4,0,9,
88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,90,
91,6,
83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,83,85,
86,8,
10,11,12,22,20,21,14,17,19,92,92,92,1,9,52,3,3,3,31,30,29,26,13,27,18,13,25,
  24,23,27,30,29,28,18,16,15,13,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,10,3,3,290,
270,270,3,270,3,1,2,11,3,3,288,
270,270,270,270,270,270,270,270,270,270,270,270,3,270,3,1,2,12,3,3,286,
270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,93,93,93,3,
  93,93,93,3,1,2,13,3,3,287,
270,270,270,270,270,270,270,270,270,270,270,270,3,270,3,1,2,14,3,3,279,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,15,58,59,45,46,61,31,48,56,55,
  55,55,54,53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
92,92,92,16,29,13,
270,270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,17,3,3,
  278,
2,2,2,2,2,2,2,2,2,2,2,2,2,18,57,
270,270,270,270,270,270,270,270,270,270,270,270,270,270,3,270,3,1,2,270,19,
  3,3,277,
270,270,3,270,3,1,2,20,3,3,282,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,21,3,3,281,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,22,3,3,284,
58,19,23,22,59,
58,19,24,21,60,
48,49,47,51,33,92,92,112,35,32,120,118,112,34,92,25,58,59,45,46,61,61,48,61,
  56,55,55,55,54,53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
62,26,63,
2,2,2,2,2,2,2,2,2,2,2,2,2,27,64,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,28,58,59,45,46,61,65,48,56,55,
  55,55,54,53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
92,92,92,29,52,66,13,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,30,58,59,45,46,61,67,48,56,55,
  55,55,54,53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
10,11,12,22,20,21,14,17,19,92,92,92,31,52,6,6,68,31,30,29,26,13,27,18,13,25,
  24,23,27,30,29,28,18,16,15,13,
127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,
  127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,127,32,69,
117,117,117,101,117,
70,70,70,34,99,70,
76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,72,71,144,143,142,141,140,
  139,138,137,136,135,134,76,76,76,76,76,35,76,76,76,75,74,73,76,76,
270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,
  270,270,270,270,270,270,3,270,3,1,2,36,3,3,317,
77,78,37,
79,94,
123,122,122,122,123,111,121,
110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,110,
  110,110,110,110,110,110,119,110,119,116,102,102,110,110,110,40,
109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,109,
  109,109,109,109,109,109,113,109,113,113,100,100,109,109,109,41,113,
270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,
  270,270,270,270,270,270,3,3,1,2,42,3,3,299,
270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,
  270,270,270,270,270,270,3,3,1,2,43,3,3,298,
270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,
  270,270,270,270,270,270,3,3,1,2,44,3,3,297,
47,52,80,
32,60,125,36,
270,270,270,270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,
  47,3,3,294,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,48,3,3,316,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,49,3,3,306,
48,49,81,83,47,51,92,92,112,35,32,120,118,112,34,92,50,58,59,45,46,61,85,48,
  56,55,55,55,54,53,52,50,52,52,84,82,46,79,80,38,37,41,33,40,39,13,44,43,
  42,36,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,51,3,3,291,
86,44,87,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,53,58,59,45,46,61,48,43,43,54,
  53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,54,58,59,45,46,61,48,42,42,54,
  53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
88,89,90,91,92,93,94,37,95,72,73,74,75,76,77,78,
96,97,49,99,100,101,102,103,104,51,35,105,98,68,62,63,64,65,66,67,69,70,71,
10,11,12,106,22,20,21,14,17,19,92,92,92,57,52,3,3,3,31,30,29,26,13,27,18,13,
  25,24,23,27,30,29,28,18,28,16,15,13,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,58,3,3,289,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,59,58,59,45,46,61,32,48,56,55,
  55,55,54,53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
92,92,92,60,30,13,
19,61,20,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,62,3,3,275,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,63,58,59,45,46,61,107,48,56,
  55,55,55,54,53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
10,11,12,22,20,21,108,14,17,19,92,92,92,64,52,3,3,3,31,30,29,26,13,27,109,
  18,13,25,24,23,27,109,30,29,28,18,16,15,13,
110,65,27,
62,66,111,
112,67,113,
114,5,115,
128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,72,71,
  144,143,142,141,140,139,138,137,136,135,134,126,128,128,128,128,69,128,
  128,128,128,75,74,73,128,
70,70,70,107,108,70,
148,148,71,
154,153,153,153,154,72,151,
150,150,146,
149,149,145,
154,153,153,153,154,133,152,
155,76,
70,70,70,98,97,70,
116,117,117,117,117,78,96,117,
116,117,117,117,117,79,95,117,
48,49,47,54,51,92,92,112,35,32,120,118,112,34,92,80,58,59,45,46,61,56,48,56,
  55,55,55,54,53,52,50,52,52,119,118,46,79,80,38,37,41,33,40,39,13,44,43,
  42,36,
270,3,3,1,2,81,3,3,296,
120,82,121,
270,3,3,1,2,83,3,3,295,
120,84,122,
120,85,46,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,86,3,3,292,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,87,58,59,45,46,61,48,45,45,54,
  53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,88,3,3,315,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,89,3,3,314,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,90,3,3,313,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,91,3,3,312,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,92,3,3,311,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,93,3,3,310,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,94,3,3,309,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,95,58,59,45,46,61,48,40,40,54,
  53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,96,3,3,308,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,97,3,3,307,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,98,58,59,45,46,61,48,123,123,
  123,54,53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,99,3,3,305,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,100,3,3,304,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,101,3,3,303,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,102,3,3,302,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,103,3,3,301,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,104,3,3,300,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,105,58,59,45,46,61,48,124,55,
  55,55,54,53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
270,270,270,270,270,270,270,270,270,270,270,270,270,270,3,270,3,1,2,270,106,
  3,3,285,
19,107,17,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,108,3,3,280,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,109,58,59,45,46,61,125,48,56,
  55,55,55,54,53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
270,270,270,270,270,270,270,270,270,270,270,3,270,3,1,2,110,3,3,283,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,111,58,59,45,46,61,126,48,56,
  55,55,55,54,53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
270,270,270,270,270,270,270,270,270,270,270,3,270,3,1,2,112,3,3,273,
10,11,12,22,20,21,14,17,19,92,92,92,113,52,8,11,31,30,29,26,13,27,18,13,25,
  24,23,27,30,29,28,18,16,15,13,
270,270,270,270,270,270,270,270,270,270,270,3,270,3,1,2,114,3,3,272,
10,11,12,22,20,21,14,17,19,92,92,92,115,52,7,10,31,30,29,26,13,27,18,13,25,
  24,23,27,30,29,28,18,16,15,13,
114,114,114,116,127,
114,114,114,117,128,
58,55,129,
120,119,53,
270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,270,
  270,270,270,270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,
  1,2,120,3,3,293,
47,92,92,112,35,32,120,118,112,34,92,121,58,59,45,46,61,48,51,50,51,51,46,
  38,37,41,33,40,39,13,44,43,42,36,
47,92,92,112,35,32,120,118,112,34,92,122,58,59,45,46,61,48,50,50,50,50,46,
  38,37,41,33,40,39,13,44,43,42,36,
88,89,90,91,92,93,94,38,95,72,73,74,75,76,77,78,
96,97,49,51,36,98,68,69,70,71,
19,125,16,
130,14,126,132,131,
115,115,115,106,115,
115,115,115,105,115,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,129,58,59,45,46,61,57,48,56,
  55,55,55,54,53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,130,3,3,276,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,131,58,59,45,46,61,15,48,56,
  55,55,55,54,53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
133,132,134,
270,270,270,270,270,270,270,3,270,270,270,270,270,270,270,3,1,2,133,3,3,274,
48,49,47,51,92,92,112,35,32,120,118,112,34,92,134,58,59,45,46,61,135,48,56,
  55,55,55,54,53,52,50,52,52,46,79,80,38,37,41,33,40,39,13,44,43,42,36,
112,135,136,
10,11,12,22,20,21,14,17,19,92,92,92,136,52,9,12,31,30,29,26,13,27,18,13,25,
  24,23,27,30,29,28,18,16,15,13,

};


static const unsigned short ag_sbt[] = {
     0,  21,  48,  76,  82,  98, 122, 124, 149, 151, 188, 210, 221, 242,
   272, 293, 339, 345, 368, 383, 407, 418, 440, 462, 467, 472, 520, 523,
   538, 584, 591, 637, 673, 710, 715, 721, 764, 798, 801, 803, 810, 845,
   881, 914, 947, 980, 983, 987,1012,1034,1056,1106,1128,1131,1174,1217,
  1233,1256,1294,1316,1362,1368,1371,1393,1439,1478,1481,1484,1487,1490,
  1534,1540,1543,1550,1553,1556,1563,1565,1571,1579,1587,1636,1645,1648,
  1657,1660,1663,1685,1728,1750,1772,1794,1816,1838,1860,1882,1925,1947,
  1969,2013,2035,2057,2079,2101,2123,2145,2190,2214,2217,2239,2285,2305,
  2351,2371,2406,2426,2461,2466,2471,2474,2477,2521,2555,2589,2605,2615,
  2618,2623,2628,2633,2679,2701,2747,2750,2772,2818,2821,2856
};


static const unsigned short ag_sbe[] = {
    17,  45,  73,  80,  95, 121, 123, 148, 150, 164, 206, 217, 238, 268,
   289, 307, 342, 364, 381, 403, 414, 436, 458, 464, 469, 487, 521, 536,
   552, 587, 605, 649, 708, 713, 718, 755, 794, 800, 802, 808, 844, 879,
   910, 943, 976, 981, 984,1008,1030,1052,1072,1124,1129,1145,1188,1224,
  1243,1269,1312,1330,1365,1369,1389,1407,1452,1479,1482,1485,1488,1525,
  1537,1542,1548,1552,1555,1561,1564,1568,1576,1584,1602,1641,1646,1653,
  1658,1661,1681,1699,1746,1768,1790,1812,1834,1856,1878,1896,1943,1965,
  1983,2031,2053,2075,2097,2119,2141,2159,2210,2215,2235,2253,2301,2319,
  2367,2383,2422,2438,2464,2469,2472,2475,2517,2532,2566,2596,2609,2616,
  2620,2626,2631,2647,2697,2715,2748,2768,2786,2819,2833,2856
};


static const unsigned char ag_fl[] = {
  2,2,0,2,1,1,2,4,4,9,4,4,9,1,0,2,5,4,1,1,3,2,2,1,1,1,1,3,3,2,3,2,3,0,1,
  1,3,1,3,1,3,1,2,2,1,3,3,1,1,1,4,4,1,4,0,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,1,2,1,3,3,3,2,2,1,1,1,0,1,
  2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,2,1,1,1,2,3,0,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2
};

static const unsigned char ag_ptt[] = {
    0,  7,  8,  8, 10, 10, 11, 11, 11, 11, 12, 12, 12, 12, 21, 21, 23, 23,
   23, 23, 23, 23, 23, 25, 26, 15, 18, 13, 28, 31, 31, 32, 32, 30, 30, 16,
   16, 44, 44, 46, 46, 48, 48, 48, 50, 50, 53, 53, 53, 53, 53, 53, 19, 58,
   61, 61, 62, 62, 57, 57, 57, 57, 45, 45, 45, 45, 45, 45, 47, 47, 47, 47,
   49, 49, 49, 49, 49, 49, 49, 52, 52,  1, 84, 84, 85, 85,  1, 89, 89, 90,
   90,  1,165,165,176,176,176, 94, 94, 94, 97, 97, 97,103,103, 96, 96, 99,
   99,175,175,175,100,100,104,104,101,101,102,102,106,106,113,113, 63, 63,
  195,116,116,117,117,119,119,119,120,120,120,120,120,120,120,120,120,120,
  120,121,121,121,134,135,136,122,122,139,139,177,141,141,105, 81, 81, 83,
   83, 83, 83, 83, 83, 83, 83, 83, 83, 83, 83, 83, 83, 83, 83, 83, 83, 83,
   83, 83, 83, 83, 83, 88, 88, 88, 88, 88, 88, 88, 88, 88, 88, 88, 88, 88,
   88, 88, 88, 88, 88, 88, 88, 88, 88, 88, 92, 92, 92, 93, 93, 93, 93, 93,
   93,107,107,108,108,108,111,111,114,114,118,118,118,118,118,118,118,118,
  118,118,118,118,118,118,118,118,118,118,118,118,118,142,142,142,142,142,
  142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,148,148,
  149,149, 14, 17, 22, 20, 24, 27, 29, 33, 34, 35, 36, 38, 37, 40, 39,  4,
   41, 42, 43, 51, 54, 56, 55, 59, 60,  2,  3,  6, 64, 65, 66, 67, 68, 69,
   70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: ag_rp_1(PCB_POINTER, VW(0, AG_WRAP_6 *)); 
            VWD(0, AG_WRAP_6 *); break;
    case 2: VNO AG_WRAP_6(ag_rp_2(PCB_POINTER)); break;
    case 3: VRO(AG_WRAP_6 *, ag_rp_3(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(AstNode * *)))); break;
    case 4: V(0,(AstNode * *)) = ag_rp_4(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *))); break;
    case 5: V(0,(AstNode * *)) = ag_rp_5(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 6: V(0,(AstNode * *)) = ag_rp_6(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 7: V(0,(AstNode * *)) = ag_rp_7(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *)), V(4,(AstNode * *)), V(6,(AstNode * *)), V(8,(AstNode * *))); break;
    case 8: V(0,(AstNode * *)) = ag_rp_8(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 9: V(0,(AstNode * *)) = ag_rp_9(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 10: V(0,(AstNode * *)) = ag_rp_10(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *)), V(4,(AstNode * *)), V(6,(AstNode * *)), V(8,(AstNode * *))); break;
    case 11: V(0,(AstNode * *)) = ag_rp_11(PCB_POINTER, V(0,(AstNode * *))); break;
    case 12: V(0,(AstNode * *)) = ag_rp_12(PCB_POINTER); break;
    case 13: V(0,(AstNode * *)) = ag_rp_13(PCB_POINTER, V(1,(AstNode * *))); break;
    case 14: V(0,(AstNode * *)) = ag_rp_14(PCB_POINTER, VW(1, AG_WRAP_6 *), V(3,(AstNode * *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 15: V(0,(AstNode * *)) = ag_rp_15(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 16: V(0,(AstNode * *)) = ag_rp_16(PCB_POINTER); break;
    case 17: V(0,(AstNode * *)) = ag_rp_17(PCB_POINTER, V(1,(AstNode * *))); break;
    case 18: V(0,(AstNode * *)) = VDO(AG_WRAP_6 *, ag_rp_18(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 19: V(0,(AstNode * *)) = VDO(AG_WRAP_6 *, ag_rp_19(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 20: V(0,(AstNode * *)) = ag_rp_20(PCB_POINTER, V(1,(AstNode * *))); break;
    case 21: V(0,(AstNode * *)) = ag_rp_21(PCB_POINTER, VW(1, AG_WRAP_6 *)); 
            VWD(1, AG_WRAP_6 *); break;
    case 22: VNO AG_WRAP_6(ag_rp_22(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 23: VRO(AG_WRAP_6 *, ag_rp_23(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 24: VNO AG_WRAP_6(ag_rp_24(PCB_POINTER, V(1,(AstNode * *)))); break;
    case 25: VRO(AG_WRAP_6 *, ag_rp_25(PCB_POINTER, VW(0, AG_WRAP_6 *), V(2,(AstNode * *)))); break;
    case 26: V(0,(AstNode * *)) = ag_rp_26(PCB_POINTER); break;
    case 27: V(0,(AstNode * *)) = ag_rp_27(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 28: V(0,(AstNode * *)) = ag_rp_28(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 29: V(0,(AstNode * *)) = ag_rp_29(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 30: V(0,(AstNode * *)) = ag_rp_30(PCB_POINTER, V(1,(AstNode * *))); break;
    case 31: V(0,(AstNode * *)) = ag_rp_31(PCB_POINTER, V(0,(Opcode *)), V(1,(AstNode * *))); break;
    case 32: V(0,(AstNode * *)) = ag_rp_32(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 33: V(0,(AstNode * *)) = ag_rp_33(PCB_POINTER, V(1,(AstNode * *))); break;
    case 34: V(0,(AstNode * *)) = ag_rp_34(PCB_POINTER, V(0,(AstNode * *))); break;
    case 35: V(0,(AstNode * *)) = ag_rp_35(PCB_POINTER, V(3,(AstNode * *))); break;
    case 36: V(0,(AstNode * *)) = ag_rp_36(PCB_POINTER, V(3,(AstNode * *))); break;
    case 37: V(0,(AstNode * *)) = VDO(AG_WRAP_4 *, ag_rp_37(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 38: V(0,(AstNode * *)) = VDO(AG_WRAP_4 *, ag_rp_38(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 39: VNO AG_WRAP_6(ag_rp_39(PCB_POINTER)); break;
    case 40: VNO AG_WRAP_6(ag_rp_40(PCB_POINTER, V(0,(AstNode * *)))); break;
    case 41: VRO(AG_WRAP_6 *, ag_rp_41(PCB_POINTER, VW(0, AG_WRAP_6 *), V(2,(AstNode * *)))); break;
    case 42: V(0,(AstNode * *)) = ag_rp_42(PCB_POINTER, V(0,(long *))); break;
    case 43: V(0,(AstNode * *)) = ag_rp_43(PCB_POINTER, V(0,(double *))); break;
    case 44: V(0,(AstNode * *)) = VDO(AG_WRAP_5 *, ag_rp_44(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 45: V(0,(AstNode * *)) = ag_rp_45(PCB_POINTER, V(0,(int *))); break;
    case 46: V(0,(Opcode *)) = ag_rp_46(PCB_POINTER); break;
    case 47: V(0,(Opcode *)) = ag_rp_47(PCB_POINTER); break;
    case 48: V(0,(Opcode *)) = ag_rp_48(PCB_POINTER); break;
    case 49: V(0,(Opcode *)) = ag_rp_49(PCB_POINTER); break;
    case 50: V(0,(Opcode *)) = ag_rp_50(PCB_POINTER); break;
    case 51: V(0,(Opcode *)) = ag_rp_51(PCB_POINTER); break;
    case 52: V(0,(Opcode *)) = ag_rp_52(PCB_POINTER); break;
    case 53: V(0,(Opcode *)) = ag_rp_53(PCB_POINTER); break;
    case 54: V(0,(Opcode *)) = ag_rp_54(PCB_POINTER); break;
    case 55: V(0,(Opcode *)) = ag_rp_55(PCB_POINTER); break;
    case 56: V(0,(Opcode *)) = ag_rp_56(PCB_POINTER); break;
    case 57: V(0,(Opcode *)) = ag_rp_57(PCB_POINTER); break;
    case 58: V(0,(Opcode *)) = ag_rp_58(PCB_POINTER); break;
    case 59: V(0,(Opcode *)) = ag_rp_59(PCB_POINTER); break;
    case 60: V(0,(Opcode *)) = ag_rp_60(PCB_POINTER); break;
    case 61: V(0,(Opcode *)) = ag_rp_61(PCB_POINTER); break;
    case 62: V(0,(Opcode *)) = ag_rp_62(PCB_POINTER); break;
    case 63: V(0,(Opcode *)) = ag_rp_63(PCB_POINTER); break;
    case 64: V(0,(Opcode *)) = ag_rp_64(PCB_POINTER); break;
    case 65: VNO AG_WRAP_4(ag_rp_65(PCB_POINTER, V(0,(int *)))); break;
    case 66: VRO(AG_WRAP_4 *, ag_rp_66(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 67: V(0,(double *)) = ag_rp_67(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 68: V(0,(double *)) = ag_rp_68(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 69: V(0,(double *)) = ag_rp_69(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 70: V(0,(double *)) = ag_rp_70(PCB_POINTER, V(0,(double *))); break;
    case 71: V(0,(double *)) = ag_rp_71(PCB_POINTER, V(1,(double *))); break;
    case 72: V(0,(double *)) = ag_rp_72(PCB_POINTER, V(0,(long *))); break;
    case 73: V(0,(double *)) = ag_rp_73(PCB_POINTER, V(0,(long *))); break;
    case 74: V(0,(double *)) = ag_rp_74(PCB_POINTER, V(0,(long *))); break;
    case 75: V(0,(long *)) = ag_rp_75(PCB_POINTER, V(1,(long *))); break;
    case 76: V(0,(long *)) = ag_rp_76(PCB_POINTER, V(1,(long *))); break;
    case 77: V(0,(double *)) = ag_rp_77(PCB_POINTER, V(0,(int *))); break;
    case 78: V(0,(double *)) = ag_rp_78(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 79: V(0,(long *)) = ag_rp_79(PCB_POINTER, V(0,(int *))); break;
    case 80: V(0,(long *)) = ag_rp_80(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 81: V(0,(long *)) = ag_rp_81(PCB_POINTER, V(0,(int *))); break;
    case 82: V(0,(long *)) = ag_rp_82(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 83: V(0,(long *)) = ag_rp_83(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 84: V(0,(long *)) = ag_rp_84(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 85: V(0,(long *)) = ag_rp_85(PCB_POINTER); break;
    case 86: V(0,(long *)) = ag_rp_86(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 87: V(0,(long *)) = ag_rp_87(PCB_POINTER); break;
    case 88: V(0,(long *)) = ag_rp_88(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 89: V(0,(int *)) = ag_rp_89(PCB_POINTER, V(0,(int *))); break;
    case 90: V(0,(int *)) = ag_rp_90(PCB_POINTER, V(0,(int *))); break;
    case 91: VRO(AG_WRAP_5 *, ag_rp_91(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 92: VNO AG_WRAP_5(ag_rp_92(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 93: VNO AG_WRAP_4(ag_rp_93(PCB_POINTER)); break;
    case 94: VRO(AG_WRAP_4 *, ag_rp_94(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 95: V(0,(int *)) = ag_rp_95(PCB_POINTER); break;
    case 96: V(0,(int *)) = ag_rp_96(PCB_POINTER); break;
    case 97: V(0,(int *)) = ag_rp_97(PCB_POINTER); break;
    case 98: V(0,(int *)) = ag_rp_98(PCB_POINTER); break;
    case 99: V(0,(int *)) = ag_rp_99(PCB_POINTER); break;
    case 100: V(0,(int *)) = ag_rp_100(PCB_POINTER); break;
    case 101: V(0,(int *)) = ag_rp_101(PCB_POINTER); break;
    case 102: V(0,(int *)) = ag_rp_102(PCB_POINTER); break;
    case 103: V(0,(int *)) = ag_rp_103(PCB_POINTER); break;
    case 104: V(0,(int *)) = ag_rp_104(PCB_POINTER); break;
    case 105: V(0,(int *)) = ag_rp_105(PCB_POINTER); break;
    case 106: V(0,(int *)) = ag_rp_106(PCB_POINTER, V(1,(int *))); break;
    case 107: V(0,(int *)) = ag_rp_107(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 108: V(0,(int *)) = ag_rp_108(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 109: V(0,(int *)) = ag_rp_109(PCB_POINTER, V(1,(int *))); break;
    case 110: V(0,(int *)) = ag_rp_110(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 111: V(0,(int *)) = ag_rp_111(PCB_POINTER, V(0,(int *))); break;
    case 112: V(0,(int *)) = ag_rp_112(PCB_POINTER, V(0,(int *))); break;
    case 113: V(0,(int *)) = ag_rp_113(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES parse_token_names
const char *const parse_token_names[196] = {
  "script",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "script",
  "statement list",
  "eof",
  "statement",
  "open statement",
  "closed statement",
  "if condition",
  "\"ELSE\"",
  "WHILE",
  "expression",
  "\"DO\"",
  "FOR",
  "lvalue",
  "\":=\"",
  "for increment",
  "\"TO\"",
  "simple statement",
  "\"BY\"",
  "REPEAT",
  "UNTIL",
  "';'",
  "compound statement",
  "\"RETURN\"",
  "optional expression",
  "dump statement",
  "print statement",
  "\"REPEAT\"",
  "\"UNTIL\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"IF\"",
  "\"THEN\"",
  "\"BEGIN\"",
  "\"END\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "simple expression",
  "relational op",
  "term",
  "additive op",
  "unary expression",
  "multiplicative op",
  "factor",
  "'+'",
  "unary op",
  "primary",
  "\"**\"",
  "'('",
  "')'",
  "constant",
  "function call",
  "\"LONG\"",
  "\"DOUBLE\"",
  "optional arg list",
  "arg list",
  "string",
  "'='",
  "\"<>\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "'.'",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0X\"",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\A\"",
  "\"\\\\B\"",
  "\"\\\\F\"",
  "\"\\\\N\"",
  "\"\\\\R\"",
  "\"\\\\T\"",
  "\"\\\\V\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\X\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "\"ELSE\"",
  "\"DO\"",
  "\"TO\"",
  "\":=\"",
  "\"BY\"",
  "';'",
  "\"RETURN\"",
  "\"REPEAT\"",
  "\"UNTIL\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"THEN\"",
  "\"IF\"",
  "\"END\"",
  "\"BEGIN\"",
  "name",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "'+'",
  "\"**\"",
  "')'",
  "'('",
  "\"LONG\"",
  "\"DOUBLE\"",
  "integer",
  "real",
  "character constant",
  "'='",
  "\"<>\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (parse_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (parse_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_parse(parse_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void parse(parse_pcb_type *PCB_POINTER) {
  init_parse(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


