/*
 dci.syn -- Direct Compilation of Bytecode
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 The grammar contained in this file describes the same language as the
 grammar in pll.syn. The differences can be categorized as follows:
   . Rules rewritten to make computation easier. In pll.syn all of the
     arithmetic operators have been factored out. Here the arithmetic
     operators have been substituted back into the rules for expressions to
     make computation more straightforward.
   . Rules rewritten to handle looping. The rules for loop constructs have
     been rewritten to handle repetitive parsing of loops.
   . Semantically determined rules (true, false) added to support parse time
     implementation of loops and if-else statements.

 The actual parser modules, dci.h and dci.cpp are created from
 dci.syn by the AnaGram parser generator using the Build Parser command.

 To build a demonstration program for this parser, compile and link as a
 single project:
   demo.cpp                // Demo program
   comdefs.cpp             // Common script definitions
   bcidefs.cpp             // Bytecode interpreter definitions
   dci.cpp                 // Direct execution parser
   agclib1.lib             // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is somewhat similar to Pascal.

 The language consists of Pascal like statements and expressions. The
 language also implements Fortran-style exponentiation and simple dump and
 print statements. The syntax below uses "open" and "closed" statements to
 eliminate the traditional "dangling else" or "if-else" ambiguity problem.
 See http://www.parsifalsoft.com/ifelse.html or doc/ifelse.htm for a more
 detailed discussion.

 Statement types supported are:
   assignment statements
   compound statements
   if/else statements
   while statements
   repeat/until statements
   for statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"                // Contains definitions of support classes
#include <math.h>
#include <ctype.h>
#include "bcidefs.h"                       // Bytecode interpreter definitions


/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 26 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef DCI_H_1027348553
#include "dci.h"
#endif

#ifndef DCI_H_1027348553
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE dci_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer

CodeFragment dci_value(PCB_DECL);

static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 450, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

// implement context tracking
#define GET_CONTEXT CONTEXT = FileLocation(PCB.pointer, PCB.line, PCB.column)

void dci_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

void dci_pcb_struct::reportError(const char *msg) {
  ag_delete_wrappers(this);
  char buf[100];
  FileLocation &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): %s", context.line, context.column, msg);
  throw ErrorDiagnostic(buf);
}

// Constructor for parser control block
// Initializes dictionary, input pointer, and loopDepth

dci_pcb_struct::dci_pcb_struct(AgDictionary<AgString> &d, const char *text)
  : pointer((unsigned char *) text),
    dictionary(d)
{
  // Nothing else to do
}

int dci_pcb_struct::idName(const AgString &name) {
  return dictionary.intern(name);
}

CodeFragment dci_pcb_struct::code(Opcode op, const AgString &name) {
  return CodeFragment().append(op,dictionary.intern(name));
}

CodeFragment dci_pcb_struct::code(Opcode op, const Constant &x) {
  int n = constants.intern(x);
  return CodeFragment().append(op,n);
}

CodeFragment dci_pcb_struct::codeCall(
    const AgString &name,
    AgStack<CodeFragment> &argStack)
{
  int argCount = argStack.size();
  CodeFragment argCode;
  for (int i = 0; i < argCount; i++) argCode.concat(argStack[i]);
  argCode.append(CALL, idFunction(name, argCount));
  return argCode;
}


ScriptMethod::ScriptMethod(const char *text, AgDictionary<AgString> &d)
: dictionary(d),
  bytecode(),
  constantList()
{
  // Create a parser using the dictionary specified by the dataset
  dci_pcb_type pcb(dictionary, text);

  // run the parser
  try {
    dci(&pcb);
  }
  catch(ErrorMessage e) {
    pcb.reportError(e.message());
  }

  //Use dci_value() to retrieve the generated code
  bytecode = dci_value(&pcb).getBytecode();

  constantList = pcb.constants.contents();
}

// Apply script to a dataset
Value interpret(const char *text, Dataset &d) {
  ScriptMethod method(text, d.dictionary);

  // List the generated code
  //method.list(cout);

  // Execute the generated code
  return method.apply(d);
}


#ifndef CONVERT_CASE

static const char agCaseTable[31] = {
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,    0,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20
};

static int agConvertCase(int c) {
  if (c >= 'a' && c <= 'z') return c ^= 0x20;
  if (c >= 0xe0 && c < 0xff) c ^= agCaseTable[c-0xe0];
  return c;
}

#define CONVERT_CASE(c) agConvertCase(c)

#endif


#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif
CodeFragment dci_value(PCB_DECL) {
  CodeFragment returnValue;
  returnValue = (*(AgObjectWrapper< CodeFragment > *) &(PCB).vs[(PCB).ssx]);
  return returnValue;
}

static inline CodeFragment ag_rp_1(PCB_DECL, CodeFragment &x) {
/* Line 127, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x.append(HLT);
}

static inline CodeFragment ag_rp_2(PCB_DECL) {
/* Line 131, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_3(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 132, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x.concat(y);
}

static inline CodeFragment ag_rp_4(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 150, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return CodeFragment::ifStatement(x, s);
}

static inline CodeFragment ag_rp_5(PCB_DECL, CodeFragment &x, CodeFragment &s1, CodeFragment &s2) {
/* Line 152, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return CodeFragment::ifElse(x,s1,s2);
}

static inline CodeFragment ag_rp_6(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 153, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return CodeFragment::whileLoop(x,s);
}

static inline CodeFragment ag_rp_7(PCB_DECL, CodeFragment &lv, CodeFragment &b, CodeFragment &i, CodeFragment &e, CodeFragment &s) {
/* Line 156, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return CodeFragment::simpleForLoop(lv,b,i,e,s);
}

static inline CodeFragment ag_rp_8(PCB_DECL, CodeFragment &x, CodeFragment &s1, CodeFragment &s2) {
/* Line 160, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return CodeFragment::ifElse(x,s1,s2);
}

static inline CodeFragment ag_rp_9(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 162, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return CodeFragment::whileLoop(x,s);
}

static inline CodeFragment ag_rp_10(PCB_DECL, CodeFragment &lv, CodeFragment &b, CodeFragment &i, CodeFragment &e, CodeFragment &s) {
/* Line 165, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return CodeFragment::simpleForLoop(lv,b,i,e,s);
}

static inline CodeFragment ag_rp_11(PCB_DECL, CodeFragment &x) {
/* Line 166, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_12(PCB_DECL) {
/* Line 169, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return PCB.code(PUSHC, Value(1));
}

static inline CodeFragment ag_rp_13(PCB_DECL, CodeFragment &x) {
/* Line 170, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_14(PCB_DECL, CodeFragment &s, CodeFragment &x) {
/* Line 183, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return CodeFragment::repeatLoop(s,x);
}

static inline CodeFragment ag_rp_15(PCB_DECL, CodeFragment &lv, CodeFragment &x) {
/* Line 184, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return lv.concat(x).append(SAP);
}

static inline CodeFragment ag_rp_16(PCB_DECL) {
/* Line 185, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_17(PCB_DECL) {
/* Line 187, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return CodeFragment().append(HLT);
}

static inline CodeFragment ag_rp_18(PCB_DECL, CodeFragment &x) {
/* Line 188, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x.append(RETURN);
}

static inline CodeFragment ag_rp_19(PCB_DECL, CodeFragment &x) {
/* Line 202, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_20(PCB_DECL, CodeFragment &s) {
/* Line 205, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return s;
}

static inline CodeFragment ag_rp_21(PCB_DECL, AgString &n) {
/* Line 209, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return PCB.code(DUMP,n);
}

static inline CodeFragment ag_rp_22(PCB_DECL, CodeFragment &code, AgString &n) {
/* Line 210, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return code.append(DUMP, PCB.idName(n));
}

static inline CodeFragment ag_rp_23(PCB_DECL, CodeFragment &code) {
/* Line 214, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return code.append(PRINT);
}

static inline CodeFragment ag_rp_24(PCB_DECL, CodeFragment &code, CodeFragment &x) {
/* Line 215, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return code.concat(x).append(PRINT);
}

static inline CodeFragment ag_rp_25(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 221, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_26(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 225, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_27(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 229, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_28(PCB_DECL, CodeFragment &x) {
/* Line 234, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_29(PCB_DECL, Opcode op, CodeFragment &x) {
/* Line 235, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x.append(op);
}

static inline CodeFragment ag_rp_30(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 243, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x.concat(y).append(POW);
}

static inline CodeFragment ag_rp_31(PCB_DECL, CodeFragment &x) {
/* Line 251, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_32(PCB_DECL, Constant &x) {
/* Line 252, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return PCB.code(PUSHC, x);
}

static inline CodeFragment ag_rp_33(PCB_DECL, CodeFragment &x) {
/* Line 253, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x.append(FETCH);
}

static inline CodeFragment ag_rp_34(PCB_DECL, CodeFragment &x) {
/* Line 255, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x.append(CAST_LONG);
}

static inline CodeFragment ag_rp_35(PCB_DECL, CodeFragment &x) {
/* Line 256, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x.append(CAST_DOUBLE);
}

static inline CodeFragment ag_rp_36(PCB_DECL, AgString &n) {
/* Line 259, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return PCB.code(LOCATE, n);
}

static inline CodeFragment ag_rp_37(PCB_DECL, AgString &n, AgStack<CodeFragment> &args) {
/* Line 262, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return PCB.codeCall(n, args);
}

static inline AgStack<CodeFragment> ag_rp_38(PCB_DECL) {
/* Line 265, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return AgStack<CodeFragment>();
}

static inline AgStack<CodeFragment> ag_rp_39(PCB_DECL, CodeFragment &code) {
/* Line 269, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return AgStack<CodeFragment>().push(code);
}

static inline AgStack<CodeFragment> ag_rp_40(PCB_DECL, AgStack<CodeFragment> &stack, CodeFragment &x) {
/* Line 270, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return stack.push(x);
}

static inline Constant ag_rp_41(PCB_DECL, long x) {
/* Line 273, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return Constant(x);
}

static inline Constant ag_rp_42(PCB_DECL, double x) {
/* Line 274, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return Constant(x);
}

static inline Constant ag_rp_43(PCB_DECL, int x) {
/* Line 276, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return Constant(x);
}

static inline Opcode ag_rp_44(PCB_DECL) {
/* Line 280, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return EQ;
}

static inline Opcode ag_rp_45(PCB_DECL) {
/* Line 281, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return NE;
}

static inline Opcode ag_rp_46(PCB_DECL) {
/* Line 282, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return LT;
}

static inline Opcode ag_rp_47(PCB_DECL) {
/* Line 283, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return LE;
}

static inline Opcode ag_rp_48(PCB_DECL) {
/* Line 284, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return GT;
}

static inline Opcode ag_rp_49(PCB_DECL) {
/* Line 285, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return GE;
}

static inline Opcode ag_rp_50(PCB_DECL) {
/* Line 288, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return ADD;
}

static inline Opcode ag_rp_51(PCB_DECL) {
/* Line 289, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return SUB;
}

static inline Opcode ag_rp_52(PCB_DECL) {
/* Line 290, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return IOR;
}

static inline Opcode ag_rp_53(PCB_DECL) {
/* Line 291, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return XOR;
}

static inline Opcode ag_rp_54(PCB_DECL) {
/* Line 294, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return MUL;
}

static inline Opcode ag_rp_55(PCB_DECL) {
/* Line 295, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return RDIV;
}

static inline Opcode ag_rp_56(PCB_DECL) {
/* Line 296, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return IDIV;
}

static inline Opcode ag_rp_57(PCB_DECL) {
/* Line 297, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return MOD;
}

static inline Opcode ag_rp_58(PCB_DECL) {
/* Line 298, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return AND;
}

static inline Opcode ag_rp_59(PCB_DECL) {
/* Line 299, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return LS;
}

static inline Opcode ag_rp_60(PCB_DECL) {
/* Line 300, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return RS;
}

static inline Opcode ag_rp_61(PCB_DECL) {
/* Line 303, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return NEG;
}

static inline Opcode ag_rp_62(PCB_DECL) {
/* Line 304, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return NOT;
}

static inline AgString ag_rp_63(PCB_DECL, int c) {
/* Line 322, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return AgString().append(tolower(c));
}

static inline AgString ag_rp_64(PCB_DECL, AgString &ns, int c) {
/* Line 323, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return ns.append(tolower(c));
}

static inline double ag_rp_65(PCB_DECL, double x, long e) {
/* Line 329, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x*pow(10,e);
}

static inline double ag_rp_66(PCB_DECL, double x, long e) {
/* Line 330, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x*pow(10,e);
}

static inline double ag_rp_67(PCB_DECL, double i, double f) {
/* Line 333, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return i+f;
}

static inline double ag_rp_68(PCB_DECL, double i) {
/* Line 334, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return i;
}

static inline double ag_rp_69(PCB_DECL, double f) {
/* Line 335, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return f;
}

static inline double ag_rp_70(PCB_DECL, long x) {
/* Line 338, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x;
}

static inline double ag_rp_71(PCB_DECL, long x) {
/* Line 339, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x;
}

static inline double ag_rp_72(PCB_DECL, long x) {
/* Line 340, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return makeDecimal(x);
}

static inline long ag_rp_73(PCB_DECL, long x) {
/* Line 343, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return (long)x;
}

static inline long ag_rp_74(PCB_DECL, long x) {
/* Line 344, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return -(long)x;
}

static inline double ag_rp_75(PCB_DECL, int d) {
/* Line 347, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_76(PCB_DECL, int d, double f) {
/* Line 348, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_77(PCB_DECL, int d) {
/* Line 356, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return d-'0';
}

static inline long ag_rp_78(PCB_DECL, long x, int d) {
/* Line 357, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_79(PCB_DECL, int d) {
/* Line 360, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return d-'0';
}

static inline long ag_rp_80(PCB_DECL, long x, int d) {
/* Line 361, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_81(PCB_DECL, long x, int d) {
/* Line 364, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_82(PCB_DECL, long x, int d) {
/* Line 365, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_83(PCB_DECL) {
/* Line 368, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return 0;
}

static inline long ag_rp_84(PCB_DECL, long n, int d) {
/* Line 369, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_85(PCB_DECL) {
/* Line 372, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return 0;
}

static inline long ag_rp_86(PCB_DECL, long n, int d) {
/* Line 373, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return 16*n + d;
}

static inline int ag_rp_87(PCB_DECL, int x) {
/* Line 376, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return x - '0';
}

static inline int ag_rp_88(PCB_DECL, int d) {
/* Line 377, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return (d & 7) + 9;
}

static inline Constant ag_rp_89(PCB_DECL, Value &s) {
/* Line 382, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return Constant(s);
}

static inline Constant ag_rp_90(PCB_DECL, Constant &s, Value &e) {
/* Line 383, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return Constant(s+=e);
}

static inline Value ag_rp_91(PCB_DECL, AgString &b) {
/* Line 386, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return Value(b);
}

static inline AgString ag_rp_92(PCB_DECL) {
/* Line 390, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return AgString();
}

static inline AgString ag_rp_93(PCB_DECL, AgString &s, int c) {
/* Line 391, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return s.append(c);
}

static inline int ag_rp_94(PCB_DECL) {
/* Line 403, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return '\'';
}

static inline int ag_rp_95(PCB_DECL) {
/* Line 404, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return '"';
}

static inline int ag_rp_96(PCB_DECL) {
/* Line 405, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return '\?';
}

static inline int ag_rp_97(PCB_DECL) {
/* Line 406, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return '\\';
}

static inline int ag_rp_98(PCB_DECL) {
/* Line 407, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return '\a';
}

static inline int ag_rp_99(PCB_DECL) {
/* Line 408, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return '\b';
}

static inline int ag_rp_100(PCB_DECL) {
/* Line 409, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return '\f';
}

static inline int ag_rp_101(PCB_DECL) {
/* Line 410, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return '\n';
}

static inline int ag_rp_102(PCB_DECL) {
/* Line 411, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return '\r';
}

static inline int ag_rp_103(PCB_DECL) {
/* Line 412, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return '\t';
}

static inline int ag_rp_104(PCB_DECL) {
/* Line 413, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return '\v';
}

static inline int ag_rp_105(PCB_DECL, int d) {
/* Line 419, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return d-'0';
}

static inline int ag_rp_106(PCB_DECL, int n, int d) {
/* Line 422, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_107(PCB_DECL, int n, int d) {
/* Line 425, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_108(PCB_DECL, int d) {
/* Line 428, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return d;
}

static inline int ag_rp_109(PCB_DECL, int n, int d) {
/* Line 429, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return 16*n + d;
}

static inline int ag_rp_110(PCB_DECL, int d) {
/* Line 432, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return d-'0';
}

static inline int ag_rp_111(PCB_DECL, int d) {
/* Line 433, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_112(PCB_DECL, int c) {
/* Line 442, W:/agdev/agex/packages/xidek/pll/base/dci.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6,
  0, 6, 0, 0, 0, 0, 6, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 4, 8, 0,
  0, 0, 0, 0, 6, 0, 0, 6, 6, 6, 0, 0, 0, 6, 0, 0, 6, 6, 6, 6, 6, 4, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 7, 7, 0, 0, 0, 6, 6, 6, 6, 0, 0, 0, 0, 0, 6, 0, 0, 6, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<CodeFragment >
#undef AG_WRAP_7
#define AG_WRAP_7 AgObjectWrapper<AgStack<CodeFragment> >
#undef AG_WRAP_8
#define AG_WRAP_8 AgObjectWrapper<Constant >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      case 7: ag_delete_object((AG_WRAP_7 *) &(PCB).vs[sx]); break;
      case 8: ag_delete_object((AG_WRAP_8 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static dci_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  2,  3,  0,  0,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
   16,  0, 17, 18,  0,  0,  0,  0,  0, 19, 20, 21, 22, 23, 24,  0, 25,  0,
   26,  0, 27,  0, 28, 29,  0, 30, 31, 32, 33,  0, 34, 35, 36, 37, 38,  0,
   39, 40, 41, 42,  0, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55,
   56, 57, 58, 59, 60, 61, 62,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   63, 64,  0, 65, 66, 67, 68, 69, 70, 71, 72,  0,  0, 73, 74, 75, 76,  0,
    0,  0, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92,
   93,  0,  0,  0,  0,  0, 94, 95, 96, 97, 98, 99,100,101,102,103,104,  0,
    0,  0,105,106,107,108,109,110,111,112
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,148,  0,149,  0,150,  0,152,  0,154,  0,155,  0,156,  0,157,
  0,158,  0,160,  0,159,  0,162,  0,161,  0,164,  0,166,  0,171,
  0,172,  0,183,  0,184,  0,187,  0,188,  0,189,  0,190,  0,191,
  0,192,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 80, 84,255, 69,255, 47, 66, 68, 70, 73, 80, 82, 87,255,
   42,255, 42, 47,255, 61, 62,255, 69, 89,255, 85,255, 73, 79, 85,255, 76,
   78,255, 80, 84,255, 69,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 48,
   58, 60, 62, 65, 66, 68, 69, 70, 73, 76, 77, 78, 79, 80, 82, 83, 84, 85,
   87, 88,255, 80, 84,255, 69,255, 66, 68, 70, 73, 80, 82, 87,255, 42, 47,
  255, 47, 48, 78,255, 42, 47,255, 47,255, 42, 47,255, 80, 84,255, 69,255,
   47, 66, 68, 69, 70, 73, 80, 82, 87,255, 42, 47,255, 61, 62,255, 42, 47,
   58, 60, 62,255, 42, 47,255, 80, 84,255, 69,255, 47, 66, 68, 70, 73, 80,
   82, 85, 87,255, 48, 78,255, 80, 84,255, 69,255, 66, 68, 69, 70, 73, 80,
   82, 87,255, 42, 47,255, 76, 78,255, 80, 84,255, 69,255, 47, 66, 68, 69,
   70, 73, 80, 82, 85, 87,255, 58,255, 80, 84,255, 69,255, 66, 68, 70, 73,
   80, 82, 85, 87,255, 34, 39, 63, 65, 66, 70, 78, 82, 84, 86, 88, 92,255,
   92,255, 42, 47,255, 61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,
  255, 42, 47, 60, 62, 65, 66, 68, 77, 79, 83, 84, 88,255, 42, 47,255, 61,
   62,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 60, 62, 77, 79, 83, 84,
   88,255, 61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 60,
   62, 65, 66, 68, 77, 79, 83, 84, 88,255, 42, 47,255, 47, 48, 68, 76, 78,
  255, 48, 68, 76, 78,255, 61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72,
   79,255, 60, 62, 65, 66, 68, 77, 79, 83, 84, 88,255, 61, 62,255, 72, 79,
  255, 60, 62, 66, 68, 79, 84, 88,255, 84,255, 68,255, 76, 78,255, 80, 84,
  255, 69,255, 66, 68, 69, 70, 73, 80, 82, 85, 87,255, 42, 47,255, 61, 62,
  255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 48, 60, 62, 65,
   66, 68, 77, 79, 83, 84, 88,255, 48,255, 72, 79,255, 66, 68, 79, 84, 88,
  255, 66, 84,255, 84,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,7,7,4,2,4,2,7,7,7,7,7,2,7,4,3,4,0,0,4,0,0,4,7,5,4,7,4,7,6,7,4,
  7,7,4,7,7,4,2,4,5,5,4,2,4,7,5,4,3,2,3,3,2,3,7,2,2,2,7,7,7,7,7,7,7,2,2,
  2,7,7,7,4,7,7,4,2,4,7,7,7,7,7,2,7,4,0,0,4,2,3,7,4,0,0,4,2,4,0,0,4,7,7,
  4,2,4,2,7,7,7,7,7,7,2,7,4,0,0,4,0,0,4,3,2,3,2,3,4,0,0,4,7,7,4,2,4,2,7,
  7,7,7,7,2,7,7,4,3,7,4,7,7,4,2,4,7,7,7,7,7,7,2,7,4,0,0,4,7,7,4,7,7,4,2,
  4,2,7,7,2,7,7,7,2,7,7,4,3,4,7,7,4,2,4,7,7,7,7,7,2,7,7,4,0,0,0,0,0,0,0,
  0,0,0,0,0,4,2,4,0,0,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,2,3,7,7,2,7,7,2,
  2,7,4,0,0,4,0,0,4,5,5,4,2,4,7,5,4,3,2,2,3,7,7,2,2,7,4,0,0,4,7,5,4,5,5,
  4,2,4,7,5,4,3,2,3,7,7,2,7,7,2,2,7,4,0,0,4,2,3,7,7,7,4,3,7,7,7,4,0,0,4,
  7,5,4,5,5,4,2,4,7,5,4,2,3,7,7,2,7,7,2,2,7,4,0,0,4,7,5,4,2,3,7,7,7,2,7,
  4,7,4,7,4,7,7,4,7,7,4,2,4,7,7,2,7,7,7,2,7,7,4,0,0,4,0,0,4,7,5,4,5,5,4,
  2,4,7,5,4,3,2,3,2,3,7,7,2,7,7,2,2,7,4,3,4,7,5,4,7,7,7,2,7,4,7,7,4,7,4
};

static const unsigned char ag_key_parm[] = {
    0, 80, 85,  0, 12, 10,  0,  0,  0,  0, 22, 26, 16, 18, 28,  0, 14,  0,
   84,  0, 80, 85,  0,179,177,  0, 22,  6,  0, 32,  0, 38,  2, 26,  0,  0,
   24,  0, 12, 10,  0,  0,  0, 44, 46,  0,  0,  0, 20,  4,  0,168,  0,110,
  151,  0,181, 42,  0,  0,  0, 16, 18, 30, 40, 48, 34, 28,  0,  0,  0,  8,
   14, 36,  0, 12, 10,  0,  0,  0, 22, 26, 16, 18, 28,  0, 14,  0, 80, 85,
    0,  0,110, 48,  0, 80, 85,  0,  0,  0, 80, 85,  0, 12, 10,  0,  0,  0,
    0, 22, 26, 24, 16, 18, 28,  0, 14,  0, 80, 85,  0,179,177,  0,168,  0,
  151,  0,181,  0, 80, 85,  0, 12, 10,  0,  0,  0,  0, 22, 26, 16, 18, 28,
    0,  8, 14,  0,110, 48,  0, 12, 10,  0,  0,  0, 22, 26, 24, 16, 18, 28,
    0, 14,  0, 80, 85,  0,  0, 24,  0, 12, 10,  0,  0,  0,  0, 22, 26,  0,
   16, 18, 28,  0,  8, 14,  0,151,  0, 12, 10,  0,  0,  0, 22, 26, 16, 18,
   28,  0,  8, 14,  0,122,121,123,125,126,127,128,129,130,131,136,124,  0,
    0,  0, 80, 85,  0,179,177,  0, 38,  2,  0, 44, 46,  0,  0,  0, 20,  4,
    0,168,  0,  0,181, 42,  6,  0, 40, 34,  0,  0, 36,  0, 80, 85,  0,179,
  177,  0, 44, 46,  0,  0,  0, 20,  4,  0,168,  0,  0,181, 40, 34,  0,  0,
   36,  0,179,177,  0, 38,  2,  0, 44, 46,  0,  0,  0, 20,  4,  0,168,  0,
  181, 42,  6,  0, 40, 34,  0,  0, 36,  0, 80, 85,  0,  0,110, 32, 30, 48,
    0,110, 32, 30, 48,  0,179,177,  0, 38,  2,  0, 44, 46,  0,  0,  0, 20,
    4,  0,  0,181, 42,  6,  0, 40, 34,  0,  0, 36,  0,179,177,  0, 20,  4,
    0,  0,181,  6,  2, 34,  0, 36,  0, 20,  0,  2,  0,  0, 24,  0, 12, 10,
    0,  0,  0, 22, 26,  0, 16, 18, 28,  0,  8, 14,  0, 80, 85,  0,179,177,
    0, 38,  2,  0, 44, 46,  0,  0,  0, 20,  4,  0,168,  0,110,  0,181, 42,
    6,  0, 40, 34,  0,  0, 36,  0,110,  0, 20,  4,  0,  6,  2, 34,  0, 36,
    0,  6,  4,  0,  4,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0, 19, 23,  0,  4,  0,  1,  0,  5,  9, 12, 14,  7, 27,  0,
   32,  0,  0,  0,  0,  0,  0,  0, 45,  0,  0, 51,  0, 49, 29, 55,  0, 58,
   61,  0, 85, 89,  0, 38,  0,  0,  0,  0, 43,  0, 93,  0,  0, 34, 20, 36,
   38, 23, 40, 42, 26, 31, 35, 63, 66, 68, 72, 75, 78, 80, 41, 46, 48, 96,
  101,106,  0,128,132,  0, 75,  0,109,114,118,121,123, 78,136,  0,  0,  0,
    0, 88,141,143,  0,  0,  0,  0, 95,  0,  0,  0,  0,168,172,  0,103,  0,
  100,146,151,155,158,161,163,106,176,  0,  0,  0,  0,  0,  0,  0,181,118,
  183,121,185,  0,  0,  0,  0,206,210,  0,133,  0,130,187,192,196,199,201,
  136,214,219,  0,224,226,  0,251,255,  0,151,  0,229,234,238,241,244,246,
  154,259,  0,  0,  0,  0,273,276,  0,288,292,  0,171,  0,165,264,269,168,
  278,281,283,174,296,301,  0,306,  0,327,331,  0,189,  0,308,313,317,320,
  322,192,335,340,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
  203,  0,  0,  0,  0,  0,  0,  0,354,  0,  0,  0,  0,  0,227,  0,361,  0,
    0,345,218,221,347,349,352,224,356,359,230,232,364,  0,  0,  0,  0,  0,
    0,  0,  0,  0,  0,254,  0,376,  0,  0,367,248,251,369,371,374,257,259,
  379,  0,  0,  0,  0,391,  0,  0,  0,  0,  0,278,  0,398,  0,  0,382,272,
  384,386,389,275,393,396,281,283,401,  0,  0,  0,  0,298,404,406,412,416,
    0,419,421,427,431,  0,  0,  0,  0,441,  0,  0,  0,  0,  0,318,  0,448,
    0,  0,312,434,436,439,315,443,446,321,323,451,  0,  0,  0,  0,462,  0,
    0,337,454,456,458,460,340,465,  0,468,  0,472,  0,483,486,  0,498,502,
    0,358,  0,474,479,355,488,491,493,361,506,511,  0,  0,  0,  0,  0,  0,
    0,527,  0,  0,  0,  0,  0,382,  0,534,  0,  0,516,373,518,376,520,522,
  525,379,529,532,385,387,537,  0,540,  0,548,  0,  0,542,544,546,406,551,
    0,554,556,  0,558,  0
};

static const unsigned short ag_key_index[] = {
    9,  0, 18, 51, 80,  0,  0, 18, 18, 80, 91, 98,108,124,138,148,  0, 91,
  156,176, 98, 91, 91,  0,  0,148,187,194,148,  0,148, 80,216,  0,  0,216,
  235,  0,235,262,235,235,235,235,235,286,286,301, 91, 91,307, 91,286,148,
  148,326,343,156, 91,148,  0,  0, 91,148,194,351,187,353,363,216,235,  0,
    0,216,216,216,  0,235,  0,  0,148, 98,  0, 98,  0,  0, 91,148, 91, 91,
   91, 91, 91, 91, 91,148, 91, 91,148, 91, 91, 91, 91, 91, 91,148,176,  0,
   91,148,  9,148,  9, 80,  9, 80,  0,  0,  0,  0,390,404,404,326,409,  0,
  415,235,235,148, 91,148,418, 91,148,353, 80
};

static const unsigned char ag_key_ends[] = {
69,71,73,78,0, 85,77,80,0, 79,82,0, 70,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 72,73,76,69,0, 47,0, 42,0, 88,0, 61,0, 
61,0, 78,68,0, 71,73,78,0, 86,0, 66,76,69,0, 77,80,0, 83,69,0, 
68,0, 79,82,0, 70,0, 79,78,71,0, 79,68,0, 79,84,0, 82,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 69,78,0, 78,84,73,76,0, 
72,73,76,69,0, 79,82,0, 69,71,73,78,0, 85,77,80,0, 79,82,0, 
70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 72,73,76,69,0, 
88,0, 79,84,0, 69,71,73,78,0, 85,77,80,0, 78,68,0, 79,82,0, 
70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 72,73,76,69,0, 
42,0, 61,0, 61,0, 69,71,73,78,0, 85,77,80,0, 79,82,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 78,84,73,76,0, 
72,73,76,69,0, 88,0, 79,84,0, 69,71,73,78,0, 85,77,80,0, 
78,68,0, 79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
72,73,76,69,0, 69,71,73,78,0, 85,77,80,0, 83,69,0, 68,0, 
79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
78,84,73,76,0, 72,73,76,69,0, 61,0, 69,71,73,78,0, 85,77,80,0, 
79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
78,84,73,76,0, 72,73,76,69,0, 42,0, 61,0, 78,68,0, 89,0, 86,0, 
79,68,0, 82,0, 69,78,0, 79,82,0, 42,0, 61,0, 79,68,0, 82,0, 
69,78,0, 79,82,0, 42,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 
82,0, 69,78,0, 79,82,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 
79,84,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 79,84,0, 61,0, 
78,68,0, 89,0, 86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 61,0, 
89,0, 79,0, 82,0, 69,78,0, 79,82,0, 72,69,78,0, 79,0, 
69,71,73,78,0, 85,77,80,0, 83,69,0, 68,0, 79,82,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 78,84,73,76,0, 
72,73,76,69,0, 42,0, 88,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 
82,0, 69,78,0, 79,82,0, 88,0, 89,0, 79,0, 82,0, 69,78,0, 
79,82,0, 89,0, 79,0, 79,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
    9,142,142,142,142,142,142,142,142,141, 89,141,141,141,142,142,142,142,
  142,142,142,142,142,142,142,142,142,142,142,142,142,142,141,142,113,142,
  142,142,142,138,170,169,185,167,165,182, 96,186,108,143,143,143,143,143,
  143,143,107,107,142,153,178,176,180,142,142,144,144,144,144, 93,144,145,
  145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
  145,142,135,142,142,145,142,144,144,144,144, 93,144,145,145,145,145,145,
  145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,142,142,142,
  142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
  142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
  142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
  142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
  142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
  142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
  142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
  142,142,142,142
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (dci_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
166,164,162,160,158,157,156,155,153,145,144,141,93,89,85,80,9,0,1,146,147,
186,185,182,180,178,176,170,169,167,165,153,145,144,143,142,141,138,135,113,
  108,107,96,93,89,0,87,88,
186,185,182,180,178,176,170,169,167,165,153,145,144,143,142,141,138,135,113,
  108,107,96,93,89,84,0,82,83,
141,89,85,80,0,1,
166,164,162,160,158,157,156,155,153,145,144,93,9,0,7,8,
186,185,182,180,178,176,170,169,167,165,153,145,144,143,142,141,138,135,113,
  108,107,96,93,0,
89,0,
186,185,182,180,178,176,170,169,167,165,153,145,144,143,142,141,138,135,113,
  108,107,96,93,89,0,
84,0,
166,164,162,160,158,157,156,155,153,145,144,93,9,0,4,10,11,12,13,15,18,19,
  23,25,27,28,29,30,31,32,33,34,35,37,39,41,163,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
145,144,141,93,89,85,80,0,1,146,147,
166,164,162,161,160,158,157,156,155,153,145,144,141,93,89,85,80,0,1,146,147,
186,185,182,181,180,179,178,177,176,170,169,168,167,165,153,151,145,144,143,
  141,108,107,93,89,85,80,0,1,146,147,
166,164,162,160,158,157,156,155,154,153,145,144,141,93,89,85,80,0,1,146,147,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,16,19,42,
  44,46,48,49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,
  193,
145,144,93,0,4,163,
192,182,170,167,153,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,
  146,147,
166,164,162,161,160,158,157,156,155,153,145,144,93,0,8,
166,164,162,161,160,158,157,156,155,154,153,148,145,144,141,93,89,85,80,9,0,
  1,146,147,
145,144,141,93,89,85,80,0,1,146,147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
165,153,0,27,40,
165,153,0,27,40,
192,182,170,167,153,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,16,19,
  27,42,44,46,48,49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,
  174,175,193,
151,0,20,
166,164,162,160,158,157,156,155,154,153,145,144,93,0,8,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,16,19,42,
  44,46,48,49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,
  193,
145,144,93,0,4,19,163,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,16,19,42,
  44,46,48,49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,
  193,
166,164,162,160,158,157,156,155,153,145,144,93,0,4,10,11,12,13,15,18,19,23,
  25,27,28,29,30,31,32,33,34,35,37,39,41,163,
186,185,182,180,178,176,170,169,167,165,153,145,144,143,142,141,138,136,135,
  131,130,129,128,127,126,125,124,123,122,121,113,108,107,96,93,0,114,
143,108,107,0,103,
143,108,107,0,97,103,
186,185,182,180,178,176,170,169,167,165,153,145,144,143,142,141,136,135,131,
  130,129,128,127,126,125,124,123,122,121,113,108,107,96,93,0,117,118,119,
  120,132,133,134,139,
191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176,169,168,167,
  165,159,153,152,150,149,141,113,89,85,80,0,1,146,147,
96,93,0,
93,0,
144,143,108,107,93,0,111,
191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176,169,168,167,
  165,159,153,152,150,149,143,141,108,107,96,93,89,85,80,0,
191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176,169,168,167,
  165,159,153,152,150,149,143,141,108,107,96,93,89,85,80,0,103,
191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176,169,168,167,
  165,159,153,152,150,149,141,89,85,80,0,1,146,147,
191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176,169,168,167,
  165,159,153,152,150,149,141,89,85,80,0,1,146,147,
191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176,169,168,167,
  165,159,153,152,150,149,141,89,85,80,0,1,146,147,
170,0,53,
113,0,5,193,
192,182,172,171,170,169,167,145,144,143,141,138,113,110,108,107,96,93,89,85,
  80,0,1,146,147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,172,171,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,
  16,19,42,44,46,48,49,50,51,53,55,56,57,58,61,68,78,92,95,98,99,100,104,
  163,173,174,175,193,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
168,0,52,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,19,46,48,
  49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,193,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,19,46,48,
  49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,193,
191,190,189,188,187,186,185,0,47,71,72,73,74,75,76,77,
184,183,182,181,180,179,178,177,176,167,0,43,45,49,62,63,64,65,66,67,68,69,
  70,
166,164,162,161,160,158,157,156,155,153,145,144,93,0,4,10,11,12,13,15,18,19,
  23,25,27,28,29,30,31,32,33,34,35,37,38,39,41,163,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,16,19,42,
  44,46,48,49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,
  193,
145,144,93,0,4,163,
153,0,27,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,16,19,42,
  44,46,48,49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,
  193,
166,164,162,160,158,157,156,155,154,153,145,144,93,0,4,10,11,12,13,15,18,19,
  23,25,26,27,28,29,30,31,32,33,34,35,37,39,41,163,
159,0,36,
151,0,20,
149,0,17,
148,0,14,
186,185,182,180,178,176,170,169,167,165,153,145,144,143,142,141,138,136,135,
  131,130,129,128,127,126,125,124,123,122,121,113,108,107,96,93,0,115,117,
  118,119,120,132,133,134,
143,108,107,0,97,103,
143,108,0,
144,143,108,107,93,0,137,
143,108,0,
143,108,0,
144,143,108,107,93,0,137,
138,0,
143,108,107,0,97,103,
182,167,143,108,107,0,94,101,
182,167,143,108,107,0,94,101,
192,182,170,169,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,16,19,
  42,44,46,48,49,50,51,53,55,56,59,60,61,68,78,92,95,98,99,100,104,163,
  173,174,175,193,
169,141,89,85,80,0,1,146,147,
169,0,54,
169,141,89,85,80,0,1,146,147,
169,0,54,
169,0,54,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,19,46,48,
  49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,193,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,19,46,48,
  49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,193,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,19,44,46,
  48,49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,193,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,19,42,44,
  46,48,49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,
  193,
166,164,162,161,160,158,157,156,155,154,153,148,145,144,141,93,89,85,80,9,0,
  1,146,147,
153,0,27,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,16,19,42,
  44,46,48,49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,
  193,
166,164,162,160,158,157,156,155,153,145,144,141,93,89,85,80,0,1,146,147,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,16,19,42,
  44,46,48,49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,
  193,
166,164,162,160,158,157,156,155,153,145,144,141,93,89,85,80,0,1,146,147,
166,164,162,160,158,157,156,155,153,145,144,93,0,4,11,12,13,15,18,19,23,25,
  27,28,29,30,31,32,33,34,35,37,39,41,163,
166,164,162,160,158,157,156,155,153,145,144,141,93,89,85,80,0,1,146,147,
166,164,162,160,158,157,156,155,153,145,144,93,0,4,11,12,13,15,18,19,23,25,
  27,28,29,30,31,32,33,34,35,37,39,41,163,
143,108,107,0,102,
143,108,107,0,102,
165,0,40,
169,0,54,
191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176,170,169,168,
  167,165,159,153,152,150,149,145,144,143,141,138,113,110,108,107,96,93,
  89,85,80,0,1,146,147,
170,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,19,51,53,55,56,61,92,
  95,98,99,100,104,163,173,174,175,193,
170,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,19,51,53,55,56,61,92,
  95,98,99,100,104,163,173,174,175,193,
191,190,189,188,187,186,185,0,47,71,72,73,74,75,76,77,
184,183,182,167,0,45,49,68,69,70,
153,0,27,
152,150,0,21,24,
143,108,107,0,103,
143,108,107,0,103,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,16,19,42,
  44,46,48,49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,
  193,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,16,19,42,
  44,46,48,49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,
  193,
150,0,22,
192,182,170,167,145,144,143,141,138,113,110,108,107,96,93,89,85,80,0,1,146,
  147,
192,182,170,167,145,144,143,138,113,110,108,107,96,93,0,2,3,4,5,6,16,19,42,
  44,46,48,49,50,51,53,55,56,61,68,78,92,95,98,99,100,104,163,173,174,175,
  193,
149,0,17,
166,164,162,160,158,157,156,155,153,145,144,93,0,4,11,12,13,15,18,19,23,25,
  27,28,29,30,31,32,33,34,35,37,39,41,163,

};


static unsigned const char ag_astt[2855] = {
  8,8,8,8,8,8,8,8,8,8,8,1,8,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,7,
  1,1,9,9,1,1,5,3,4,4,4,4,4,4,4,4,4,4,4,4,4,7,0,1,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,9,9,9,9,9,9,5,3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  5,3,7,1,1,1,1,1,1,1,1,1,2,2,2,2,7,2,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,
  5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,
  10,10,1,10,10,10,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,
  1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,2,2,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,2,2,2,7,2,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,
  1,3,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,7,3,1,1,1,7,3,1,1,1,1,1,1,2,2,
  2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,2,2,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,7,1,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,
  2,2,1,2,2,1,2,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,
  1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,2,1,1,1,1,1,1,1,1,2,1,1,2,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,1,2,1,2,2,1,1,
  1,1,1,1,1,1,1,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,7,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,1,5,2,2,2,2,2,5,2,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,2,4,4,5,5,5,7,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,10,4,4,5,5,5,7,2,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,1,4,1,1,5,2,1,5,5,5,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,2,2,2,1,1,2,
  2,2,1,2,7,2,2,1,2,2,1,2,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,5,1,1,1,1,1,2,2,2,1,1,2,2,
  2,1,2,7,2,2,1,2,2,2,2,2,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,2,2,2,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,5,1,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,5,1,1,2,2,2,2,2,2,2,
  2,2,2,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,
  2,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,
  2,1,2,7,2,2,1,2,2,2,2,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,
  2,7,2,1,1,7,2,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,
  1,2,2,2,1,2,7,2,2,1,2,2,1,2,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,2,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,
  1,1,1,7,2,1,7,1,1,7,1,1,5,1,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
  10,10,1,1,2,2,2,2,2,2,2,2,2,2,2,2,10,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,2,
  1,2,2,7,2,2,2,2,2,7,2,2,2,5,2,2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,8,
  7,2,1,1,1,8,8,8,7,2,1,1,1,1,4,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,2,2,1,1,1,
  1,1,1,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,1,1,1,1,7,1,1,3,1,7,1,5,1,
  1,1,1,7,1,1,3,1,7,1,1,7,2,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,
  1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,2,2,2,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,2,2,2,1,1,1,1,
  2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,
  7,2,2,1,2,2,2,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,
  5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,2,1,1,
  1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,
  5,1,1,1,5,7,1,1,3,1,7,2,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,2,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,
  2,1,2,7,2,2,1,2,2,1,2,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,
  5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,
  1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,
  1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,
  7,1,2,2,2,7,1,1,5,1,1,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,
  2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,2,2,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,2,2,2,2,2,2,2,1,1,1,1,4,1,2,
  2,2,2,1,7,2,1,4,7,1,1,10,10,10,4,2,10,10,10,4,2,1,1,1,1,2,2,2,1,1,2,2,2,1,
  2,7,2,2,1,2,2,2,2,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,
  2,2,2,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,7,1,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,2,2,1,2,
  1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,7,1,1,1,1,1,1,1,1,1,1,2,
  2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
4,4,4,4,4,4,4,4,4,4,4,3,4,3,1,2,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
267,267,1,2,269,267,
2,2,2,2,2,2,2,2,2,2,2,2,2,4,0,9,
86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,86,88,
89,6,
81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,81,83,
84,8,
10,11,12,22,20,21,14,17,19,90,90,90,1,9,50,3,3,3,31,30,29,26,13,27,18,13,25,
  24,23,27,30,29,28,18,16,15,13,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,10,3,3,288,
268,268,3,268,3,1,2,11,3,3,286,
268,268,268,268,268,268,268,268,268,268,268,268,3,268,3,1,2,12,3,3,284,
268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,91,91,91,3,
  91,91,91,3,1,2,13,3,3,285,
268,268,268,268,268,268,268,268,268,268,268,268,3,268,3,1,2,14,3,3,278,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,15,56,57,45,122,59,31,46,56,
  55,55,55,54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
90,90,90,16,29,13,
268,268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,17,3,3,
  277,
2,2,2,2,2,2,2,2,2,2,2,2,2,18,57,
268,268,268,268,268,268,268,268,268,268,268,268,268,268,3,268,3,1,2,268,19,
  3,3,275,
268,268,3,268,3,1,2,20,3,3,280,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,21,3,3,279,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,22,3,3,282,
58,19,23,23,59,
58,19,24,22,60,
48,49,47,51,19,90,90,110,35,32,118,116,110,34,90,25,56,57,45,122,59,61,46,
  20,56,55,55,55,54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,
  36,
62,26,63,
2,2,2,2,2,2,2,2,2,2,2,2,2,27,64,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,28,56,57,45,122,59,65,46,56,
  55,55,55,54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
90,90,90,29,50,66,13,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,30,56,57,45,122,59,67,46,56,
  55,55,55,54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
10,11,12,22,20,21,14,17,19,90,90,90,31,50,6,6,68,31,30,29,26,13,27,18,13,25,
  24,23,27,30,29,28,18,16,15,13,
125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,
  125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,32,69,
115,115,115,99,115,
70,70,70,34,97,70,
76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,76,72,71,142,141,140,139,138,
  137,136,135,134,133,132,76,76,76,76,76,35,76,76,76,75,74,73,76,76,
268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,
  268,268,268,268,268,268,3,268,3,1,2,36,3,3,315,
77,78,37,
79,92,
121,120,120,120,121,109,119,
108,108,108,108,108,108,108,108,108,108,108,108,108,108,108,108,108,108,108,
  108,108,108,108,108,108,117,108,117,114,100,100,108,108,108,40,
107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,107,
  107,107,107,107,107,107,111,107,111,111,98,98,107,107,107,41,111,
268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,
  268,268,268,268,268,268,3,3,1,2,42,3,3,297,
268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,
  268,268,268,268,268,268,3,3,1,2,43,3,3,296,
268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,
  268,268,268,268,268,268,3,3,1,2,44,3,3,295,
47,50,80,
32,58,123,36,
268,268,268,268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,
  47,3,3,292,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,48,3,3,314,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,49,3,3,304,
48,49,81,83,47,51,90,90,110,35,32,118,116,110,34,90,50,56,57,45,122,59,85,
  46,56,55,55,55,54,53,52,50,45,52,84,82,46,77,78,38,37,41,33,40,39,13,44,
  43,42,36,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,51,3,3,289,
86,42,87,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,53,56,57,45,122,59,46,41,41,
  54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,54,56,57,45,122,59,46,40,40,
  54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
88,89,90,91,92,93,94,35,95,70,71,72,73,74,75,76,
96,97,49,99,100,101,102,103,104,51,33,105,98,66,60,61,62,63,64,65,67,68,69,
10,11,12,106,22,20,21,14,17,19,90,90,90,57,50,3,3,3,31,30,29,26,13,27,18,13,
  25,24,23,27,30,29,28,18,28,16,15,13,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,58,3,3,287,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,59,56,57,45,122,59,32,46,56,
  55,55,55,54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
90,90,90,60,30,13,
19,61,21,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,62,3,3,273,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,63,56,57,45,122,59,107,46,56,
  55,55,55,54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
10,11,12,22,20,21,14,17,108,19,90,90,90,64,50,3,3,3,31,30,29,26,13,27,109,
  18,13,25,24,23,27,30,29,28,18,16,15,13,
110,65,27,
62,66,111,
112,67,113,
114,5,115,
126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,72,71,
  142,141,140,139,138,137,136,135,134,133,132,124,126,126,126,126,69,126,
  126,126,126,75,74,73,126,
70,70,70,105,106,70,
146,146,71,
152,151,151,151,152,72,149,
148,148,144,
147,147,143,
152,151,151,151,152,131,150,
153,76,
70,70,70,96,95,70,
116,117,117,117,117,78,94,117,
116,117,117,117,117,79,93,117,
48,49,47,52,51,90,90,110,35,32,118,116,110,34,90,80,56,57,45,122,59,54,46,
  56,55,55,55,54,53,52,50,45,52,119,118,46,77,78,38,37,41,33,40,39,13,44,
  43,42,36,
268,3,3,1,2,81,3,3,294,
120,82,121,
268,3,3,1,2,83,3,3,293,
120,84,122,
120,85,44,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,86,3,3,290,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,87,56,57,45,122,59,46,43,43,
  54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,88,3,3,313,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,89,3,3,312,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,90,3,3,311,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,91,3,3,310,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,92,3,3,309,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,93,3,3,308,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,94,3,3,307,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,95,56,57,45,122,59,46,38,38,
  54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,96,3,3,306,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,97,3,3,305,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,98,56,57,45,122,59,46,123,123,
  123,54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,99,3,3,303,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,100,3,3,302,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,101,3,3,301,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,102,3,3,300,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,103,3,3,299,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,104,3,3,298,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,105,56,57,45,122,59,46,124,55,
  55,55,54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
268,268,268,268,268,268,268,268,268,268,268,268,268,268,3,268,3,1,2,268,106,
  3,3,283,
19,107,17,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,108,3,3,276,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,109,56,57,45,122,59,125,46,56,
  55,55,55,54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
268,268,268,268,268,268,268,268,268,268,268,3,268,3,1,2,110,3,3,281,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,111,56,57,45,122,59,126,46,56,
  55,55,55,54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
268,268,268,268,268,268,268,268,268,268,268,3,268,3,1,2,112,3,3,271,
10,11,12,22,20,21,14,17,19,90,90,90,113,50,8,11,31,30,29,26,13,27,18,13,25,
  24,23,27,30,29,28,18,16,15,13,
268,268,268,268,268,268,268,268,268,268,268,3,268,3,1,2,114,3,3,270,
10,11,12,22,20,21,14,17,19,90,90,90,115,50,7,10,31,30,29,26,13,27,18,13,25,
  24,23,27,30,29,28,18,16,15,13,
112,112,112,116,127,
112,112,112,117,128,
58,53,129,
120,119,51,
268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,268,
  268,268,268,268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,
  1,2,120,3,3,291,
47,90,90,110,35,32,118,116,110,34,90,121,56,57,45,122,59,46,49,50,45,49,46,
  38,37,41,33,40,39,13,44,43,42,36,
47,90,90,110,35,32,118,116,110,34,90,122,56,57,45,122,59,46,48,50,45,48,46,
  38,37,41,33,40,39,13,44,43,42,36,
88,89,90,91,92,93,94,36,95,70,71,72,73,74,75,76,
96,97,49,51,34,98,66,67,68,69,
19,125,16,
130,14,126,132,131,
113,113,113,104,113,
113,113,113,103,113,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,129,56,57,45,122,59,55,46,56,
  55,55,55,54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,130,3,3,274,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,131,56,57,45,122,59,15,46,56,
  55,55,55,54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
133,132,134,
268,268,268,268,268,268,268,3,268,268,268,268,268,268,268,3,1,2,133,3,3,272,
48,49,47,51,90,90,110,35,32,118,116,110,34,90,134,56,57,45,122,59,135,46,56,
  55,55,55,54,53,52,50,45,52,46,77,78,38,37,41,33,40,39,13,44,43,42,36,
112,135,136,
10,11,12,22,20,21,14,17,19,90,90,90,136,50,9,12,31,30,29,26,13,27,18,13,25,
  24,23,27,30,29,28,18,16,15,13,

};


static const unsigned short ag_sbt[] = {
     0,  21,  48,  76,  82,  98, 122, 124, 149, 151, 188, 210, 221, 242,
   272, 293, 339, 345, 368, 383, 407, 418, 440, 462, 467, 472, 520, 523,
   538, 584, 591, 637, 673, 710, 715, 721, 764, 798, 801, 803, 810, 845,
   881, 914, 947, 980, 983, 987,1012,1034,1056,1106,1128,1131,1174,1217,
  1233,1256,1294,1316,1362,1368,1371,1393,1439,1477,1480,1483,1486,1489,
  1533,1539,1542,1549,1552,1555,1562,1564,1570,1578,1586,1635,1644,1647,
  1656,1659,1662,1684,1727,1749,1771,1793,1815,1837,1859,1881,1924,1946,
  1968,2012,2034,2056,2078,2100,2122,2144,2189,2213,2216,2238,2284,2304,
  2350,2370,2405,2425,2460,2465,2470,2473,2476,2520,2554,2588,2604,2614,
  2617,2622,2627,2632,2678,2700,2746,2749,2771,2817,2820,2855
};


static const unsigned short ag_sbe[] = {
    17,  45,  73,  80,  95, 121, 123, 148, 150, 164, 206, 217, 238, 268,
   289, 307, 342, 364, 381, 403, 414, 436, 458, 464, 469, 487, 521, 536,
   552, 587, 605, 649, 708, 713, 718, 755, 794, 800, 802, 808, 844, 879,
   910, 943, 976, 981, 984,1008,1030,1052,1072,1124,1129,1145,1188,1224,
  1243,1269,1312,1330,1365,1369,1389,1407,1452,1478,1481,1484,1487,1524,
  1536,1541,1547,1551,1554,1560,1563,1567,1575,1583,1601,1640,1645,1652,
  1657,1660,1680,1698,1745,1767,1789,1811,1833,1855,1877,1895,1942,1964,
  1982,2030,2052,2074,2096,2118,2140,2158,2209,2214,2234,2252,2300,2318,
  2366,2382,2421,2437,2463,2468,2471,2474,2516,2531,2565,2595,2608,2615,
  2619,2625,2630,2646,2696,2714,2747,2767,2785,2818,2832,2855
};


static const unsigned char ag_fl[] = {
  2,2,0,2,1,1,2,4,4,9,4,4,9,1,0,2,5,4,1,1,2,3,2,2,1,1,1,3,3,2,3,2,3,1,3,
  1,3,1,3,1,2,2,1,3,3,1,1,1,4,4,1,4,0,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,1,2,1,3,3,3,2,2,1,1,1,0,1,2,2,
  1,2,1,1,1,1,2,1,2,2,2,1,2,1,2,1,1,1,2,3,0,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2
};

static const unsigned char ag_ptt[] = {
    0,  7,  8,  8, 10, 10, 11, 11, 11, 11, 12, 12, 12, 12, 21, 21, 23, 23,
   23, 23, 23, 23, 23, 23, 25, 15, 18, 13, 28, 30, 30, 31, 31, 16, 16, 42,
   42, 44, 44, 46, 46, 46, 48, 48, 51, 51, 51, 51, 51, 51, 19, 56, 59, 59,
   60, 60, 55, 55, 55, 55, 43, 43, 43, 43, 43, 43, 45, 45, 45, 45, 47, 47,
   47, 47, 47, 47, 47, 50, 50,  1, 82, 82, 83, 83,  1, 87, 87, 88, 88,  1,
  163,163,174,174,174, 92, 92, 92, 95, 95, 95,101,101, 94, 94, 97, 97,173,
  173,173, 98, 98,102,102, 99, 99,100,100,104,104,111,111, 61, 61,193,114,
  114,115,115,117,117,117,118,118,118,118,118,118,118,118,118,118,118,119,
  119,119,132,133,134,120,120,137,137,175,139,139,103, 79, 79, 81, 81, 81,
   81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81,
   81, 81, 81, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86,
   86, 86, 86, 86, 86, 86, 86, 86, 90, 90, 90, 91, 91, 91, 91, 91, 91,105,
  105,106,106,106,109,109,112,112,116,116,116,116,116,116,116,116,116,116,
  116,116,116,116,116,116,116,116,116,116,116,140,140,140,140,140,140,140,
  140,140,140,140,140,140,140,140,140,140,140,140,140,140,146,146,147,147,
   14, 17, 22, 20, 24, 27, 26, 29, 32, 33, 34, 36, 35, 38, 37,  4, 39, 40,
   41, 49, 52, 54, 53, 57, 58,  2,  3,  6, 62, 63, 64, 65, 66, 67, 68, 69,
   70, 71, 72, 73, 74, 75, 76, 77, 78,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: VRO(AG_WRAP_6 *, ag_rp_1(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 2: VNO AG_WRAP_6(ag_rp_2(PCB_POINTER)); break;
    case 3: VRO(AG_WRAP_6 *, ag_rp_3(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 4: VRO(AG_WRAP_6 *, ag_rp_4(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 5: VRO(AG_WRAP_6 *, ag_rp_5(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 6: VNO AG_WRAP_6(ag_rp_6(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 7: VNO AG_WRAP_6(ag_rp_7(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *), VW(4, AG_WRAP_6 *), VW(6, AG_WRAP_6 *), VW(8, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); VWD(6, AG_WRAP_6 *); VWD(8, AG_WRAP_6 *); break;
    case 8: VRO(AG_WRAP_6 *, ag_rp_8(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 9: VNO AG_WRAP_6(ag_rp_9(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 10: VNO AG_WRAP_6(ag_rp_10(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *), VW(4, AG_WRAP_6 *), VW(6, AG_WRAP_6 *), VW(8, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); VWD(6, AG_WRAP_6 *); VWD(8, AG_WRAP_6 *); break;
    case 11: VRO(AG_WRAP_6 *, ag_rp_11(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 12: VNO AG_WRAP_6(ag_rp_12(PCB_POINTER)); break;
    case 13: VNO AG_WRAP_6(ag_rp_13(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 14: VNO AG_WRAP_6(ag_rp_14(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 15: VRO(AG_WRAP_6 *, ag_rp_15(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 16: VNO AG_WRAP_6(ag_rp_16(PCB_POINTER)); break;
    case 17: VNO AG_WRAP_6(ag_rp_17(PCB_POINTER)); break;
    case 18: VNO AG_WRAP_6(ag_rp_18(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 19: VNO AG_WRAP_6(ag_rp_19(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 20: VNO AG_WRAP_6(ag_rp_20(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 21: VNO AG_WRAP_6(ag_rp_21(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 22: VRO(AG_WRAP_6 *, ag_rp_22(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 23: VNO AG_WRAP_6(ag_rp_23(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 24: VRO(AG_WRAP_6 *, ag_rp_24(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 25: VRO(AG_WRAP_6 *, ag_rp_25(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 26: VRO(AG_WRAP_6 *, ag_rp_26(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 27: VRO(AG_WRAP_6 *, ag_rp_27(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 28: VNO AG_WRAP_6(ag_rp_28(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 29: VNO AG_WRAP_6(ag_rp_29(PCB_POINTER, V(0,(Opcode *)), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 30: VRO(AG_WRAP_6 *, ag_rp_30(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 31: VNO AG_WRAP_6(ag_rp_31(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 32: VRO(AG_WRAP_8 *, ag_rp_32(PCB_POINTER, VW(0, AG_WRAP_8 *))); break;
    case 33: VRO(AG_WRAP_6 *, ag_rp_33(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 34: VNO AG_WRAP_6(ag_rp_34(PCB_POINTER, VW(3, AG_WRAP_6 *))); 
            VWD(3, AG_WRAP_6 *); break;
    case 35: VNO AG_WRAP_6(ag_rp_35(PCB_POINTER, VW(3, AG_WRAP_6 *))); 
            VWD(3, AG_WRAP_6 *); break;
    case 36: VRO(AG_WRAP_4 *, ag_rp_36(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 37: VRO(AG_WRAP_4 *, ag_rp_37(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_7 *))); 
            VWD(2, AG_WRAP_7 *); break;
    case 38: VNO AG_WRAP_7(ag_rp_38(PCB_POINTER)); break;
    case 39: VRO(AG_WRAP_6 *, ag_rp_39(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 40: VRO(AG_WRAP_7 *, ag_rp_40(PCB_POINTER, VW(0, AG_WRAP_7 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 41: VNO AG_WRAP_8(ag_rp_41(PCB_POINTER, V(0,(long *)))); break;
    case 42: VNO AG_WRAP_8(ag_rp_42(PCB_POINTER, V(0,(double *)))); break;
    case 43: VNO AG_WRAP_8(ag_rp_43(PCB_POINTER, V(0,(int *)))); break;
    case 44: V(0,(Opcode *)) = ag_rp_44(PCB_POINTER); break;
    case 45: V(0,(Opcode *)) = ag_rp_45(PCB_POINTER); break;
    case 46: V(0,(Opcode *)) = ag_rp_46(PCB_POINTER); break;
    case 47: V(0,(Opcode *)) = ag_rp_47(PCB_POINTER); break;
    case 48: V(0,(Opcode *)) = ag_rp_48(PCB_POINTER); break;
    case 49: V(0,(Opcode *)) = ag_rp_49(PCB_POINTER); break;
    case 50: V(0,(Opcode *)) = ag_rp_50(PCB_POINTER); break;
    case 51: V(0,(Opcode *)) = ag_rp_51(PCB_POINTER); break;
    case 52: V(0,(Opcode *)) = ag_rp_52(PCB_POINTER); break;
    case 53: V(0,(Opcode *)) = ag_rp_53(PCB_POINTER); break;
    case 54: V(0,(Opcode *)) = ag_rp_54(PCB_POINTER); break;
    case 55: V(0,(Opcode *)) = ag_rp_55(PCB_POINTER); break;
    case 56: V(0,(Opcode *)) = ag_rp_56(PCB_POINTER); break;
    case 57: V(0,(Opcode *)) = ag_rp_57(PCB_POINTER); break;
    case 58: V(0,(Opcode *)) = ag_rp_58(PCB_POINTER); break;
    case 59: V(0,(Opcode *)) = ag_rp_59(PCB_POINTER); break;
    case 60: V(0,(Opcode *)) = ag_rp_60(PCB_POINTER); break;
    case 61: V(0,(Opcode *)) = ag_rp_61(PCB_POINTER); break;
    case 62: V(0,(Opcode *)) = ag_rp_62(PCB_POINTER); break;
    case 63: VNO AG_WRAP_4(ag_rp_63(PCB_POINTER, V(0,(int *)))); break;
    case 64: VRO(AG_WRAP_4 *, ag_rp_64(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 65: V(0,(double *)) = ag_rp_65(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 66: V(0,(double *)) = ag_rp_66(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 67: V(0,(double *)) = ag_rp_67(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 68: V(0,(double *)) = ag_rp_68(PCB_POINTER, V(0,(double *))); break;
    case 69: V(0,(double *)) = ag_rp_69(PCB_POINTER, V(1,(double *))); break;
    case 70: V(0,(double *)) = ag_rp_70(PCB_POINTER, V(0,(long *))); break;
    case 71: V(0,(double *)) = ag_rp_71(PCB_POINTER, V(0,(long *))); break;
    case 72: V(0,(double *)) = ag_rp_72(PCB_POINTER, V(0,(long *))); break;
    case 73: V(0,(long *)) = ag_rp_73(PCB_POINTER, V(1,(long *))); break;
    case 74: V(0,(long *)) = ag_rp_74(PCB_POINTER, V(1,(long *))); break;
    case 75: V(0,(double *)) = ag_rp_75(PCB_POINTER, V(0,(int *))); break;
    case 76: V(0,(double *)) = ag_rp_76(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 77: V(0,(long *)) = ag_rp_77(PCB_POINTER, V(0,(int *))); break;
    case 78: V(0,(long *)) = ag_rp_78(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 79: V(0,(long *)) = ag_rp_79(PCB_POINTER, V(0,(int *))); break;
    case 80: V(0,(long *)) = ag_rp_80(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 81: V(0,(long *)) = ag_rp_81(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 82: V(0,(long *)) = ag_rp_82(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 83: V(0,(long *)) = ag_rp_83(PCB_POINTER); break;
    case 84: V(0,(long *)) = ag_rp_84(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 85: V(0,(long *)) = ag_rp_85(PCB_POINTER); break;
    case 86: V(0,(long *)) = ag_rp_86(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 87: V(0,(int *)) = ag_rp_87(PCB_POINTER, V(0,(int *))); break;
    case 88: V(0,(int *)) = ag_rp_88(PCB_POINTER, V(0,(int *))); break;
    case 89: VRO(AG_WRAP_5 *, ag_rp_89(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 90: VRO(AG_WRAP_8 *, ag_rp_90(PCB_POINTER, VW(0, AG_WRAP_8 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 91: VNO AG_WRAP_5(ag_rp_91(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 92: VNO AG_WRAP_4(ag_rp_92(PCB_POINTER)); break;
    case 93: VRO(AG_WRAP_4 *, ag_rp_93(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 94: V(0,(int *)) = ag_rp_94(PCB_POINTER); break;
    case 95: V(0,(int *)) = ag_rp_95(PCB_POINTER); break;
    case 96: V(0,(int *)) = ag_rp_96(PCB_POINTER); break;
    case 97: V(0,(int *)) = ag_rp_97(PCB_POINTER); break;
    case 98: V(0,(int *)) = ag_rp_98(PCB_POINTER); break;
    case 99: V(0,(int *)) = ag_rp_99(PCB_POINTER); break;
    case 100: V(0,(int *)) = ag_rp_100(PCB_POINTER); break;
    case 101: V(0,(int *)) = ag_rp_101(PCB_POINTER); break;
    case 102: V(0,(int *)) = ag_rp_102(PCB_POINTER); break;
    case 103: V(0,(int *)) = ag_rp_103(PCB_POINTER); break;
    case 104: V(0,(int *)) = ag_rp_104(PCB_POINTER); break;
    case 105: V(0,(int *)) = ag_rp_105(PCB_POINTER, V(1,(int *))); break;
    case 106: V(0,(int *)) = ag_rp_106(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 107: V(0,(int *)) = ag_rp_107(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 108: V(0,(int *)) = ag_rp_108(PCB_POINTER, V(1,(int *))); break;
    case 109: V(0,(int *)) = ag_rp_109(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 110: V(0,(int *)) = ag_rp_110(PCB_POINTER, V(0,(int *))); break;
    case 111: V(0,(int *)) = ag_rp_111(PCB_POINTER, V(0,(int *))); break;
    case 112: V(0,(int *)) = ag_rp_112(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES dci_token_names
const char *const dci_token_names[194] = {
  "script",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "script",
  "statement list",
  "eof",
  "statement",
  "open statement",
  "closed statement",
  "if condition",
  "\"ELSE\"",
  "WHILE",
  "expression",
  "\"DO\"",
  "FOR",
  "lvalue",
  "\":=\"",
  "for increment",
  "\"TO\"",
  "simple statement",
  "\"BY\"",
  "REPEAT",
  "\"UNTIL\"",
  "';'",
  "compound statement",
  "\"RETURN\"",
  "dump statement",
  "print statement",
  "\"REPEAT\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"IF\"",
  "\"THEN\"",
  "\"BEGIN\"",
  "\"END\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "simple expression",
  "relational op",
  "term",
  "additive op",
  "unary expression",
  "multiplicative op",
  "factor",
  "'+'",
  "unary op",
  "primary",
  "\"**\"",
  "'('",
  "')'",
  "constant",
  "function call",
  "\"LONG\"",
  "\"DOUBLE\"",
  "optional arg list",
  "arg list",
  "string",
  "'='",
  "\"<>\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "'.'",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0X\"",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\A\"",
  "\"\\\\B\"",
  "\"\\\\F\"",
  "\"\\\\N\"",
  "\"\\\\R\"",
  "\"\\\\T\"",
  "\"\\\\V\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\X\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "\"ELSE\"",
  "\"DO\"",
  "\"TO\"",
  "\":=\"",
  "\"BY\"",
  "';'",
  "\"UNTIL\"",
  "\"RETURN\"",
  "\"REPEAT\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"THEN\"",
  "\"IF\"",
  "\"END\"",
  "\"BEGIN\"",
  "name",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "'+'",
  "\"**\"",
  "')'",
  "'('",
  "\"LONG\"",
  "\"DOUBLE\"",
  "integer",
  "real",
  "character constant",
  "'='",
  "\"<>\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (dci_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (dci_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_dci(dci_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void dci(dci_pcb_type *PCB_POINTER) {
  init_dci(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


