/*
 ast.syn -- Create an Abstract Syntax Tree
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 Modified to support local function definitions.

 The grammar contained in this file describes the same language as the
 grammar in pll.syn. The differences can be categorized as follows:
   . Rules rewritten to make computation easier. In pll.syn all of the
     arithmetic operators have been factored out. Here the arithmetic
     operators have been substituted back into the rules for expressions to
     make computation more straightforward.
   . Rules rewritten to handle looping. The rules for loop constructs have
     been rewritten to handle repetitive parsing of loops.
   . Semantically determined rules (true, false) added to support parse time
     implementation of loops and if-else statements.

 The actual parser modules, ast.h and ast.cpp are created from
 ast.syn by the AnaGram parser generator using the Build Parser command.

 To build a demonstration program that illustrates the use of this parser in
 an interpreter that executes from an abstract syntax tree, compile and link
 as a single single project:
   astxi.cpp               // Execution module
   demo.cpp                // Demo program
   ast.cpp                 // This parser, creates abstract syntax tree
   astdefs.cpp             // Node definitions
   comdefs.cpp             // Common script definitions
   agclib1\src\ag*.cpp     // Supporting class library
 Include files describing the class library are in agclib1\include

 To build a demonstration program that illustrates the use of this parser in
 an interpreter that compiles bytecode from an abstract syntax tree, compile
 and link as a single single project:
   astci.cpp               // Execution module
   demo.cpp                // Demo program
   ast.cpp                 // This parser, creates abstract syntax tree
   astdefs.cpp             // Node definitions
   bcidefs.cpp             // Bytecode and compiler definitions
   comdefs.cpp             // Common script definitions
   agclib1\src\ag*.cpp     // Supporting class library
 Include files describing the class library are in agclib1\include


 This grammar describes a language that is somewhat similar to Pascal.

 The language consists of Pascal like statements and expressions. The
 language also implements Fortran-style exponentiation and simple dump and
 print statements. The syntax below uses "open" and "closed" statements to
 eliminate the traditional "dangling else" or "if-else" ambiguity problem.
 See http://www.parsifalsoft.com/ifelse.html or doc/ifelse.htm for a more
 detailed discussion.

 Statement types supported are:
   assignment statements
   compound statements
   if/else statements
   while statements
   repeat/until statements
   for statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"
#include <math.h>
#include "astdefs.h"         // Defines node structure of abstract syntax tree
#include <ctype.h>


// Forward reference for parxer control block
struct parse_pcb_struct;

struct Tree : public AbstractSyntaxTree {
  parse_pcb_struct *pcbPointer;
  FileLocation context() const;
};

#define AST PCB.ast

/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 26 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef AST_H_1027348850
#include "ast.h"
#endif

#ifndef AST_H_1027348850
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE parse_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer
static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 497, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

#define GET_CONTEXT CONTEXT = FileLocation(PCB.pointer, PCB.line, PCB.column)

FileLocation Tree::context() const {
  return PCONTEXT(*pcbPointer);
}

// Function definitions for parser control block

// Constructor
parse_pcb_struct::parse_pcb_struct(const char *text)
  : pointer((const unsigned char *) text), loopDepth(0)
{
  ast.pcbPointer = this;
}

void parse_pcb_struct::checkLoop() {
  if (loopDepth) return;
  char buf[100];
  FileLocation &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): No loop active", context.line, context.column);
  throw ErrorDiagnostic(buf);
}

// Record syntax error
void parse_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

// External Interface
AbstractSyntaxTree buildTree(const char *text) {
  parse_pcb_type pcb(text);
  parse(&pcb);
  return pcb.ast;
}


#ifndef CONVERT_CASE

static const char agCaseTable[31] = {
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,    0,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20
};

static int agConvertCase(int c) {
  if (c >= 'a' && c <= 'z') return c ^= 0x20;
  if (c >= 0xe0 && c < 0xff) c ^= agCaseTable[c-0xe0];
  return c;
}

#define CONVERT_CASE(c) agConvertCase(c)

#endif


#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif

static inline void ag_rp_1(PCB_DECL, AstNode * list, AgStack<AstNode *> &sl) {
/* Line 149, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  AST.root = AST.node(typeScript, list, AST.listNode(typeStatementBlock, sl));
}

static inline AgStack<AstNode *> ag_rp_2(PCB_DECL) {
/* Line 153, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AgStack<AstNode *>();
}

static inline AgStack<AstNode *> ag_rp_3(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 154, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return s.push(x);
}

static inline AstNode * ag_rp_4(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 172, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.node(typeIfStatement, x, s);
}

static inline AstNode * ag_rp_5(PCB_DECL, AstNode * x, AstNode * s1, AstNode * s2) {
/* Line 174, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.node(typeIfElseStatement, x, s1, s2);
}

static inline AstNode * ag_rp_6(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 176, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return PCB.loopDepth--, AST.node(typeWhileStatement, x, s);
}

static inline AstNode * ag_rp_7(PCB_DECL, AstNode * lv, AstNode * b, AstNode * i, AstNode * e, AstNode * s) {
/* Line 179, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.node(typeSimpleFor, lv, b, i, e, s);
}

static inline AstNode * ag_rp_8(PCB_DECL, AstNode * x, AstNode * s1, AstNode * s2) {
/* Line 183, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.node(typeIfElseStatement,x,s1,s2);
}

static inline AstNode * ag_rp_9(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 185, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.node(typeWhileStatement,x,s);
}

static inline AstNode * ag_rp_10(PCB_DECL, AstNode * lv, AstNode * b, AstNode * i, AstNode * e, AstNode * s) {
/* Line 188, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.node(typeSimpleFor, lv, b, i, e, s);
}

static inline AstNode * ag_rp_11(PCB_DECL, AstNode * x) {
/* Line 189, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return x;
}

static inline AstNode * ag_rp_12(PCB_DECL) {
/* Line 192, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.constantNode(1);
}

static inline AstNode * ag_rp_13(PCB_DECL, AstNode * x) {
/* Line 193, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return x;
}

static inline AstNode * ag_rp_14(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 202, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.node(typeRepeatStatement,AST.listNode(typeStatementBlock, s), x);
}

static inline AstNode * ag_rp_15(PCB_DECL, AstNode * lv, AstNode * x) {
/* Line 203, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.assignmentNode(lv,SAP,x);
}

static inline AstNode * ag_rp_16(PCB_DECL) {
/* Line 204, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.node(typeNull);
}

static inline AstNode * ag_rp_17(PCB_DECL, AstNode * x) {
/* Line 206, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.node(typeReturn, x);
}

static inline AstNode * ag_rp_18(PCB_DECL, AgStack<AstNode *> &ns) {
/* Line 207, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.listNode(typeDump, ns);
}

static inline AstNode * ag_rp_19(PCB_DECL, AgStack<AstNode *> &ns) {
/* Line 208, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.listNode(typePrint, ns);
}

static inline AstNode * ag_rp_20(PCB_DECL) {
/* Line 212, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.nameListNode(AgStack<AgString>());
}

static inline AstNode * ag_rp_21(PCB_DECL, AgStack<AgString> &list) {
/* Line 213, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.nameListNode(list);
}

static inline AstNode * ag_rp_22(PCB_DECL, AstNode * x) {
/* Line 228, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return x;
}

static inline AstNode * ag_rp_23(PCB_DECL, AgStack<AstNode *> &s) {
/* Line 231, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.listNode(typeStatementBlock, s);
}

static inline AgStack<AstNode *> ag_rp_24(PCB_DECL, AgString &k) {
/* Line 235, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AgStack<AstNode *>().push(AST.variableNode(k));
}

static inline AgStack<AstNode *> ag_rp_25(PCB_DECL, AgStack<AstNode *> &ns, AgString &k) {
/* Line 236, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return ns.push(AST.variableNode(k));
}

static inline AgStack<AstNode *> ag_rp_26(PCB_DECL, AstNode * x) {
/* Line 240, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AgStack<AstNode *>().push(x);
}

static inline AgStack<AstNode *> ag_rp_27(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 241, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return s.push(x);
}

static AstNode * ag_rp_28(PCB_DECL, AgString &n, AgStack<AgString> &list, AgString &text) {
/* Line 246, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
      return AST.node(typeFunction,
                      AST.variableNode(n),
                      AST.nameListNode(list),
                      AST.constantNode(text));
    
}

static AgString ag_rp_29(PCB_DECL) {
/* Line 253, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
return AgString().append(
               (const char *) CONTEXT.pointer, PCB.pointer - CONTEXT.pointer);
 
}

static inline AgStack<AgString> ag_rp_30(PCB_DECL, AgString &n) {
/* Line 258, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AgStack<AgString>().push(n);
}

static inline AgStack<AgString> ag_rp_31(PCB_DECL, AgStack<AgString> &stack, AgString &n) {
/* Line 259, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return stack.push(n);
}

static inline AstNode * ag_rp_32(PCB_DECL) {
/* Line 262, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.node(typeNull);
}

static inline AstNode * ag_rp_33(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 269, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.binaryNode(op,x,y);
}

static inline AstNode * ag_rp_34(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 273, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.binaryNode(op,x,y);
}

static inline AstNode * ag_rp_35(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 277, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.binaryNode(op,x,y);
}

static inline AstNode * ag_rp_36(PCB_DECL, AstNode * x) {
/* Line 281, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return x;
}

static inline AstNode * ag_rp_37(PCB_DECL, Opcode op, AstNode * x) {
/* Line 282, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.unaryNode(op, x);
}

static inline AstNode * ag_rp_38(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 290, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.binaryNode(POW, x, y);
}

static inline AstNode * ag_rp_39(PCB_DECL, AstNode * x) {
/* Line 298, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return x;
}

static inline AstNode * ag_rp_40(PCB_DECL, AstNode * x) {
/* Line 300, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.unaryNode(FETCH, x);
}

static inline AstNode * ag_rp_41(PCB_DECL, AstNode * x) {
/* Line 302, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.unaryNode(CAST_LONG, x);
}

static inline AstNode * ag_rp_42(PCB_DECL, AstNode * x) {
/* Line 303, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.unaryNode(CAST_DOUBLE, x);
}

static inline AstNode * ag_rp_43(PCB_DECL, AgString &n) {
/* Line 306, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.variableNode(n);
}

static inline AstNode * ag_rp_44(PCB_DECL, AgString &k, AgStack<AstNode *> &args) {
/* Line 309, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.functionCallNode(k, args);
}

static inline AgStack<AstNode *> ag_rp_45(PCB_DECL) {
/* Line 312, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AgStack<AstNode *>();
}

static inline AgStack<AstNode *> ag_rp_46(PCB_DECL, AstNode * x) {
/* Line 316, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AgStack<AstNode *>().push(x);
}

static inline AgStack<AstNode *> ag_rp_47(PCB_DECL, AgStack<AstNode *> &al, AstNode * x) {
/* Line 317, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return al.push(x);
}

static inline AstNode * ag_rp_48(PCB_DECL, long x) {
/* Line 320, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_49(PCB_DECL, double x) {
/* Line 321, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_50(PCB_DECL, Value &x) {
/* Line 322, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_51(PCB_DECL, int x) {
/* Line 323, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AST.constantNode(x);
}

static inline Opcode ag_rp_52(PCB_DECL) {
/* Line 327, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return EQ;
}

static inline Opcode ag_rp_53(PCB_DECL) {
/* Line 328, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return NE;
}

static inline Opcode ag_rp_54(PCB_DECL) {
/* Line 329, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return LT;
}

static inline Opcode ag_rp_55(PCB_DECL) {
/* Line 330, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return LE;
}

static inline Opcode ag_rp_56(PCB_DECL) {
/* Line 331, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return GT;
}

static inline Opcode ag_rp_57(PCB_DECL) {
/* Line 332, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return GE;
}

static inline Opcode ag_rp_58(PCB_DECL) {
/* Line 335, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return ADD;
}

static inline Opcode ag_rp_59(PCB_DECL) {
/* Line 336, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return SUB;
}

static inline Opcode ag_rp_60(PCB_DECL) {
/* Line 337, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return IOR;
}

static inline Opcode ag_rp_61(PCB_DECL) {
/* Line 338, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return XOR;
}

static inline Opcode ag_rp_62(PCB_DECL) {
/* Line 341, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return MUL;
}

static inline Opcode ag_rp_63(PCB_DECL) {
/* Line 342, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return RDIV;
}

static inline Opcode ag_rp_64(PCB_DECL) {
/* Line 343, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return IDIV;
}

static inline Opcode ag_rp_65(PCB_DECL) {
/* Line 344, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return MOD;
}

static inline Opcode ag_rp_66(PCB_DECL) {
/* Line 345, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AND;
}

static inline Opcode ag_rp_67(PCB_DECL) {
/* Line 346, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return LS;
}

static inline Opcode ag_rp_68(PCB_DECL) {
/* Line 347, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return RS;
}

static inline Opcode ag_rp_69(PCB_DECL) {
/* Line 350, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return NEG;
}

static inline Opcode ag_rp_70(PCB_DECL) {
/* Line 351, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return NOT;
}

static inline AgString ag_rp_71(PCB_DECL, int c) {
/* Line 369, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AgString().append(tolower(c));
}

static inline AgString ag_rp_72(PCB_DECL, AgString &ns, int c) {
/* Line 370, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return ns.append(tolower(c));
}

static inline double ag_rp_73(PCB_DECL, double x, long e) {
/* Line 376, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return x*pow(10,e);
}

static inline double ag_rp_74(PCB_DECL, double x, long e) {
/* Line 377, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return x*pow(10,e);
}

static inline double ag_rp_75(PCB_DECL, double i, double f) {
/* Line 380, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return i+f;
}

static inline double ag_rp_76(PCB_DECL, double i) {
/* Line 381, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return i;
}

static inline double ag_rp_77(PCB_DECL, double f) {
/* Line 382, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return f;
}

static inline double ag_rp_78(PCB_DECL, long x) {
/* Line 385, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return x;
}

static inline double ag_rp_79(PCB_DECL, long x) {
/* Line 386, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return x;
}

static inline double ag_rp_80(PCB_DECL, long x) {
/* Line 387, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return makeDecimal(x);
}

static inline long ag_rp_81(PCB_DECL, long x) {
/* Line 390, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return (long)x;
}

static inline long ag_rp_82(PCB_DECL, long x) {
/* Line 391, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return -(long)x;
}

static inline double ag_rp_83(PCB_DECL, int d) {
/* Line 394, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_84(PCB_DECL, int d, double f) {
/* Line 395, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_85(PCB_DECL, int d) {
/* Line 403, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return d-'0';
}

static inline long ag_rp_86(PCB_DECL, long x, int d) {
/* Line 404, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_87(PCB_DECL, int d) {
/* Line 407, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return d-'0';
}

static inline long ag_rp_88(PCB_DECL, long x, int d) {
/* Line 408, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_89(PCB_DECL, long x, int d) {
/* Line 411, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_90(PCB_DECL, long x, int d) {
/* Line 412, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_91(PCB_DECL) {
/* Line 415, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return 0;
}

static inline long ag_rp_92(PCB_DECL, long n, int d) {
/* Line 416, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_93(PCB_DECL) {
/* Line 419, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return 0;
}

static inline long ag_rp_94(PCB_DECL, long n, int d) {
/* Line 420, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return 16*n + d;
}

static inline int ag_rp_95(PCB_DECL, int x) {
/* Line 423, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return x - '0';
}

static inline int ag_rp_96(PCB_DECL, int d) {
/* Line 424, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return (d & 7) + 9;
}

static inline Value ag_rp_97(PCB_DECL, Value &s, Value &e) {
/* Line 430, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return s+=e;
}

static inline Value ag_rp_98(PCB_DECL, AgString &b) {
/* Line 433, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return Value(b);
}

static inline AgString ag_rp_99(PCB_DECL) {
/* Line 437, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return AgString();
}

static inline AgString ag_rp_100(PCB_DECL, AgString &s, int c) {
/* Line 438, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return s.append(c);
}

static inline int ag_rp_101(PCB_DECL) {
/* Line 450, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return '\'';
}

static inline int ag_rp_102(PCB_DECL) {
/* Line 451, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return '"';
}

static inline int ag_rp_103(PCB_DECL) {
/* Line 452, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return '\?';
}

static inline int ag_rp_104(PCB_DECL) {
/* Line 453, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return '\\';
}

static inline int ag_rp_105(PCB_DECL) {
/* Line 454, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return '\a';
}

static inline int ag_rp_106(PCB_DECL) {
/* Line 455, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return '\b';
}

static inline int ag_rp_107(PCB_DECL) {
/* Line 456, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return '\f';
}

static inline int ag_rp_108(PCB_DECL) {
/* Line 457, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return '\n';
}

static inline int ag_rp_109(PCB_DECL) {
/* Line 458, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return '\r';
}

static inline int ag_rp_110(PCB_DECL) {
/* Line 459, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return '\t';
}

static inline int ag_rp_111(PCB_DECL) {
/* Line 460, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return '\v';
}

static inline int ag_rp_112(PCB_DECL, int d) {
/* Line 466, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return d-'0';
}

static inline int ag_rp_113(PCB_DECL, int n, int d) {
/* Line 469, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_114(PCB_DECL, int n, int d) {
/* Line 472, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_115(PCB_DECL, int d) {
/* Line 475, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return d;
}

static inline int ag_rp_116(PCB_DECL, int n, int d) {
/* Line 476, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return 16*n + d;
}

static inline int ag_rp_117(PCB_DECL, int d) {
/* Line 479, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return d-'0';
}

static inline int ag_rp_118(PCB_DECL, int d) {
/* Line 480, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_119(PCB_DECL, int c) {
/* Line 489, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 7, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 6, 6, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 5, 0, 0,
  0, 0, 0, 0, 0, 0, 4, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0,
  0, 6, 0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 6, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 4, 0, 6, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<AgStack<AstNode *> >
#undef AG_WRAP_7
#define AG_WRAP_7 AgObjectWrapper<AgStack<AgString> >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      case 7: ag_delete_object((AG_WRAP_7 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static parse_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  2,  3,  0,  0,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
   16,  0, 17, 18, 19,  0, 20, 21,  0,  0,  0,  0, 22, 23, 24, 25, 26, 27,
   28, 29, 30, 31, 32,  0,  0, 33,  0, 34,  0, 35,  0, 36, 37,  0, 38, 39,
    0, 40,  0, 41, 42, 43, 44, 45,  0, 46, 47, 48, 49, 50, 51, 52, 53, 54,
   55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0, 71, 72,  0, 73, 74, 75, 76, 77, 78,
   79, 80,  0,  0, 81, 82, 83, 84,  0,  0,  0, 85, 86, 87, 88, 89, 90, 91,
   92, 93, 94, 95, 96,  0, 97, 98, 99,100,  0,  0,  0,  0,  0,101,102,103,
  104,105,106,107,108,109,110,111,  0,  0,  0,112,113,114,115,116,117,118,
  119
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,156,  0,157,  0,158,  0,160,  0,162,  0,163,  0,164,  0,165,
  0,166,  0,167,  0,169,  0,168,  0,171,  0,170,  0,173,  0,175,
  0,178,  0,181,  0,182,  0,193,  0,194,  0,197,  0,198,  0,199,
  0,200,  0,201,  0,202,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 79, 85,255, 80, 84,255, 69,255, 47, 66, 68, 70, 73, 76,
   80, 82, 87,255, 42,255, 42, 47,255, 61, 62,255, 69, 89,255, 85,255, 73,
   79, 85,255, 76, 78,255, 79, 85,255, 67, 78,255, 79,255, 80, 84,255, 69,
  255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 48, 58, 60, 62, 65, 66, 68,
   69, 70, 73, 76, 77, 78, 79, 80, 82, 83, 84, 85, 87, 88,255, 79, 85,255,
   80, 84,255, 69,255, 66, 68, 70, 73, 76, 80, 82, 87,255, 42, 47,255, 47,
  255, 79, 85,255, 80, 84,255, 69,255, 66, 68, 70, 73, 80, 82, 87,255, 42,
   47,255, 61, 62,255, 42, 47, 58, 60, 62,255, 42, 47,255, 47, 48, 78,255,
   42, 47,255, 76, 78,255, 79, 85,255, 80, 84,255, 69,255, 47, 66, 68, 69,
   70, 73, 80, 82, 85, 87,255, 42, 47,255, 79, 85,255, 80, 84,255, 69,255,
   47, 66, 68, 69, 70, 73, 76, 80, 82, 87,255, 42, 47,255, 79, 85,255, 80,
   84,255, 69,255, 47, 66, 68, 70, 73, 80, 82, 85, 87,255, 48, 78,255, 79,
   85,255, 80, 84,255, 69,255, 66, 68, 69, 70, 73, 80, 82, 87,255, 58,255,
   79, 85,255, 80, 84,255, 69,255, 66, 68, 70, 73, 80, 82, 85, 87,255, 34,
   39, 63, 65, 66, 70, 78, 82, 84, 86, 88, 92,255, 92,255, 42, 47,255, 61,
   62,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 60, 62, 65,
   66, 68, 77, 79, 83, 84, 88,255, 42, 47,255, 61, 62,255, 76, 82,255, 72,
  255, 72, 79,255, 42, 47, 60, 62, 77, 79, 83, 84, 88,255, 61, 62,255, 73,
   79,255, 76, 82,255, 72,255, 72, 79,255, 42, 60, 62, 65, 66, 68, 77, 79,
   83, 84, 88,255, 42, 47,255, 47, 48, 68, 76, 78,255, 48, 68, 76, 78,255,
   61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 60, 62, 65, 66,
   68, 77, 79, 83, 84, 88,255, 61, 62,255, 72, 79,255, 60, 62, 66, 68, 79,
   84, 88,255, 84,255, 68,255, 76, 78,255, 79, 85,255, 80, 84,255, 69,255,
   66, 68, 69, 70, 73, 80, 82, 85, 87,255, 42, 47,255, 79, 85,255, 80, 84,
  255, 69,255, 47, 66, 68, 70, 73, 80, 82, 87,255, 42, 47,255, 61, 62,255,
   69, 89,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 48, 60,
   62, 65, 66, 68, 77, 79, 83, 84, 88,255, 48,255, 72, 79,255, 66, 68, 79,
   84, 88,255, 66, 84,255, 66,255, 84,255, 79, 85,255, 80, 84,255, 69,255,
   66, 68, 69, 70, 73, 76, 80, 82, 87,255, 69,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,7,7,4,7,7,4,2,4,2,7,7,2,7,7,7,2,7,4,3,4,0,0,4,0,0,4,7,5,4,7,4,
  7,6,7,4,7,7,4,7,7,4,7,7,4,2,4,7,7,4,2,4,5,5,4,2,4,7,5,4,3,2,3,3,2,3,7,
  2,2,2,2,7,2,7,7,7,7,2,2,2,7,7,7,4,7,7,4,7,7,4,2,4,7,7,2,7,7,7,2,7,4,0,
  0,4,2,4,7,7,4,7,7,4,2,4,7,7,2,7,7,2,7,4,0,0,4,0,0,4,3,2,3,2,3,4,0,0,4,
  2,3,7,4,0,0,4,7,7,4,7,7,4,7,7,4,2,4,2,7,7,2,2,7,7,2,7,7,4,0,0,4,7,7,4,
  7,7,4,2,4,2,7,7,7,2,7,7,7,2,7,4,0,0,4,7,7,4,7,7,4,2,4,2,7,7,2,7,7,2,7,
  7,4,3,7,4,7,7,4,7,7,4,2,4,7,7,7,2,7,7,2,7,4,3,4,7,7,4,7,7,4,2,4,7,7,2,
  7,7,2,7,7,4,0,0,0,0,0,0,0,0,0,0,0,0,4,2,4,0,0,4,0,0,4,7,5,4,5,5,4,2,4,
  7,5,4,3,2,2,3,7,7,2,7,7,2,2,7,4,0,0,4,0,0,4,5,5,4,2,4,7,5,4,3,2,2,3,7,
  7,2,2,7,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,3,7,7,2,7,7,2,2,7,4,0,0,4,2,
  3,7,7,7,4,3,7,7,7,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,2,3,7,7,2,7,7,2,2,7,4,
  0,0,4,7,5,4,2,3,7,7,7,2,7,4,7,4,7,4,7,7,4,7,7,4,7,7,4,2,4,7,7,2,2,7,7,
  2,7,7,4,0,0,4,7,7,4,7,7,4,2,4,2,7,7,2,7,7,2,7,4,0,0,4,0,0,4,7,5,4,7,5,
  4,5,5,4,2,4,7,5,4,3,2,3,2,3,7,2,2,7,7,2,2,7,4,3,4,7,5,4,7,7,7,2,7,4,7,
  7,4,7,4,7,4,7,7,4,7,7,4,2,4,7,7,7,2,7,7,7,2,7,4,7,4
};

static const unsigned char ag_key_parm[] = {
    0, 88, 93,  0, 18, 32,  0, 12,  8,  0,  0,  0,  0, 24, 28,  0, 20, 10,
   30,  0, 16,  0, 92,  0, 88, 93,  0,189,187,  0, 24,  6,  0, 36,  0, 42,
    2, 28,  0,  0, 26,  0, 18, 32,  0, 10, 34,  0,  0,  0, 12,  8,  0,  0,
    0, 48, 50,  0,  0,  0, 22,  4,  0,180,  0,118,159,  0,191, 46,  0,  0,
    0,  0, 20,  0, 44, 52, 38, 30,  0,  0,  0, 14, 16, 40,  0, 18, 32,  0,
   12,  8,  0,  0,  0, 24, 28,  0, 20, 10, 30,  0, 16,  0, 88, 93,  0,  0,
    0, 18, 32,  0, 12,  8,  0,  0,  0, 24, 28,  0, 20, 30,  0, 16,  0, 88,
   93,  0,189,187,  0,180,  0,159,  0,191,  0, 88, 93,  0,  0,118, 52,  0,
   88, 93,  0,  0, 26,  0, 18, 32,  0, 12,  8,  0,  0,  0,  0, 24, 28,  0,
    0, 20, 30,  0, 14, 16,  0, 88, 93,  0, 18, 32,  0, 12,  8,  0,  0,  0,
    0, 24, 28, 26,  0, 20, 10, 30,  0, 16,  0, 88, 93,  0, 18, 32,  0, 12,
    8,  0,  0,  0,  0, 24, 28,  0, 20, 30,  0, 14, 16,  0,118, 52,  0, 18,
   32,  0, 12,  8,  0,  0,  0, 24, 28, 26,  0, 20, 30,  0, 16,  0,159,  0,
   18, 32,  0, 12,  8,  0,  0,  0, 24, 28,  0, 20, 30,  0, 14, 16,  0,130,
  129,131,133,134,135,136,137,138,139,144,132,  0,  0,  0, 88, 93,  0,189,
  187,  0, 42,  2,  0, 48, 50,  0,  0,  0, 22,  4,  0,180,  0,  0,191, 46,
    6,  0, 44, 38,  0,  0, 40,  0, 88, 93,  0,189,187,  0, 48, 50,  0,  0,
    0, 22,  4,  0,180,  0,  0,191, 44, 38,  0,  0, 40,  0,189,187,  0, 42,
    2,  0, 48, 50,  0,  0,  0, 22,  4,  0,180,  0,191, 46,  6,  0, 44, 38,
    0,  0, 40,  0, 88, 93,  0,  0,118, 36, 34, 52,  0,118, 36, 34, 52,  0,
  189,187,  0, 42,  2,  0, 48, 50,  0,  0,  0, 22,  4,  0,  0,191, 46,  6,
    0, 44, 38,  0,  0, 40,  0,189,187,  0, 22,  4,  0,  0,191,  6,  2, 38,
    0, 40,  0, 22,  0,  2,  0,  0, 26,  0, 18, 32,  0, 12,  8,  0,  0,  0,
   24, 28,  0,  0, 20, 30,  0, 14, 16,  0, 88, 93,  0, 18, 32,  0, 12,  8,
    0,  0,  0,  0, 24, 28,  0, 20, 30,  0, 16,  0, 88, 93,  0,189,187,  0,
   24,  6,  0, 42,  2,  0, 48, 50,  0,  0,  0, 22,  4,  0,180,  0,118,  0,
  191, 46,  0,  0, 44, 38,  0,  0, 40,  0,118,  0, 22,  4,  0,  6,  2, 38,
    0, 40,  0,  6,  4,  0, 24,  0,  4,  0, 18, 32,  0, 12,  8,  0,  0,  0,
   24, 28, 26,  0, 20, 10, 30,  0, 16,  0, 26,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0,  9, 11,  0, 30, 34,  0,  7,  0,  1,  0,  5,  4, 18, 20,
   25, 10, 38,  0, 43,  0,  0,  0,  0,  0,  0,  0, 56,  0,  0, 62,  0, 60,
   33, 66,  0, 69, 72,  0, 74, 76,  0, 85, 88,  0, 45,  0,103,107,  0, 50,
    0,  0,  0,  0, 55,  0,111,  0,  0, 45, 24, 47, 49, 27, 51, 53, 30, 35,
   39, 42, 83, 48, 90, 93, 96, 98, 53, 58, 60,114,119,124,  0,136,138,  0,
  157,161,  0, 90,  0,127,132, 87,145,147,152, 93,165,  0,  0,  0,  0,104,
    0,179,181,  0,195,199,  0,112,  0,170,175,109,188,190,115,203,  0,  0,
    0,  0,  0,  0,  0,208,125,210,128,212,  0,  0,  0,  0,137,214,216,  0,
    0,  0,  0,228,231,  0,233,235,  0,249,253,  0,153,  0,144,219,224,147,
  150,242,244,156,257,262,  0,  0,  0,  0,279,281,  0,300,304,  0,175,  0,
  169,267,272,276,172,288,290,295,178,308,  0,  0,  0,  0,322,324,  0,338,
  342,  0,197,  0,191,313,318,194,331,333,200,346,351,  0,356,358,  0,373,
  375,  0,389,393,  0,218,  0,361,366,370,215,382,384,221,397,  0,402,  0,
  413,415,  0,429,433,  0,237,  0,404,409,234,422,424,240,437,442,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,251,  0,  0,  0,  0,  0,
    0,  0,456,  0,  0,  0,  0,  0,275,  0,463,  0,  0,447,266,269,449,451,
  454,272,458,461,278,280,466,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,302,
    0,478,  0,  0,469,296,299,471,473,476,305,307,481,  0,  0,  0,  0,493,
    0,  0,  0,  0,  0,326,  0,500,  0,  0,484,320,486,488,491,323,495,498,
  329,331,503,  0,  0,  0,  0,346,506,508,514,518,  0,521,523,529,533,  0,
    0,  0,  0,543,  0,  0,  0,  0,  0,366,  0,550,  0,  0,360,536,538,541,
  363,545,548,369,371,553,  0,  0,  0,  0,564,  0,  0,385,556,558,560,562,
  388,567,  0,570,  0,574,  0,585,588,  0,590,592,  0,606,610,  0,409,  0,
  576,581,403,406,599,601,412,614,619,  0,  0,  0,  0,633,635,  0,649,653,
    0,430,  0,424,624,629,427,642,644,433,657,  0,  0,  0,  0,  0,  0,  0,
  671,  0,  0,675,  0,  0,  0,  0,  0,456,  0,682,  0,  0,662,444,664,447,
  666,668,450,453,677,680,459,461,685,  0,688,  0,696,  0,  0,690,692,694,
  480,699,  0,702,704,  0,706,  0,711,  0,725,727,  0,746,750,  0,499,  0,
  713,718,722,496,734,736,741,502,754,  0,759,  0
};

static const unsigned short ag_key_index[] = {
   12,  0, 22, 63, 95,  0,  0, 22, 22,107,  0,117,131,  0,117,140,  0,158,
  107,140,107,180,202,  0,212,  0,140,223,107,140,140,  0,  0,212,232,242,
  212,  0,212,117,  0,264,  0,  0,264,283,  0,283,310,283,283,283,283,283,
  334,334,349,140,140,355,140,334,212,212,374,391,223,212,  0,  0,140,212,
  242,399,232,401,414,  0,264,283,  0,  0,264,264,264,  0,283,  0,  0,212,
  107,  0,107,  0,  0,140,212,140,140,140,140,140,140,140,212,140,140,212,
  140,140,140,140,140,140,212,158,  0,140,212,435,212,435,117,435,117,  0,
    0,  0,  0,  0,464,478,478,374,483,  0,489,492,283,283,212,140,212,494,
  504,140,212,223,514,401,223,117
};

static const unsigned char ag_key_ends[] = {
69,71,73,78,0, 85,77,80,0, 82,0, 78,67,84,73,79,78,0, 70,0, 
79,67,65,76,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
72,73,76,69,0, 47,0, 42,0, 88,0, 61,0, 61,0, 78,68,0, 
71,73,78,0, 86,0, 66,76,69,0, 77,80,0, 83,69,0, 68,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 65,76,0, 71,0, 79,68,0, 79,84,0, 
82,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 69,78,0, 
78,84,73,76,0, 72,73,76,69,0, 79,82,0, 69,71,73,78,0, 85,77,80,0, 
82,0, 78,67,84,73,79,78,0, 70,0, 79,67,65,76,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 72,73,76,69,0, 69,71,73,78,0, 85,77,80,0, 
82,0, 78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 
85,82,78,0, 72,73,76,69,0, 42,0, 61,0, 61,0, 88,0, 79,84,0, 
69,71,73,78,0, 85,77,80,0, 83,69,0, 68,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
78,84,73,76,0, 72,73,76,69,0, 69,71,73,78,0, 85,77,80,0, 78,68,0, 
82,0, 78,67,84,73,79,78,0, 70,0, 79,67,65,76,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 72,73,76,69,0, 69,71,73,78,0, 85,77,80,0, 
82,0, 78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 
85,82,78,0, 78,84,73,76,0, 72,73,76,69,0, 88,0, 79,84,0, 
69,71,73,78,0, 85,77,80,0, 78,68,0, 82,0, 78,67,84,73,79,78,0, 
70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 72,73,76,69,0, 
61,0, 69,71,73,78,0, 85,77,80,0, 82,0, 78,67,84,73,79,78,0, 
70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 78,84,73,76,0, 
72,73,76,69,0, 42,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 82,0, 
69,78,0, 79,82,0, 42,0, 61,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 
42,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 82,0, 69,78,0, 
79,82,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 79,84,0, 88,0, 
79,85,66,76,69,0, 79,78,71,0, 79,84,0, 61,0, 78,68,0, 89,0, 
86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 61,0, 89,0, 79,0, 
82,0, 69,78,0, 79,82,0, 72,69,78,0, 79,0, 69,71,73,78,0, 
85,77,80,0, 83,69,0, 68,0, 82,0, 78,67,84,73,79,78,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 78,84,73,76,0, 
72,73,76,69,0, 69,71,73,78,0, 85,77,80,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
72,73,76,69,0, 42,0, 88,0, 61,0, 78,68,0, 71,73,78,0, 86,0, 
79,68,0, 82,0, 69,78,0, 79,82,0, 88,0, 89,0, 79,0, 82,0, 
69,78,0, 79,82,0, 89,0, 79,0, 69,71,73,78,0, 79,0, 
69,71,73,78,0, 85,77,80,0, 78,68,0, 82,0, 78,67,84,73,79,78,0, 
70,0, 79,67,65,76,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
72,73,76,69,0, 78,68,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
   10,150,150,150,150,150,150,150,150,149, 97,149,149,149,150,150,150,150,
  150,150,150,150,150,150,150,150,150,150,150,150,150,150,149,150,121,150,
  150,150,150,146,177,176,195,179,174,192,104,196,116,151,151,151,151,151,
  151,151,115,115,150,161,188,186,190,150,150,152,152,152,152,101,152,153,
  153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
  153,150,143,150,150,153,150,152,152,152,152,101,152,153,153,153,153,153,
  153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,150,150,150,
  150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
  150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
  150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
  150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
  150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
  150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
  150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
  150,150,150,150
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (parse_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
178,175,173,171,169,167,166,164,163,162,161,153,152,149,101,97,93,88,10,0,1,
  154,155,
196,195,192,190,188,186,179,177,176,174,161,153,152,151,150,149,146,143,121,
  116,115,104,101,97,0,95,96,
196,195,192,190,188,186,179,177,176,174,161,153,152,151,150,149,146,143,121,
  116,115,104,101,97,92,0,90,91,
149,97,93,88,0,1,
178,175,173,171,169,167,166,164,163,162,161,153,152,101,10,0,7,8,35,
196,195,192,190,188,186,179,177,176,174,161,153,152,151,150,149,146,143,121,
  116,115,104,101,0,
97,0,
196,195,192,190,188,186,179,177,176,174,161,153,152,151,150,149,146,143,121,
  116,115,104,101,97,0,
92,0,
153,152,149,101,97,93,88,0,1,154,155,
153,152,101,0,4,36,172,
178,175,173,171,169,167,166,164,162,161,153,152,101,10,0,9,
196,195,192,191,190,189,188,187,186,180,179,177,176,174,161,159,153,152,151,
  149,116,115,101,97,93,88,0,1,154,155,
174,161,0,28,46,
178,175,173,171,169,167,166,164,162,161,153,152,101,10,0,4,11,12,13,14,16,
  19,20,24,26,28,29,30,32,33,34,37,39,40,41,43,45,47,48,172,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
153,152,101,0,4,172,
178,175,173,171,170,169,167,166,165,164,162,161,156,153,152,149,101,97,93,
  88,10,0,1,154,155,
153,152,149,101,97,93,88,0,1,154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
153,152,149,101,97,93,88,0,1,154,155,
178,175,173,171,170,169,167,166,164,163,162,161,153,152,149,101,97,93,88,0,
  1,154,155,
178,175,173,171,169,167,166,165,164,162,161,153,152,149,101,97,93,88,0,1,
  154,155,
153,152,101,0,4,172,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,17,20,
  49,52,54,56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,
  184,185,203,
153,152,101,0,4,172,
202,192,179,177,161,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,
  1,154,155,
178,175,173,171,170,169,167,166,164,162,161,153,152,101,0,9,
153,152,149,101,97,93,88,0,1,154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
174,161,0,28,46,
174,161,0,28,46,
202,192,179,177,161,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,17,
  20,31,49,52,54,56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,
  172,183,184,185,203,
159,0,21,
178,175,173,171,169,167,166,165,164,162,161,153,152,101,0,9,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,17,20,
  49,52,54,56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,
  184,185,203,
153,152,101,0,4,20,172,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,17,20,
  49,52,54,56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,
  184,185,203,
178,175,173,171,169,167,166,164,162,161,153,152,101,0,4,11,12,13,14,16,19,
  20,24,26,28,29,30,32,33,34,37,39,40,41,43,45,47,48,172,
177,0,49,
196,195,192,190,188,186,179,177,176,174,161,153,152,151,150,149,146,144,143,
  139,138,137,136,135,134,133,132,131,130,129,121,116,115,104,101,0,122,
151,116,115,0,111,
151,116,115,0,105,111,
196,195,192,190,188,186,179,177,176,174,161,153,152,151,150,149,144,143,139,
  138,137,136,135,134,133,132,131,130,129,121,116,115,104,101,0,125,126,
  127,128,140,141,142,147,
201,200,199,198,197,196,195,194,193,192,191,190,189,188,187,186,180,179,176,
  174,168,161,160,158,157,149,121,97,93,88,0,1,154,155,
104,101,0,
101,0,
152,151,116,115,101,0,119,
201,200,199,198,197,196,195,194,193,192,191,190,189,188,187,186,180,179,176,
  174,168,161,160,158,157,151,149,116,115,104,101,97,93,88,0,
201,200,199,198,197,196,195,194,193,192,191,190,189,188,187,186,180,179,176,
  174,168,161,160,158,157,151,149,116,115,104,101,97,93,88,0,111,
201,200,199,198,197,196,195,194,193,192,191,190,189,188,187,186,180,179,176,
  174,168,161,160,158,157,149,97,93,88,0,1,154,155,
201,200,199,198,197,196,195,194,193,192,191,190,189,188,187,186,180,179,176,
  174,168,161,160,158,157,149,97,93,88,0,1,154,155,
201,200,199,198,197,196,195,194,193,192,191,190,189,188,187,186,180,179,176,
  174,168,161,160,158,157,149,97,93,88,0,1,154,155,
177,0,49,
121,0,5,203,
202,192,182,181,179,177,176,153,152,151,149,146,121,118,116,115,104,101,97,
  93,88,0,1,154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,182,181,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,
  17,20,49,52,54,56,58,59,60,61,63,64,65,66,69,76,86,100,103,106,107,108,
  112,172,183,184,185,203,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
180,0,62,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,20,49,
  56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,184,185,
  203,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,20,49,
  56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,184,185,
  203,
201,200,199,198,197,196,195,0,57,79,80,81,82,83,84,85,
194,193,192,191,190,189,188,187,186,179,0,53,55,59,70,71,72,73,74,75,76,77,
  78,
178,175,173,171,170,169,167,166,164,162,161,153,152,101,0,4,11,12,13,14,16,
  19,20,24,26,28,29,30,32,33,34,37,39,40,41,43,44,45,47,48,172,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,17,20,
  49,52,54,56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,
  184,185,203,
153,152,101,0,4,172,
161,0,28,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,17,20,
  49,52,54,56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,
  184,185,203,
178,175,173,171,169,167,166,165,164,162,161,153,152,101,0,4,11,12,13,14,16,
  19,20,24,26,27,28,29,30,32,33,34,37,38,39,40,41,43,45,47,48,172,
168,0,42,
159,0,21,
157,0,18,
156,0,15,
153,152,101,0,4,36,172,
196,195,192,190,188,186,179,177,176,174,161,153,152,151,150,149,146,144,143,
  139,138,137,136,135,134,133,132,131,130,129,121,116,115,104,101,0,123,
  125,126,127,128,140,141,142,
151,116,115,0,105,111,
151,116,0,
152,151,116,115,101,0,145,
151,116,0,
151,116,0,
152,151,116,115,101,0,145,
146,0,
151,116,115,0,105,111,
192,179,151,116,115,0,102,109,
192,179,151,116,115,0,102,109,
202,192,179,177,176,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,17,
  20,49,52,54,56,58,59,60,61,63,64,67,68,69,76,86,100,103,106,107,108,112,
  172,183,184,185,203,
176,149,97,93,88,0,1,154,155,
176,0,50,
176,149,97,93,88,0,1,154,155,
176,0,50,
176,0,50,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,20,49,
  56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,184,185,
  203,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,20,49,
  56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,184,185,
  203,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,20,49,
  54,56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,184,
  185,203,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,20,49,
  52,54,56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,184,
  185,203,
178,175,173,171,170,169,167,166,165,164,162,161,156,153,152,149,101,97,93,
  88,10,0,1,154,155,
161,0,28,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,17,20,
  49,52,54,56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,
  184,185,203,
178,175,173,171,169,167,166,164,162,161,153,152,149,101,97,93,88,0,1,154,
  155,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,17,20,
  49,52,54,56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,
  184,185,203,
178,175,173,171,169,167,166,164,162,161,153,152,149,101,97,93,88,0,1,154,
  155,
178,175,173,171,169,167,166,164,162,161,153,152,101,0,4,12,13,14,16,19,20,
  24,26,28,29,30,32,33,34,37,39,40,41,43,45,47,48,172,
178,175,173,171,169,167,166,164,162,161,153,152,149,101,97,93,88,0,1,154,
  155,
178,175,173,171,169,167,166,164,162,161,153,152,101,0,4,12,13,14,16,19,20,
  24,26,28,29,30,32,33,34,37,39,40,41,43,45,47,48,172,
176,174,0,46,50,
151,116,115,0,110,
151,116,115,0,110,
174,0,46,
176,0,50,
201,200,199,198,197,196,195,194,193,192,191,190,189,188,187,186,180,179,177,
  176,174,171,168,161,160,158,157,153,152,151,149,146,121,118,116,115,104,
  101,97,93,88,0,1,154,155,
177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,20,49,61,63,64,69,
  100,103,106,107,108,112,172,183,184,185,203,
177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,20,49,61,63,64,69,
  100,103,106,107,108,112,172,183,184,185,203,
201,200,199,198,197,196,195,0,57,79,80,81,82,83,84,85,
194,193,192,179,0,55,59,76,77,78,
161,0,28,
160,158,0,22,25,
171,0,43,
151,116,115,0,111,
151,116,115,0,111,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,17,20,
  49,52,54,56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,
  184,185,203,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,17,20,
  49,52,54,56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,
  184,185,203,
158,0,23,
178,175,173,171,170,169,167,166,164,163,162,161,153,152,101,0,8,35,51,
202,192,179,177,153,152,151,149,146,121,118,116,115,104,101,97,93,88,0,1,
  154,155,
202,192,179,177,153,152,151,146,121,118,116,115,104,101,0,2,3,4,5,6,17,20,
  49,52,54,56,58,59,60,61,63,64,69,76,86,100,103,106,107,108,112,172,183,
  184,185,203,
178,175,173,171,170,169,167,166,164,162,161,153,152,101,0,9,
170,0,44,
157,0,18,
178,175,173,171,169,167,166,164,162,161,153,152,101,0,4,11,12,13,14,16,19,
  20,24,26,28,29,30,32,33,34,37,39,40,41,43,45,47,48,172,
178,175,173,171,169,167,166,164,162,161,153,152,101,0,4,12,13,14,16,19,20,
  24,26,28,29,30,32,33,34,37,39,40,41,43,45,47,48,172,

};


static unsigned const char ag_astt[3050] = {
  8,8,8,8,8,8,8,8,8,8,8,8,8,1,8,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  8,7,1,1,9,9,1,1,5,3,4,4,4,4,4,4,4,4,1,4,4,4,4,4,4,7,0,1,1,9,9,9,9,9,9,9,9,
  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,9,5,3,7,5,5,1,5,1,1,1,7,1,1,3,2,2,2,7,2,1,1,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,10,10,1,10,10,10,1,1,1,7,1,1,
  3,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,7,2,2,2,2,1,1,1,1,2,1,2,2,1,1,1,2,
  1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,2,1,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,
  1,1,3,2,2,2,7,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,2,1,1,1,1,1,1,
  1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,5,1,5,1,1,1,7,1,1,3,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,1,1,7,2,1,1,1,7,2,1,1,1,1,1,4,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,
  2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,7,1,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,7,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,
  1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,1,2,1,2,2,1,1,1,2,1,1,1,1,1,1,1,1,1,
  1,7,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,
  2,2,2,2,2,2,2,1,1,1,1,1,7,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,1,5,2,2,2,2,2,5,2,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,2,4,4,5,5,5,7,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,10,10,4,4,5,5,5,7,2,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,1,4,1,1,4,2,1,5,5,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,
  2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,
  5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,5,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,
  1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,1,2,7,2,2,1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,5,1,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,5,1,1,2,2,2,2,2,2,2,2,2,2,1,1,1,
  1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,2,1,1,1,1,2,1,2,2,1,1,1,2,1,1,1,1,1,2,1,1,1,
  1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,1,7,2,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,
  7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,2,1,1,1,1,2,
  1,1,2,2,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,7,2,1,7,1,1,7,1,1,5,1,2,2,2,7,2,1,1,
  10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,1,1,2,2,2,2,2,2,2,2,2,2,
  2,2,10,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,2,1,2,2,7,2,2,2,2,2,7,2,2,2,5,2,
  2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,8,7,2,1,1,1,8,8,8,7,2,1,1,1,1,1,
  4,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,5,1,1,1,1,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,1,1,7,2,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,
  1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,
  2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,
  5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,
  1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,
  5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,
  1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,
  1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,1,7,2,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,
  1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,
  5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,1,1,
  1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,
  1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,2,1,1,
  1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,
  1,2,2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,7,1,1,2,2,2,
  7,1,2,2,2,7,1,1,5,1,1,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,
  2,2,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,
  2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,2,2,2,2,2,2,2,1,1,1,1,4,1,
  2,2,2,2,1,7,2,1,4,7,1,1,1,7,1,10,10,10,4,2,10,10,10,4,2,1,1,1,1,2,2,2,1,1,
  2,2,2,1,2,7,2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,
  2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,7,1,4,4,4,
  4,4,4,4,4,4,1,4,4,4,4,4,7,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,2,2,1,
  1,1,1,1,1,1,1,1,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,7,2,1,7,1,1,1,1,1,1,
  1,1,1,1,1,2,2,2,4,2,2,2,2,1,1,1,1,2,1,2,2,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,2,1,1,1,1,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
4,4,4,4,4,4,4,4,4,4,4,4,4,3,4,3,1,2,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
276,276,1,2,278,276,
24,24,24,24,24,24,24,24,9,24,24,24,24,24,24,4,0,11,10,
95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,95,97,
98,6,
90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,92,
93,8,
277,277,3,277,3,1,2,9,3,3,286,
99,99,99,10,38,13,12,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,11,14,
277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,100,100,100,
  3,100,100,100,3,1,2,12,3,3,295,
15,17,13,25,16,
18,19,20,21,30,28,29,22,26,17,99,99,99,1,14,59,3,3,3,39,38,37,34,13,35,18,
  13,33,32,31,13,35,38,37,36,27,25,24,23,12,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,15,3,3,297,
99,99,99,16,39,12,
277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,3,277,3,1,2,277,
  17,3,3,284,
277,277,3,277,3,1,2,18,3,3,301,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,19,3,3,298,
277,277,3,277,3,1,2,20,3,3,296,
277,277,277,277,277,277,277,277,277,277,277,277,277,277,3,277,3,1,2,21,3,3,
  294,
277,277,277,277,277,277,277,277,277,277,277,277,277,3,277,3,1,2,22,3,3,287,
99,99,99,23,40,12,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,24,65,66,54,55,68,34,55,59,65,
  64,64,64,63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
99,99,99,25,32,12,
277,277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,26,3,3,
  285,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,27,66,
277,277,3,277,3,1,2,28,3,3,290,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,29,3,3,289,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,30,3,3,292,
15,17,31,22,67,
15,17,32,21,68,
57,58,60,56,40,99,99,119,44,41,127,125,119,43,99,33,65,66,54,55,68,69,55,69,
  59,65,64,64,64,63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
70,34,71,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,35,72,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,36,65,66,54,55,68,73,55,59,65,
  64,64,64,63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
99,99,99,37,59,74,12,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,38,65,66,54,55,68,75,55,59,65,
  64,64,64,63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
18,19,20,21,30,28,29,22,26,17,99,99,99,39,59,6,6,76,39,38,37,34,13,35,18,13,
  33,32,31,13,35,38,37,36,27,25,24,23,12,
56,40,77,
134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,
  134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,134,41,78,
124,124,124,108,124,
79,79,79,43,106,79,
85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,81,80,151,150,149,148,147,
  146,145,144,143,142,141,85,85,85,85,85,44,85,85,85,84,83,82,85,85,
277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,
  277,277,277,277,277,277,3,277,3,1,2,45,3,3,326,
86,87,46,
88,101,
130,129,129,129,130,118,128,
117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,117,
  117,117,117,117,117,117,126,117,126,123,109,109,117,117,117,49,
116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,
  116,116,116,116,116,116,120,116,120,120,107,107,116,116,116,50,120,
277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,
  277,277,277,277,277,277,3,3,1,2,51,3,3,308,
277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,
  277,277,277,277,277,277,3,3,1,2,52,3,3,307,
277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,
  277,277,277,277,277,277,3,3,1,2,53,3,3,306,
56,59,89,
41,67,132,45,
277,277,277,277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,
  56,3,3,300,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,57,3,3,325,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,58,3,3,315,
57,58,90,92,60,56,99,99,119,44,41,127,125,119,43,99,59,65,66,54,55,68,94,55,
  59,65,64,64,64,63,62,61,61,61,93,91,55,86,87,47,46,50,42,49,48,12,53,52,
  51,45,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,60,3,3,302,
95,51,96,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,62,65,66,54,55,68,55,59,50,50,
  63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,63,65,66,54,55,68,55,59,49,49,
  63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
97,98,99,100,101,102,103,44,104,79,80,81,82,83,84,85,
105,106,58,108,109,110,111,112,113,60,42,114,107,75,69,70,71,72,73,74,76,77,
  78,
18,19,20,21,115,30,28,29,22,26,17,99,99,99,66,59,3,3,3,39,38,37,34,13,35,18,
  13,33,32,31,13,35,38,37,36,27,31,25,24,23,12,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,67,65,66,54,55,68,35,55,59,65,
  64,64,64,63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
99,99,99,68,33,12,
17,69,20,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,70,3,3,282,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,71,65,66,54,55,68,116,55,59,
  65,64,64,64,63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
18,19,20,21,30,28,29,117,22,26,17,99,99,99,72,59,3,3,3,39,38,37,34,13,35,
  118,18,13,33,32,31,13,35,118,38,37,36,27,25,24,23,12,
119,73,30,
70,74,120,
121,75,122,
123,5,124,
99,99,99,77,38,125,12,
135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,135,81,80,
  151,150,149,148,147,146,145,144,143,142,141,133,135,135,135,135,78,135,
  135,135,135,84,83,82,135,
79,79,79,114,115,79,
155,155,80,
161,160,160,160,161,81,158,
157,157,153,
156,156,152,
161,160,160,160,161,140,159,
162,85,
79,79,79,105,104,79,
126,127,127,127,127,87,103,127,
126,127,127,127,127,88,102,127,
57,58,60,56,61,99,99,119,44,41,127,125,119,43,99,89,65,66,54,55,68,63,55,59,
  65,64,64,64,63,62,61,61,61,129,128,55,86,87,47,46,50,42,49,48,12,53,52,
  51,45,
277,3,3,1,2,90,3,3,305,
130,91,131,
277,3,3,1,2,92,3,3,304,
130,93,132,
130,94,53,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,95,3,3,303,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,96,65,66,54,55,68,55,59,52,52,
  63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,97,3,3,324,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,98,3,3,323,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,99,3,3,322,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,100,3,3,321,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,101,3,3,320,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,102,3,3,319,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,103,3,3,318,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,104,65,66,54,55,68,55,59,47,
  47,63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,105,3,3,317,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,106,3,3,316,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,107,65,66,54,55,68,55,59,133,
  133,133,63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,108,3,3,314,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,109,3,3,313,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,110,3,3,312,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,111,3,3,311,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,112,3,3,310,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,113,3,3,309,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,114,65,66,54,55,68,55,59,134,
  64,64,64,63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,3,277,3,1,2,277,
  115,3,3,293,
17,116,17,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,117,3,3,288,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,118,65,66,54,55,68,135,55,59,
  65,64,64,64,63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
277,277,277,277,277,277,277,277,277,277,277,277,3,277,3,1,2,119,3,3,291,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,120,65,66,54,55,68,136,55,59,
  65,64,64,64,63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
277,277,277,277,277,277,277,277,277,277,277,277,3,277,3,1,2,121,3,3,280,
18,19,20,21,30,28,29,22,26,17,99,99,99,122,59,8,11,39,38,37,34,13,35,18,13,
  33,32,31,13,35,38,37,36,27,25,24,23,12,
277,277,277,277,277,277,277,277,277,277,277,277,3,277,3,1,2,123,3,3,279,
18,19,20,21,30,28,29,22,26,17,99,99,99,124,59,7,10,39,38,37,34,13,35,18,13,
  33,32,31,13,35,38,37,36,27,25,24,23,12,
130,15,125,16,137,
121,121,121,126,138,
121,121,121,127,139,
15,62,140,
130,129,60,
277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,
  277,277,277,277,277,277,277,277,277,277,277,3,277,277,277,277,277,277,
  277,3,1,2,130,3,3,299,
56,99,99,119,44,41,127,125,119,43,99,131,65,66,54,55,68,55,59,58,58,58,55,
  47,46,50,42,49,48,12,53,52,51,45,
56,99,99,119,44,41,127,125,119,43,99,132,65,66,54,55,68,55,59,57,57,57,55,
  47,46,50,42,49,48,12,53,52,51,45,
97,98,99,100,101,102,103,45,104,79,80,81,82,83,84,85,
105,106,58,60,43,107,75,76,77,78,
17,135,16,
141,14,136,143,142,
21,137,144,
122,122,122,113,122,
122,122,122,112,122,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,140,65,66,54,55,68,64,55,59,
  65,64,64,64,63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,141,3,3,283,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,142,65,66,54,55,68,15,55,59,
  65,64,64,64,63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
145,143,146,
24,24,24,24,24,24,24,24,24,9,24,24,24,24,24,144,147,10,148,
277,277,277,277,277,277,277,3,277,277,277,277,277,277,277,3,1,2,145,3,3,281,
57,58,60,56,99,99,119,44,41,127,125,119,43,99,146,65,66,54,55,68,149,55,59,
  65,64,64,64,63,62,61,61,61,55,86,87,47,46,50,42,49,48,12,53,52,51,45,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,147,150,
115,148,36,
121,149,151,
18,19,20,21,30,28,29,22,26,17,99,99,99,37,59,3,3,3,39,38,37,34,13,35,18,13,
  33,32,31,13,35,38,37,36,27,25,24,23,12,
18,19,20,21,30,28,29,22,26,17,99,99,99,151,59,9,12,39,38,37,34,13,35,18,13,
  33,32,31,13,35,38,37,36,27,25,24,23,12,

};


static const unsigned short ag_sbt[] = {
     0,  23,  50,  78,  84, 103, 127, 129, 154, 156, 167, 174, 190, 220,
   225, 265, 287, 293, 318, 329, 351, 362, 385, 407, 413, 459, 465, 488,
   504, 515, 537, 559, 564, 569, 617, 620, 636, 682, 689, 735, 774, 777,
   814, 819, 825, 868, 902, 905, 907, 914, 949, 985,1018,1051,1084,1087,
  1091,1116,1138,1160,1210,1232,1235,1278,1321,1337,1360,1401,1447,1453,
  1456,1478,1524,1566,1569,1572,1575,1578,1585,1629,1635,1638,1645,1648,
  1651,1658,1660,1666,1674,1682,1731,1740,1743,1752,1755,1758,1780,1823,
  1845,1867,1889,1911,1933,1955,1977,2020,2042,2064,2108,2130,2152,2174,
  2196,2218,2240,2285,2310,2313,2335,2381,2402,2448,2469,2507,2528,2566,
  2571,2576,2581,2584,2587,2632,2666,2700,2716,2726,2729,2734,2737,2742,
  2747,2793,2815,2861,2864,2883,2905,2951,2967,2970,2973,3012,3050
};


static const unsigned short ag_sbe[] = {
    19,  47,  75,  82,  99, 126, 128, 153, 155, 163, 170, 188, 216, 222,
   239, 283, 290, 314, 325, 347, 358, 381, 403, 410, 427, 462, 484, 502,
   511, 533, 555, 561, 566, 584, 618, 634, 650, 685, 703, 748, 775, 812,
   817, 822, 859, 898, 904, 906, 912, 948, 983,1014,1047,1080,1085,1088,
  1112,1134,1156,1176,1228,1233,1249,1292,1328,1347,1374,1415,1450,1454,
  1474,1492,1538,1567,1570,1573,1576,1581,1620,1632,1637,1643,1647,1650,
  1656,1659,1663,1671,1679,1697,1736,1741,1748,1753,1756,1776,1794,1841,
  1863,1885,1907,1929,1951,1973,1991,2038,2060,2078,2126,2148,2170,2192,
  2214,2236,2254,2306,2311,2331,2349,2398,2416,2465,2482,2524,2541,2568,
  2574,2579,2582,2585,2628,2643,2677,2707,2720,2727,2731,2735,2740,2745,
  2761,2811,2829,2862,2879,2901,2919,2965,2968,2971,2986,3025,3050
};


static const unsigned char ag_fl[] = {
  2,3,0,2,1,1,2,4,4,9,4,4,9,1,0,2,5,4,1,1,3,2,2,1,0,3,1,1,1,1,3,3,2,3,2,
  3,8,2,1,3,0,1,1,3,1,3,1,3,1,2,2,1,3,3,1,1,1,4,4,1,4,0,1,1,3,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,1,2,1,3,3,3,
  2,2,1,1,1,0,1,2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,2,1,1,1,2,3,0,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2
};

static const unsigned char ag_ptt[] = {
    0,  7,  9,  9, 11, 11, 12, 12, 12, 12, 13, 13, 13, 13, 22, 22, 24, 24,
   24, 24, 24, 24, 24, 24,  8,  8, 26, 27, 16, 19, 14, 29, 32, 32, 33, 33,
   34, 51, 36, 36, 31, 31, 17, 17, 52, 52, 54, 54, 56, 56, 56, 58, 58, 61,
   61, 61, 61, 61, 61, 20, 64, 67, 67, 68, 68, 63, 63, 63, 63, 53, 53, 53,
   53, 53, 53, 55, 55, 55, 55, 57, 57, 57, 57, 57, 57, 57, 60, 60,  1, 90,
   90, 91, 91,  1, 95, 95, 96, 96,  1,172,172,184,184,184,100,100,100,103,
  103,103,109,109,102,102,105,105,183,183,183,106,106,110,110,107,107,108,
  108,112,112,119,119, 69, 69,203,122,122,123,123,125,125,125,126,126,126,
  126,126,126,126,126,126,126,126,127,127,127,140,141,142,128,128,145,145,
  185,147,147,111, 87, 87, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89,
   89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 94, 94, 94, 94, 94, 94,
   94, 94, 94, 94, 94, 94, 94, 94, 94, 94, 94, 94, 94, 94, 94, 94, 94, 98,
   98, 98, 99, 99, 99, 99, 99, 99,113,113,114,114,114,117,117,120,120,124,
  124,124,124,124,124,124,124,124,124,124,124,124,124,124,124,124,124,124,
  124,124,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,
  148,148,148,148,148,154,154,155,155, 15, 18, 23, 21, 25, 28, 30, 35, 37,
   38, 39, 40, 42, 41, 44, 43,  4, 45, 46, 47, 50, 49, 48, 59, 62, 65, 66,
    2,  3,  6, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84,
   85, 86,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: ag_rp_1(PCB_POINTER, V(0,(AstNode * *)), VW(1, AG_WRAP_6 *)); 
            VWD(1, AG_WRAP_6 *); break;
    case 2: VNO AG_WRAP_6(ag_rp_2(PCB_POINTER)); break;
    case 3: VRO(AG_WRAP_6 *, ag_rp_3(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(AstNode * *)))); break;
    case 4: V(0,(AstNode * *)) = ag_rp_4(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *))); break;
    case 5: V(0,(AstNode * *)) = ag_rp_5(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 6: V(0,(AstNode * *)) = ag_rp_6(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 7: V(0,(AstNode * *)) = ag_rp_7(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *)), V(4,(AstNode * *)), V(6,(AstNode * *)), V(8,(AstNode * *))); break;
    case 8: V(0,(AstNode * *)) = ag_rp_8(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 9: V(0,(AstNode * *)) = ag_rp_9(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 10: V(0,(AstNode * *)) = ag_rp_10(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *)), V(4,(AstNode * *)), V(6,(AstNode * *)), V(8,(AstNode * *))); break;
    case 11: V(0,(AstNode * *)) = ag_rp_11(PCB_POINTER, V(0,(AstNode * *))); break;
    case 12: V(0,(AstNode * *)) = ag_rp_12(PCB_POINTER); break;
    case 13: V(0,(AstNode * *)) = ag_rp_13(PCB_POINTER, V(1,(AstNode * *))); break;
    case 14: V(0,(AstNode * *)) = ag_rp_14(PCB_POINTER, VW(1, AG_WRAP_6 *), V(3,(AstNode * *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 15: V(0,(AstNode * *)) = ag_rp_15(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 16: V(0,(AstNode * *)) = ag_rp_16(PCB_POINTER); break;
    case 17: V(0,(AstNode * *)) = ag_rp_17(PCB_POINTER, V(1,(AstNode * *))); break;
    case 18: V(0,(AstNode * *)) = VDO(AG_WRAP_6 *, ag_rp_18(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 19: V(0,(AstNode * *)) = VDO(AG_WRAP_6 *, ag_rp_19(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 20: V(0,(AstNode * *)) = ag_rp_20(PCB_POINTER); break;
    case 21: V(0,(AstNode * *)) = ag_rp_21(PCB_POINTER, VW(1, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_7 *); break;
    case 22: V(0,(AstNode * *)) = ag_rp_22(PCB_POINTER, V(1,(AstNode * *))); break;
    case 23: V(0,(AstNode * *)) = ag_rp_23(PCB_POINTER, VW(1, AG_WRAP_6 *)); 
            VWD(1, AG_WRAP_6 *); break;
    case 24: VNO AG_WRAP_6(ag_rp_24(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 25: VRO(AG_WRAP_6 *, ag_rp_25(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 26: VNO AG_WRAP_6(ag_rp_26(PCB_POINTER, V(1,(AstNode * *)))); break;
    case 27: VRO(AG_WRAP_6 *, ag_rp_27(PCB_POINTER, VW(0, AG_WRAP_6 *), V(2,(AstNode * *)))); break;
    case 28: V(0,(AstNode * *)) = ag_rp_28(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_7 *), VW(6, AG_WRAP_4 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_7 *); VWD(6, AG_WRAP_4 *); break;
    case 29: VNO AG_WRAP_4(ag_rp_29(PCB_POINTER)); break;
    case 30: VRO(AG_WRAP_4 *, ag_rp_30(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 31: VRO(AG_WRAP_7 *, ag_rp_31(PCB_POINTER, VW(0, AG_WRAP_7 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 32: V(0,(AstNode * *)) = ag_rp_32(PCB_POINTER); break;
    case 33: V(0,(AstNode * *)) = ag_rp_33(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 34: V(0,(AstNode * *)) = ag_rp_34(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 35: V(0,(AstNode * *)) = ag_rp_35(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 36: V(0,(AstNode * *)) = ag_rp_36(PCB_POINTER, V(1,(AstNode * *))); break;
    case 37: V(0,(AstNode * *)) = ag_rp_37(PCB_POINTER, V(0,(Opcode *)), V(1,(AstNode * *))); break;
    case 38: V(0,(AstNode * *)) = ag_rp_38(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 39: V(0,(AstNode * *)) = ag_rp_39(PCB_POINTER, V(1,(AstNode * *))); break;
    case 40: V(0,(AstNode * *)) = ag_rp_40(PCB_POINTER, V(0,(AstNode * *))); break;
    case 41: V(0,(AstNode * *)) = ag_rp_41(PCB_POINTER, V(3,(AstNode * *))); break;
    case 42: V(0,(AstNode * *)) = ag_rp_42(PCB_POINTER, V(3,(AstNode * *))); break;
    case 43: V(0,(AstNode * *)) = VDO(AG_WRAP_4 *, ag_rp_43(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 44: V(0,(AstNode * *)) = VDO(AG_WRAP_4 *, ag_rp_44(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 45: VNO AG_WRAP_6(ag_rp_45(PCB_POINTER)); break;
    case 46: VNO AG_WRAP_6(ag_rp_46(PCB_POINTER, V(0,(AstNode * *)))); break;
    case 47: VRO(AG_WRAP_6 *, ag_rp_47(PCB_POINTER, VW(0, AG_WRAP_6 *), V(2,(AstNode * *)))); break;
    case 48: V(0,(AstNode * *)) = ag_rp_48(PCB_POINTER, V(0,(long *))); break;
    case 49: V(0,(AstNode * *)) = ag_rp_49(PCB_POINTER, V(0,(double *))); break;
    case 50: V(0,(AstNode * *)) = VDO(AG_WRAP_5 *, ag_rp_50(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 51: V(0,(AstNode * *)) = ag_rp_51(PCB_POINTER, V(0,(int *))); break;
    case 52: V(0,(Opcode *)) = ag_rp_52(PCB_POINTER); break;
    case 53: V(0,(Opcode *)) = ag_rp_53(PCB_POINTER); break;
    case 54: V(0,(Opcode *)) = ag_rp_54(PCB_POINTER); break;
    case 55: V(0,(Opcode *)) = ag_rp_55(PCB_POINTER); break;
    case 56: V(0,(Opcode *)) = ag_rp_56(PCB_POINTER); break;
    case 57: V(0,(Opcode *)) = ag_rp_57(PCB_POINTER); break;
    case 58: V(0,(Opcode *)) = ag_rp_58(PCB_POINTER); break;
    case 59: V(0,(Opcode *)) = ag_rp_59(PCB_POINTER); break;
    case 60: V(0,(Opcode *)) = ag_rp_60(PCB_POINTER); break;
    case 61: V(0,(Opcode *)) = ag_rp_61(PCB_POINTER); break;
    case 62: V(0,(Opcode *)) = ag_rp_62(PCB_POINTER); break;
    case 63: V(0,(Opcode *)) = ag_rp_63(PCB_POINTER); break;
    case 64: V(0,(Opcode *)) = ag_rp_64(PCB_POINTER); break;
    case 65: V(0,(Opcode *)) = ag_rp_65(PCB_POINTER); break;
    case 66: V(0,(Opcode *)) = ag_rp_66(PCB_POINTER); break;
    case 67: V(0,(Opcode *)) = ag_rp_67(PCB_POINTER); break;
    case 68: V(0,(Opcode *)) = ag_rp_68(PCB_POINTER); break;
    case 69: V(0,(Opcode *)) = ag_rp_69(PCB_POINTER); break;
    case 70: V(0,(Opcode *)) = ag_rp_70(PCB_POINTER); break;
    case 71: VNO AG_WRAP_4(ag_rp_71(PCB_POINTER, V(0,(int *)))); break;
    case 72: VRO(AG_WRAP_4 *, ag_rp_72(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 73: V(0,(double *)) = ag_rp_73(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 74: V(0,(double *)) = ag_rp_74(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 75: V(0,(double *)) = ag_rp_75(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 76: V(0,(double *)) = ag_rp_76(PCB_POINTER, V(0,(double *))); break;
    case 77: V(0,(double *)) = ag_rp_77(PCB_POINTER, V(1,(double *))); break;
    case 78: V(0,(double *)) = ag_rp_78(PCB_POINTER, V(0,(long *))); break;
    case 79: V(0,(double *)) = ag_rp_79(PCB_POINTER, V(0,(long *))); break;
    case 80: V(0,(double *)) = ag_rp_80(PCB_POINTER, V(0,(long *))); break;
    case 81: V(0,(long *)) = ag_rp_81(PCB_POINTER, V(1,(long *))); break;
    case 82: V(0,(long *)) = ag_rp_82(PCB_POINTER, V(1,(long *))); break;
    case 83: V(0,(double *)) = ag_rp_83(PCB_POINTER, V(0,(int *))); break;
    case 84: V(0,(double *)) = ag_rp_84(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 85: V(0,(long *)) = ag_rp_85(PCB_POINTER, V(0,(int *))); break;
    case 86: V(0,(long *)) = ag_rp_86(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 87: V(0,(long *)) = ag_rp_87(PCB_POINTER, V(0,(int *))); break;
    case 88: V(0,(long *)) = ag_rp_88(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 89: V(0,(long *)) = ag_rp_89(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 90: V(0,(long *)) = ag_rp_90(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 91: V(0,(long *)) = ag_rp_91(PCB_POINTER); break;
    case 92: V(0,(long *)) = ag_rp_92(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 93: V(0,(long *)) = ag_rp_93(PCB_POINTER); break;
    case 94: V(0,(long *)) = ag_rp_94(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 95: V(0,(int *)) = ag_rp_95(PCB_POINTER, V(0,(int *))); break;
    case 96: V(0,(int *)) = ag_rp_96(PCB_POINTER, V(0,(int *))); break;
    case 97: VRO(AG_WRAP_5 *, ag_rp_97(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 98: VNO AG_WRAP_5(ag_rp_98(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 99: VNO AG_WRAP_4(ag_rp_99(PCB_POINTER)); break;
    case 100: VRO(AG_WRAP_4 *, ag_rp_100(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 101: V(0,(int *)) = ag_rp_101(PCB_POINTER); break;
    case 102: V(0,(int *)) = ag_rp_102(PCB_POINTER); break;
    case 103: V(0,(int *)) = ag_rp_103(PCB_POINTER); break;
    case 104: V(0,(int *)) = ag_rp_104(PCB_POINTER); break;
    case 105: V(0,(int *)) = ag_rp_105(PCB_POINTER); break;
    case 106: V(0,(int *)) = ag_rp_106(PCB_POINTER); break;
    case 107: V(0,(int *)) = ag_rp_107(PCB_POINTER); break;
    case 108: V(0,(int *)) = ag_rp_108(PCB_POINTER); break;
    case 109: V(0,(int *)) = ag_rp_109(PCB_POINTER); break;
    case 110: V(0,(int *)) = ag_rp_110(PCB_POINTER); break;
    case 111: V(0,(int *)) = ag_rp_111(PCB_POINTER); break;
    case 112: V(0,(int *)) = ag_rp_112(PCB_POINTER, V(1,(int *))); break;
    case 113: V(0,(int *)) = ag_rp_113(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 114: V(0,(int *)) = ag_rp_114(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 115: V(0,(int *)) = ag_rp_115(PCB_POINTER, V(1,(int *))); break;
    case 116: V(0,(int *)) = ag_rp_116(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 117: V(0,(int *)) = ag_rp_117(PCB_POINTER, V(0,(int *))); break;
    case 118: V(0,(int *)) = ag_rp_118(PCB_POINTER, V(0,(int *))); break;
    case 119: V(0,(int *)) = ag_rp_119(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES parse_token_names
const char *const parse_token_names[204] = {
  "script",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "script",
  "local statement",
  "statement list",
  "eof",
  "statement",
  "open statement",
  "closed statement",
  "if condition",
  "\"ELSE\"",
  "WHILE",
  "expression",
  "\"DO\"",
  "FOR",
  "lvalue",
  "\":=\"",
  "for increment",
  "\"TO\"",
  "simple statement",
  "\"BY\"",
  "REPEAT",
  "UNTIL",
  "';'",
  "compound statement",
  "\"RETURN\"",
  "optional expression",
  "dump statement",
  "print statement",
  "function definition",
  "\"LOCAL\"",
  "name list",
  "\"REPEAT\"",
  "\"UNTIL\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"IF\"",
  "\"THEN\"",
  "\"BEGIN\"",
  "\"END\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "\"FUNCTION\"",
  "'('",
  "')'",
  "function body",
  "simple expression",
  "relational op",
  "term",
  "additive op",
  "unary expression",
  "multiplicative op",
  "factor",
  "'+'",
  "unary op",
  "primary",
  "\"**\"",
  "constant",
  "function call",
  "\"LONG\"",
  "\"DOUBLE\"",
  "optional arg list",
  "arg list",
  "string",
  "'='",
  "\"<>\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "'.'",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0X\"",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\A\"",
  "\"\\\\B\"",
  "\"\\\\F\"",
  "\"\\\\N\"",
  "\"\\\\R\"",
  "\"\\\\T\"",
  "\"\\\\V\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\X\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "\"ELSE\"",
  "\"DO\"",
  "\"TO\"",
  "\":=\"",
  "\"BY\"",
  "';'",
  "\"RETURN\"",
  "\"LOCAL\"",
  "\"REPEAT\"",
  "\"UNTIL\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"THEN\"",
  "\"IF\"",
  "\"END\"",
  "\"BEGIN\"",
  "name",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "')'",
  "'('",
  "\"FUNCTION\"",
  "'+'",
  "\"**\"",
  "\"LONG\"",
  "\"DOUBLE\"",
  "integer",
  "real",
  "character constant",
  "'='",
  "\"<>\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (parse_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (parse_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_parse(parse_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void parse(parse_pcb_type *PCB_POINTER) {
  init_parse(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


