/*
   dci.syn -- Direct Compilation of Bytecode
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 Modified to support arrays, structs and functions

 The grammar contained in this file describes the same language as the
 grammar in pll.syn. The differences can be categorized as follows:
   . Rules rewritten to make computation easier. In pll.syn all of the
     arithmetic operators have been factored out. Here the arithmetic
     operators have been substituted back into the rules for expressions to
     make computation more straightforward.
   . Rules rewritten to handle looping. The rules for loop constructs have
     been rewritten to handle repetitive parsing of loops.
   . Semantically determined rules (true, false) added to support parse time
     implementation of loops and if-else statements.

 The actual parser modules, dci.h and dci.cpp are created from
 dci.syn by the AnaGram parser generator using the Build Parser command.

 To build a demonstration program for this parser, compile and link as a
 single project:
   demo.cpp                // Demo program
   comdefs.cpp             // Common script definitions
   bcidefs.cpp             // Bytecode interpreter definitions
   dci.cpp                 // Direct execution parser
   agclib1.lib             // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is somewhat similar to Pascal.

 The language consists of Pascal like statements and expressions. The
 language also implements Fortran-style exponentiation and simple dump and
 print statements. The syntax below uses "open" and "closed" statements to
 eliminate the traditional "dangling else" or "if-else" ambiguity problem.
 See http://www.parsifalsoft.com/ifelse.html or doc/ifelse.htm for a more
 detailed discussion.

 Statement types supported are:
   assignment statements
   compound statements
   if/else statements
   while statements
   repeat/until statements
   for statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"                // Contains definitions of support classes
#include <math.h>
#include <ctype.h>
#include "bcidefs.h"                       // Bytecode interpreter definitions


/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 26 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef DCI_H_1027348899
#include "dci.h"
#endif

#ifndef DCI_H_1027348899
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE dci_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer

CodeFragment dci_value(PCB_DECL);

static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 503, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

// implement context tracking
#define GET_CONTEXT CONTEXT = FileLocation(PCB.pointer, PCB.line, PCB.column)

void dci_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

void dci_pcb_struct::reportError(const char *msg) {
  ag_delete_wrappers(this);
  char buf[100];
  FileLocation &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): %s", context.line, context.column, msg);
  throw ErrorDiagnostic(buf);
}

// Constructor for parser control block
// Initializes dictionary, input pointer, and loopDepth

dci_pcb_struct::dci_pcb_struct(AgDictionary<AgString> &d, const char *text)
  : pointer((unsigned char *) text),
    dictionary(d)
{
  // Nothing else to do
}

int dci_pcb_struct::idName(const AgString &name) {
  return dictionary.intern(name);
}

CodeFragment dci_pcb_struct::code(Opcode op, const AgString &name) {
  return CodeFragment().append(op,dictionary.intern(name));
}

CodeFragment dci_pcb_struct::codeMember(const AgString &name) {
  return CodeFragment().append(MEMBER,fieldNames.intern(name));
}

CodeFragment dci_pcb_struct::code(Opcode op, const Constant &x) {
  int n = constants.intern(x);
  return CodeFragment().append(op,n);
}

CodeFragment dci_pcb_struct::codeCall(
    const AgString &name,
    AgStack<CodeFragment> &argStack)
{
  int argCount = argStack.size();
  CodeFragment argCode;
  for (int i = 0; i < argCount; i++) argCode.concat(argStack[i]);
  int argCountId = constants.intern(Value(argCount));
  argCode.append(PUSHC, argCountId)
         .append(CALL, functionNames.intern(name));
  return argCode;
}

CodeFragment dci_pcb_struct::codeStructDecl(
    const AgString &name,
    const AgStack<AgString> &fields,
    const AgStack<AgString> &instances)
{
  int nid = dictionary.intern(name);
  AgDictionary<AgString> fieldDictionary(fields.contents());
  Constant constant(fieldDictionary);
  int sid = constants.intern(constant);

  CodeFragment structDef;
  structDef.append(LOCATE, nid)
           .append(PUSHC, sid)
           .append(SAP);
  for (int i = 0; i < instances.size(); i++) {
    structDef.append(LOCATE, dictionary.intern(instances[i]))
             .append(PUSHC, sid)
             .append(INSTANCE)
             .append(SAP);
  }
  return structDef;
}

CodeFragment dci_pcb_struct::codeStructDecl(
    const AgString &name,
    const AgStack<AgString> &fields)
{
  int nid = dictionary.intern(name);
  AgDictionary<AgString> fieldDictionary(fields.contents());
  Constant constant(fieldDictionary);
  int sid = constants.intern(constant);

  CodeFragment structDef;
  structDef.append(LOCATE, nid)
           .append(PUSHC, sid)
           .append(SAP);
  return structDef;
}

CodeFragment dci_pcb_struct::codeInstanceDecl(
    const AgStack<AgString> &fields,
    const AgStack<AgString> &instances)
{
  AgDictionary<AgString> fieldDictionary(fields.contents());
  Constant constant(fieldDictionary);
  int sid = constants.intern(constant);
  CodeFragment instanceDef;
  for (int i = 0; i < instances.size(); i++) {
    instanceDef.append(LOCATE, dictionary.intern(instances[i]))
               .append(PUSHC, sid)
               .append(INSTANCE)
               .append(SAP);
  }
  return instanceDef;
}

CodeFragment dci_pcb_struct::codeInstanceDecl(
    const AgString &name,
    const AgStack<AgString> &instances)
{
  CodeFragment instanceDef;
  int n = dictionary.intern(name);
  for (int i = 0; i < instances.size(); i++) {
    instanceDef.append(LOCATE, dictionary.intern(instances[i]))
               .append(LOCATE, n)
               .append(FETCH)
               .append(INSTANCE)
               .append(SAP);
  }
  return instanceDef;
}

CodeFragment dci_pcb_struct::defineFunction(
    const AgString &n,
    const AgStack<AgString> &nameList,
    const AgString &text)
{
  LocalFunction f(nameList, text);
  int constantId = constants.intern(Value(f));
  int nameId = dictionary.intern(n);
  CodeFragment code;
  return code.append(LOCATE, nameId)
             .append(PUSHC, constantId)
             .append(SAP);
}

ScriptMethod::ScriptMethod(const char *text, AgDictionary<AgString> &d)
: dictionary(d),
  bytecode(),
  constantList(),
  fieldNames(),
  localList(),
  functionNames()
{
  // Create a parser using the dictionary specified by the dataset
  dci_pcb_type pcb(dictionary, text);

  // run the parser
  try {
    dci(&pcb);
  }
  catch(ErrorMessage e) {
    pcb.reportError(e.message());
  }

  //Use dci_value() to retrieve the generated code
  bytecode = dci_value(&pcb).getBytecode();

  constantList = pcb.constants.contents();
  fieldNames = pcb.fieldNames.contents();
  localList = pcb.localList.contents();
  functionNames = pcb.functionNames.contents();
}

// Apply script to a dataset
Value interpret(const char *text, Dataset &d) {
  ScriptMethod method(text, d.dictionary);

  // List the generated code
  //method.list(cout);

  // Execute the generated code
  return method.apply(d);
}


#ifndef CONVERT_CASE

static const char agCaseTable[31] = {
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,    0,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20
};

static int agConvertCase(int c) {
  if (c >= 'a' && c <= 'z') return c ^= 0x20;
  if (c >= 0xe0 && c < 0xff) c ^= agCaseTable[c-0xe0];
  return c;
}

#define CONVERT_CASE(c) agConvertCase(c)

#endif


#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif
CodeFragment dci_value(PCB_DECL) {
  CodeFragment returnValue;
  returnValue = (*(AgObjectWrapper< CodeFragment > *) &(PCB).vs[(PCB).ssx]);
  return returnValue;
}

static inline CodeFragment ag_rp_1(PCB_DECL, AgStack<AgString> &list, CodeFragment &x) {
/* Line 149, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return PCB.localList = list, x.append(HLT);
}

static inline CodeFragment ag_rp_2(PCB_DECL) {
/* Line 153, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_3(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 154, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x.concat(y);
}

static inline CodeFragment ag_rp_4(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 172, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return CodeFragment::ifStatement(x, s);
}

static inline CodeFragment ag_rp_5(PCB_DECL, CodeFragment &x, CodeFragment &s1, CodeFragment &s2) {
/* Line 174, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return CodeFragment::ifElse(x,s1,s2);
}

static inline CodeFragment ag_rp_6(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 175, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return CodeFragment::whileLoop(x,s);
}

static inline CodeFragment ag_rp_7(PCB_DECL, CodeFragment &lv, CodeFragment &b, CodeFragment &i, CodeFragment &e, CodeFragment &s) {
/* Line 178, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return CodeFragment::simpleForLoop(lv,b,i,e,s);
}

static inline CodeFragment ag_rp_8(PCB_DECL, CodeFragment &x, CodeFragment &s1, CodeFragment &s2) {
/* Line 182, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return CodeFragment::ifElse(x,s1,s2);
}

static inline CodeFragment ag_rp_9(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 184, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return CodeFragment::whileLoop(x,s);
}

static inline CodeFragment ag_rp_10(PCB_DECL, CodeFragment &lv, CodeFragment &b, CodeFragment &i, CodeFragment &e, CodeFragment &s) {
/* Line 187, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return CodeFragment::simpleForLoop(lv,b,i,e,s);
}

static inline CodeFragment ag_rp_11(PCB_DECL, CodeFragment &x) {
/* Line 188, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_12(PCB_DECL) {
/* Line 191, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return PCB.code(PUSHC, Value(1));
}

static inline CodeFragment ag_rp_13(PCB_DECL, CodeFragment &x) {
/* Line 192, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_14(PCB_DECL, CodeFragment &s, CodeFragment &x) {
/* Line 205, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return CodeFragment::repeatLoop(s,x);
}

static inline CodeFragment ag_rp_15(PCB_DECL, CodeFragment &lv, CodeFragment &x) {
/* Line 206, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return lv.concat(x).append(SAP);
}

static inline CodeFragment ag_rp_16(PCB_DECL) {
/* Line 207, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_17(PCB_DECL) {
/* Line 209, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return CodeFragment().append(HLT);
}

static inline CodeFragment ag_rp_18(PCB_DECL, CodeFragment &x) {
/* Line 210, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x.append(RETURN);
}

static inline CodeFragment ag_rp_19(PCB_DECL, AgString &n, CodeFragment &x) {
/* Line 214, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return PCB.code(LOCATE, n).concat(x).append(ARRAY).append(SAP);
}

static inline AgStack<AgString> ag_rp_20(PCB_DECL) {
/* Line 219, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return AgStack<AgString>();
}

static inline AgStack<AgString> ag_rp_21(PCB_DECL, AgStack<AgString> &list) {
/* Line 220, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return list;
}

static inline CodeFragment ag_rp_22(PCB_DECL, CodeFragment &x) {
/* Line 232, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_23(PCB_DECL, CodeFragment &s) {
/* Line 235, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return s;
}

static inline CodeFragment ag_rp_24(PCB_DECL, AgString &n) {
/* Line 239, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return PCB.code(DUMP,n);
}

static inline CodeFragment ag_rp_25(PCB_DECL, CodeFragment &code, AgString &n) {
/* Line 240, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return code.append(DUMP, PCB.idName(n));
}

static inline CodeFragment ag_rp_26(PCB_DECL, CodeFragment &code) {
/* Line 244, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return code.append(PRINT);
}

static inline CodeFragment ag_rp_27(PCB_DECL, CodeFragment &code, CodeFragment &x) {
/* Line 245, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return code.concat(x).append(PRINT);
}

static inline CodeFragment ag_rp_28(PCB_DECL, AgString &n, AgStack<AgString> &fl, AgStack<AgString> &il) {
/* Line 249, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return PCB.codeStructDecl(n, fl, il);
}

static inline CodeFragment ag_rp_29(PCB_DECL, AgString &n, AgStack<AgString> &fl) {
/* Line 250, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return PCB.codeStructDecl(n, fl);
}

static inline CodeFragment ag_rp_30(PCB_DECL, AgStack<AgString> &fl, AgStack<AgString> &nl) {
/* Line 252, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return PCB.codeInstanceDecl(fl, nl);
}

static inline CodeFragment ag_rp_31(PCB_DECL, AgString &n, AgStack<AgString> &nl) {
/* Line 253, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return PCB.codeInstanceDecl(n, nl);
}

static inline CodeFragment ag_rp_32(PCB_DECL, AgString &n, AgStack<AgString> &list, AgString &text) {
/* Line 257, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return PCB.defineFunction(n, list, text);
}

static AgString ag_rp_33(PCB_DECL) {
/* Line 260, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
return AgString().append(
               (const char *) CONTEXT.pointer, PCB.pointer - CONTEXT.pointer);
 
}

static inline AgStack<AgString> ag_rp_34(PCB_DECL, AgString &n) {
/* Line 265, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return AgStack<AgString>().push(n);
}

static inline AgStack<AgString> ag_rp_35(PCB_DECL, AgStack<AgString> &stack, AgString &n) {
/* Line 266, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return stack.push(n);
}

static inline CodeFragment ag_rp_36(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 272, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_37(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 276, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_38(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 280, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_39(PCB_DECL, CodeFragment &x) {
/* Line 285, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_40(PCB_DECL, Opcode op, CodeFragment &x) {
/* Line 286, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x.append(op);
}

static inline CodeFragment ag_rp_41(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 294, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x.concat(y).append(POW);
}

static inline CodeFragment ag_rp_42(PCB_DECL, CodeFragment &x) {
/* Line 302, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_43(PCB_DECL, Constant &x) {
/* Line 303, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return PCB.code(PUSHC, x);
}

static inline CodeFragment ag_rp_44(PCB_DECL, CodeFragment &x) {
/* Line 304, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x.append(FETCH);
}

static inline CodeFragment ag_rp_45(PCB_DECL, CodeFragment &x) {
/* Line 306, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x.append(CAST_LONG);
}

static inline CodeFragment ag_rp_46(PCB_DECL, CodeFragment &x) {
/* Line 307, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x.append(CAST_DOUBLE);
}

static inline CodeFragment ag_rp_47(PCB_DECL, AgString &n) {
/* Line 310, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return PCB.code(LOCATE, n);
}

static inline CodeFragment ag_rp_48(PCB_DECL, CodeFragment &lv, CodeFragment &x) {
/* Line 311, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return lv.concat(x).append(SUBSCRIPT);
}

static inline CodeFragment ag_rp_49(PCB_DECL, CodeFragment &lv, AgString &n) {
/* Line 312, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return lv.concat(PCB.codeMember(n));
}

static inline CodeFragment ag_rp_50(PCB_DECL, AgString &n, AgStack<CodeFragment> &args) {
/* Line 315, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return PCB.codeCall(n, args);
}

static inline AgStack<CodeFragment> ag_rp_51(PCB_DECL) {
/* Line 318, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return AgStack<CodeFragment>();
}

static inline AgStack<CodeFragment> ag_rp_52(PCB_DECL, CodeFragment &code) {
/* Line 322, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return AgStack<CodeFragment>().push(code);
}

static inline AgStack<CodeFragment> ag_rp_53(PCB_DECL, AgStack<CodeFragment> &stack, CodeFragment &x) {
/* Line 323, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return stack.push(x);
}

static inline Constant ag_rp_54(PCB_DECL, long x) {
/* Line 326, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return Constant(x);
}

static inline Constant ag_rp_55(PCB_DECL, double x) {
/* Line 327, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return Constant(x);
}

static inline Constant ag_rp_56(PCB_DECL, int x) {
/* Line 329, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return Constant(x);
}

static inline Opcode ag_rp_57(PCB_DECL) {
/* Line 333, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return EQ;
}

static inline Opcode ag_rp_58(PCB_DECL) {
/* Line 334, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return NE;
}

static inline Opcode ag_rp_59(PCB_DECL) {
/* Line 335, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return LT;
}

static inline Opcode ag_rp_60(PCB_DECL) {
/* Line 336, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return LE;
}

static inline Opcode ag_rp_61(PCB_DECL) {
/* Line 337, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return GT;
}

static inline Opcode ag_rp_62(PCB_DECL) {
/* Line 338, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return GE;
}

static inline Opcode ag_rp_63(PCB_DECL) {
/* Line 341, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return ADD;
}

static inline Opcode ag_rp_64(PCB_DECL) {
/* Line 342, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return SUB;
}

static inline Opcode ag_rp_65(PCB_DECL) {
/* Line 343, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return IOR;
}

static inline Opcode ag_rp_66(PCB_DECL) {
/* Line 344, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return XOR;
}

static inline Opcode ag_rp_67(PCB_DECL) {
/* Line 347, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return MUL;
}

static inline Opcode ag_rp_68(PCB_DECL) {
/* Line 348, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return RDIV;
}

static inline Opcode ag_rp_69(PCB_DECL) {
/* Line 349, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return IDIV;
}

static inline Opcode ag_rp_70(PCB_DECL) {
/* Line 350, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return MOD;
}

static inline Opcode ag_rp_71(PCB_DECL) {
/* Line 351, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return AND;
}

static inline Opcode ag_rp_72(PCB_DECL) {
/* Line 352, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return LS;
}

static inline Opcode ag_rp_73(PCB_DECL) {
/* Line 353, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return RS;
}

static inline Opcode ag_rp_74(PCB_DECL) {
/* Line 356, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return NEG;
}

static inline Opcode ag_rp_75(PCB_DECL) {
/* Line 357, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return NOT;
}

static inline AgString ag_rp_76(PCB_DECL, int c) {
/* Line 375, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return AgString().append(tolower(c));
}

static inline AgString ag_rp_77(PCB_DECL, AgString &ns, int c) {
/* Line 376, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return ns.append(tolower(c));
}

static inline double ag_rp_78(PCB_DECL, double x, long e) {
/* Line 382, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x*pow(10,e);
}

static inline double ag_rp_79(PCB_DECL, double x, long e) {
/* Line 383, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x*pow(10,e);
}

static inline double ag_rp_80(PCB_DECL, double i, double f) {
/* Line 386, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return i+f;
}

static inline double ag_rp_81(PCB_DECL, double i) {
/* Line 387, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return i;
}

static inline double ag_rp_82(PCB_DECL, double f) {
/* Line 388, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return f;
}

static inline double ag_rp_83(PCB_DECL, long x) {
/* Line 391, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x;
}

static inline double ag_rp_84(PCB_DECL, long x) {
/* Line 392, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x;
}

static inline double ag_rp_85(PCB_DECL, long x) {
/* Line 393, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return makeDecimal(x);
}

static inline long ag_rp_86(PCB_DECL, long x) {
/* Line 396, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return (long)x;
}

static inline long ag_rp_87(PCB_DECL, long x) {
/* Line 397, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return -(long)x;
}

static inline double ag_rp_88(PCB_DECL, int d) {
/* Line 400, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_89(PCB_DECL, int d, double f) {
/* Line 401, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_90(PCB_DECL, int d) {
/* Line 409, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return d-'0';
}

static inline long ag_rp_91(PCB_DECL, long x, int d) {
/* Line 410, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_92(PCB_DECL, int d) {
/* Line 413, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return d-'0';
}

static inline long ag_rp_93(PCB_DECL, long x, int d) {
/* Line 414, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_94(PCB_DECL, long x, int d) {
/* Line 417, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_95(PCB_DECL, long x, int d) {
/* Line 418, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_96(PCB_DECL) {
/* Line 421, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return 0;
}

static inline long ag_rp_97(PCB_DECL, long n, int d) {
/* Line 422, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_98(PCB_DECL) {
/* Line 425, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return 0;
}

static inline long ag_rp_99(PCB_DECL, long n, int d) {
/* Line 426, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return 16*n + d;
}

static inline int ag_rp_100(PCB_DECL, int x) {
/* Line 429, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return x - '0';
}

static inline int ag_rp_101(PCB_DECL, int d) {
/* Line 430, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return (d & 7) + 9;
}

static inline Constant ag_rp_102(PCB_DECL, Value &s) {
/* Line 435, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return Constant(s);
}

static inline Constant ag_rp_103(PCB_DECL, Constant &s, Value &e) {
/* Line 436, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return Constant(s+=e);
}

static inline Value ag_rp_104(PCB_DECL, AgString &b) {
/* Line 439, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return Value(b);
}

static inline AgString ag_rp_105(PCB_DECL) {
/* Line 443, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return AgString();
}

static inline AgString ag_rp_106(PCB_DECL, AgString &s, int c) {
/* Line 444, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return s.append(c);
}

static inline int ag_rp_107(PCB_DECL) {
/* Line 456, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return '\'';
}

static inline int ag_rp_108(PCB_DECL) {
/* Line 457, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return '"';
}

static inline int ag_rp_109(PCB_DECL) {
/* Line 458, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return '\?';
}

static inline int ag_rp_110(PCB_DECL) {
/* Line 459, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return '\\';
}

static inline int ag_rp_111(PCB_DECL) {
/* Line 460, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return '\a';
}

static inline int ag_rp_112(PCB_DECL) {
/* Line 461, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return '\b';
}

static inline int ag_rp_113(PCB_DECL) {
/* Line 462, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return '\f';
}

static inline int ag_rp_114(PCB_DECL) {
/* Line 463, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return '\n';
}

static inline int ag_rp_115(PCB_DECL) {
/* Line 464, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return '\r';
}

static inline int ag_rp_116(PCB_DECL) {
/* Line 465, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return '\t';
}

static inline int ag_rp_117(PCB_DECL) {
/* Line 466, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return '\v';
}

static inline int ag_rp_118(PCB_DECL, int d) {
/* Line 472, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return d-'0';
}

static inline int ag_rp_119(PCB_DECL, int n, int d) {
/* Line 475, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_120(PCB_DECL, int n, int d) {
/* Line 478, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_121(PCB_DECL, int d) {
/* Line 481, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return d;
}

static inline int ag_rp_122(PCB_DECL, int n, int d) {
/* Line 482, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return 16*n + d;
}

static inline int ag_rp_123(PCB_DECL, int d) {
/* Line 485, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return d-'0';
}

static inline int ag_rp_124(PCB_DECL, int d) {
/* Line 486, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_125(PCB_DECL, int c) {
/* Line 495, W:/agdev/agex/packages/xidek/pll/combine/dci.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 4, 9, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 6, 6, 0, 6, 0, 0, 0, 0, 6, 4, 0, 0,
  4, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 4, 8, 0, 0, 0, 6, 0, 0, 6,
  0, 0, 6, 6, 6, 4, 0, 0, 6, 0, 0, 0, 0, 0, 0, 6, 6, 6, 6, 6, 0, 9, 9,
  0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6, 0, 0, 0, 0,
  0, 0, 0, 0, 9, 0, 0, 9, 0, 0, 7, 7, 0, 0, 0, 6, 6, 6, 0, 6, 6, 0, 9,
  0, 0, 0, 0, 0, 0, 0, 6, 0, 9, 0, 0, 9, 4, 6, 6, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<CodeFragment >
#undef AG_WRAP_7
#define AG_WRAP_7 AgObjectWrapper<AgStack<CodeFragment> >
#undef AG_WRAP_8
#define AG_WRAP_8 AgObjectWrapper<Constant >
#undef AG_WRAP_9
#define AG_WRAP_9 AgObjectWrapper<AgStack<AgString> >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      case 7: ag_delete_object((AG_WRAP_7 *) &(PCB).vs[sx]); break;
      case 8: ag_delete_object((AG_WRAP_8 *) &(PCB).vs[sx]); break;
      case 9: ag_delete_object((AG_WRAP_9 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static dci_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  2,  3,  0,  0,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
   16,  0, 17, 18,  0,  0, 19,  0,  0, 20, 21,  0,  0,  0, 22, 23, 24, 25,
   26, 27, 28, 29, 30, 31, 32, 33, 34, 35,  0, 36,  0, 37,  0, 38,  0, 39,
   40,  0, 41, 42, 43, 44,  0, 45, 46, 47, 48, 49, 50, 51,  0, 52, 53, 54,
   55,  0, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71,
   72, 73, 74, 75,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 76, 77,  0,
   78, 79, 80, 81, 82, 83, 84, 85,  0,  0, 86, 87, 88, 89,  0,  0,  0, 90,
   91, 92, 93, 94, 95, 96, 97, 98, 99,100,101,102,103,104,105,106,  0,  0,
    0,  0,  0,107,108,109,110,111,112,113,114,115,116,117,  0,  0,  0,118,
  119,120,121,122,123,124,125
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,161,  0,162,  0,163,  0,165,  0,167,  0,168,  0,172,  0,173,
  0,174,  0,175,  0,176,  0,178,  0,177,  0,180,  0,179,  0,181,
  0,183,  0,186,  0,189,  0,192,  0,193,  0,205,  0,206,  0,209,
  0,210,  0,211,  0,212,  0,213,  0,214,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 79, 85,255, 80, 84,255, 69,255, 47, 65, 66, 68, 70, 73,
   76, 80, 82, 83, 87,255, 42,255, 42, 47,255, 61, 62,255, 78, 82,255, 69,
   89,255, 85,255, 73, 79, 85,255, 76, 78,255, 79, 85,255, 67, 78,255, 79,
  255, 80, 84,255, 69,255, 76, 82,255, 72, 84,255, 72, 79,255, 42, 47, 48,
   58, 60, 62, 65, 66, 68, 69, 70, 73, 76, 77, 78, 79, 80, 82, 83, 84, 85,
   87, 88,255, 79, 85,255, 80, 84,255, 69,255, 65, 66, 68, 70, 73, 76, 80,
   82, 83, 87,255, 42, 47,255, 47,255, 79, 85,255, 80, 84,255, 69,255, 65,
   66, 68, 70, 73, 80, 82, 83, 87,255, 42, 47,255, 61, 62,255, 42, 47, 58,
   60, 62,255, 42, 47,255, 47, 48, 78,255, 42, 47,255, 76, 78,255, 79, 85,
  255, 80, 84,255, 69,255, 47, 65, 66, 68, 69, 70, 73, 80, 82, 83, 85, 87,
  255, 42, 47,255, 79, 85,255, 80, 84,255, 69,255, 47, 65, 66, 68, 69, 70,
   73, 76, 80, 82, 83, 87,255, 42, 47,255, 79, 85,255, 80, 84,255, 69,255,
   47, 65, 66, 68, 70, 73, 80, 82, 83, 85, 87,255, 48, 78,255, 79, 85,255,
   80, 84,255, 69,255, 65, 66, 68, 69, 70, 73, 80, 82, 83, 87,255, 58,255,
   79, 85,255, 80, 84,255, 69,255, 65, 66, 68, 70, 73, 80, 82, 83, 85, 87,
  255, 34, 39, 63, 65, 66, 70, 78, 82, 84, 86, 88, 92,255, 92,255, 42, 47,
  255, 61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 60,
   62, 65, 66, 68, 77, 79, 83, 84, 88,255, 42, 47,255, 61, 62,255, 76, 82,
  255, 72,255, 72, 79,255, 42, 47, 60, 62, 77, 79, 83, 84, 88,255, 61, 62,
  255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 60, 62, 65, 66, 68,
   77, 79, 83, 84, 88,255, 42, 47,255, 47, 48, 68, 76, 78,255, 48, 68, 76,
   78,255, 61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 60, 62,
   65, 66, 68, 77, 79, 83, 84, 88,255, 61, 62,255, 72, 79,255, 60, 62, 66,
   68, 79, 84, 88,255, 84,255, 68,255, 76, 78,255, 79, 85,255, 80, 84,255,
   69,255, 65, 66, 68, 69, 70, 73, 80, 82, 83, 85, 87,255, 42, 47,255, 79,
   85,255, 80, 84,255, 69,255, 47, 65, 66, 68, 70, 73, 80, 82, 83, 87,255,
   42, 47,255, 61, 62,255, 69, 89,255, 73, 79,255, 76, 82,255, 72,255, 72,
   79,255, 42, 47, 48, 60, 62, 65, 66, 68, 77, 79, 83, 84, 88,255, 48,255,
   72, 79,255, 66, 68, 79, 84, 88,255, 42, 47,255, 61, 62,255, 73, 79,255,
   76, 82,255, 72,255, 72, 79,255, 42, 47, 58, 60, 62, 65, 66, 68, 77, 79,
   83, 84, 88,255, 66, 84,255, 66,255, 84,255, 79, 85,255, 80, 84,255, 69,
  255, 65, 66, 68, 69, 70, 73, 76, 80, 82, 83, 87,255, 69,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,7,7,4,7,7,4,2,4,2,7,7,7,2,7,7,7,2,7,7,4,3,4,0,0,4,0,0,4,7,7,4,
  7,5,4,7,4,7,6,7,4,7,7,4,7,7,4,7,7,4,2,4,7,7,4,2,4,5,5,4,2,7,4,7,5,4,3,
  2,3,3,2,3,2,2,2,2,2,7,2,7,7,7,7,2,2,2,7,7,7,4,7,7,4,7,7,4,2,4,7,7,7,2,
  7,7,7,2,7,7,4,0,0,4,2,4,7,7,4,7,7,4,2,4,7,7,7,2,7,7,2,7,7,4,0,0,4,0,0,
  4,3,2,3,2,3,4,0,0,4,2,3,7,4,0,0,4,7,7,4,7,7,4,7,7,4,2,4,2,7,7,7,2,2,7,
  7,2,7,7,7,4,0,0,4,7,7,4,7,7,4,2,4,2,7,7,7,7,2,7,7,7,2,7,7,4,0,0,4,7,7,
  4,7,7,4,2,4,2,7,7,7,2,7,7,2,7,7,7,4,3,7,4,7,7,4,7,7,4,2,4,7,7,7,7,2,7,
  7,2,7,7,4,3,4,7,7,4,7,7,4,2,4,7,7,7,2,7,7,2,7,7,7,4,0,0,0,0,0,0,0,0,0,
  0,0,0,4,2,4,0,0,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,2,3,7,7,2,7,7,2,2,7,
  4,0,0,4,0,0,4,5,5,4,2,4,7,5,4,3,2,2,3,7,7,2,2,7,4,0,0,4,7,5,4,5,5,4,2,
  4,7,5,4,3,2,3,7,7,2,7,7,2,2,7,4,0,0,4,2,3,7,7,7,4,3,7,7,7,4,0,0,4,7,5,
  4,5,5,4,2,4,7,5,4,2,3,7,7,2,7,7,2,2,7,4,0,0,4,7,5,4,2,3,7,7,7,2,7,4,7,
  4,7,4,7,7,4,7,7,4,7,7,4,2,4,7,7,7,2,2,7,7,2,7,7,7,4,0,0,4,7,7,4,7,7,4,
  2,4,2,7,7,7,2,7,7,2,7,7,4,0,0,4,0,0,4,7,5,4,7,5,4,5,5,4,2,4,7,5,4,3,2,
  3,2,3,7,2,2,7,7,2,2,7,4,3,4,7,5,4,7,7,7,2,7,4,0,0,4,0,0,4,7,5,4,5,5,4,
  2,4,7,5,4,3,2,3,2,3,7,7,2,7,7,2,2,7,4,7,7,4,7,4,7,4,7,7,4,7,7,4,2,4,7,
  7,7,7,2,7,7,7,2,7,7,4,7,4
};

static const unsigned char ag_key_parm[] = {
    0, 94, 99,  0, 20, 36,  0, 16, 10,  0,  0,  0,  0, 12, 26, 30,  0, 22,
   14, 32,  0, 34, 18,  0, 98,  0, 94, 99,  0,201,199,  0, 50, 12,  0, 26,
    6,  0, 40,  0, 46,  2, 30,  0,  0, 28,  0, 20, 36,  0, 14, 38,  0,  0,
    0, 16, 10,  0,  0,  0, 52, 54,  0,  0, 34,  0, 24,  4,  0,191,  0,123,
  164,  0,203,  0,  0,  0,  0,  0, 22,  0, 48, 56, 42, 32,  0,  0,  0,  8,
   18, 44,  0, 20, 36,  0, 16, 10,  0,  0,  0, 12, 26, 30,  0, 22, 14, 32,
    0, 34, 18,  0, 94, 99,  0,  0,  0, 20, 36,  0, 16, 10,  0,  0,  0, 12,
   26, 30,  0, 22, 32,  0, 34, 18,  0, 94, 99,  0,201,199,  0,191,  0,164,
    0,203,  0, 94, 99,  0,  0,123, 56,  0, 94, 99,  0,  0, 28,  0, 20, 36,
    0, 16, 10,  0,  0,  0,  0, 12, 26, 30,  0,  0, 22, 32,  0, 34,  8, 18,
    0, 94, 99,  0, 20, 36,  0, 16, 10,  0,  0,  0,  0, 12, 26, 30, 28,  0,
   22, 14, 32,  0, 34, 18,  0, 94, 99,  0, 20, 36,  0, 16, 10,  0,  0,  0,
    0, 12, 26, 30,  0, 22, 32,  0, 34,  8, 18,  0,123, 56,  0, 20, 36,  0,
   16, 10,  0,  0,  0, 12, 26, 30, 28,  0, 22, 32,  0, 34, 18,  0,164,  0,
   20, 36,  0, 16, 10,  0,  0,  0, 12, 26, 30,  0, 22, 32,  0, 34,  8, 18,
    0,135,134,136,138,139,140,141,142,143,144,149,137,  0,  0,  0, 94, 99,
    0,201,199,  0, 46,  2,  0, 52, 54,  0,  0,  0, 24,  4,  0,191,  0,  0,
  203, 50,  6,  0, 48, 42,  0,  0, 44,  0, 94, 99,  0,201,199,  0, 52, 54,
    0,  0,  0, 24,  4,  0,191,  0,  0,203, 48, 42,  0,  0, 44,  0,201,199,
    0, 46,  2,  0, 52, 54,  0,  0,  0, 24,  4,  0,191,  0,203, 50,  6,  0,
   48, 42,  0,  0, 44,  0, 94, 99,  0,  0,123, 40, 38, 56,  0,123, 40, 38,
   56,  0,201,199,  0, 46,  2,  0, 52, 54,  0,  0,  0, 24,  4,  0,  0,203,
   50,  6,  0, 48, 42,  0,  0, 44,  0,201,199,  0, 24,  4,  0,  0,203,  6,
    2, 42,  0, 44,  0, 24,  0,  2,  0,  0, 28,  0, 20, 36,  0, 16, 10,  0,
    0,  0, 12, 26, 30,  0,  0, 22, 32,  0, 34,  8, 18,  0, 94, 99,  0, 20,
   36,  0, 16, 10,  0,  0,  0,  0, 12, 26, 30,  0, 22, 32,  0, 34, 18,  0,
   94, 99,  0,201,199,  0, 26,  6,  0, 46,  2,  0, 52, 54,  0,  0,  0, 24,
    4,  0,191,  0,123,  0,203, 50,  0,  0, 48, 42,  0,  0, 44,  0,123,  0,
   24,  4,  0,  6,  2, 42,  0, 44,  0, 94, 99,  0,201,199,  0, 46,  2,  0,
   52, 54,  0,  0,  0, 24,  4,  0,191,  0,164,  0,203, 50,  6,  0, 48, 42,
    0,  0, 44,  0,  6,  4,  0, 26,  0,  4,  0, 20, 36,  0, 16, 10,  0,  0,
    0, 12, 26, 30, 28,  0, 22, 14, 32,  0, 34, 18,  0, 28,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0, 14, 16,  0, 35, 39,  0,  7,  0,  1,  0,  5, 10,  4, 23,
   25, 30, 10, 43, 49,  0, 54,  0,  0,  0,  0,  0,  0,  0, 64, 66,  0, 70,
    0,  0, 76,  0, 74, 38, 80,  0, 83, 86,  0, 88, 90,  0, 99,102,  0, 50,
    0,117,121,  0, 55,  0,  0,  0,  0, 60,125,  0,130,  0,  0, 56, 26, 58,
   60, 29, 62, 32, 35, 40, 44, 47, 97, 53,104,107,110,112, 58, 63, 66,133,
  138,143,  0,160,162,  0,181,185,  0, 96,  0,146,151,156, 93,169,171,176,
   99,189,195,  0,  0,  0,  0,112,  0,214,216,  0,230,234,  0,120,  0,200,
  205,210,117,223,225,123,238,244,  0,  0,  0,  0,  0,  0,  0,249,135,251,
  138,253,  0,  0,  0,  0,147,255,257,  0,  0,  0,  0,274,277,  0,279,281,
    0,295,299,  0,163,  0,154,260,265,270,157,160,288,290,166,303,309,314,
    0,  0,  0,  0,336,338,  0,357,361,  0,187,  0,181,319,324,329,333,184,
  345,347,352,190,365,371,  0,  0,  0,  0,390,392,  0,406,410,  0,211,  0,
  205,376,381,386,208,399,401,214,414,420,425,  0,430,432,  0,452,454,  0,
  468,472,  0,234,  0,435,440,445,449,231,461,463,237,476,482,  0,487,  0,
  503,505,  0,519,523,  0,255,  0,489,494,499,252,512,514,258,527,533,538,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,271,  0,  0,  0,
    0,  0,  0,  0,552,  0,  0,  0,  0,  0,295,  0,559,  0,  0,543,286,289,
  545,547,550,292,554,557,298,300,562,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,322,  0,574,  0,  0,565,316,319,567,569,572,325,327,577,  0,  0,  0,
    0,589,  0,  0,  0,  0,  0,346,  0,596,  0,  0,580,340,582,584,587,343,
  591,594,349,351,599,  0,  0,  0,  0,366,602,604,610,614,  0,617,619,625,
  629,  0,  0,  0,  0,639,  0,  0,  0,  0,  0,386,  0,646,  0,  0,380,632,
  634,637,383,641,644,389,391,649,  0,  0,  0,  0,660,  0,  0,405,652,654,
  656,658,408,663,  0,666,  0,670,  0,686,689,  0,691,693,  0,707,711,  0,
  429,  0,672,677,682,423,426,700,702,432,715,721,726,  0,  0,  0,  0,745,
  747,  0,761,765,  0,452,  0,446,731,736,741,449,754,756,455,769,775,  0,
    0,  0,  0,  0,  0,  0,789,  0,  0,793,  0,  0,  0,  0,  0,480,  0,800,
    0,  0,780,468,782,471,784,786,474,477,795,798,483,485,803,  0,806,  0,
  814,  0,  0,808,810,812,504,817,  0,  0,  0,  0,  0,  0,  0,831,  0,  0,
    0,  0,  0,522,  0,838,  0,  0,820,513,822,516,824,826,829,519,833,836,
  525,527,841,  0,844,846,  0,848,  0,853,  0,872,874,  0,893,897,  0,554,
    0,855,860,865,869,551,881,883,888,557,901,907,  0,912,  0
};

static const unsigned short ag_key_index[] = {
   12,  0, 24, 69,101,  0,  0, 24, 24,115,  0,125,141,  0,125,150,  0,168,
  115,115,150,115,192,216,  0,  0,115,228,  0,150,239,115,150,150,  0,  0,
    0,  0,228,250,260,228,  0,228,125,  0,115,  0,  0,284,  0,  0,284,303,
    0,303,330,303,303,303,303,303,354,354,369,150,150,354,375,150,354,228,
  228,394,411,239,  0,228,  0,  0,115,  0,150,228,150,228,260,419,250,421,
  434,  0,  0,  0,  0,284,303,  0,  0,284,284,284,  0,303,  0,  0,228,115,
    0,115,  0,  0,150,228,150,150,150,150,150,150,150,228,150,150,228,150,
  150,150,150,150,150,228,168,228,  0,  0,150,228,457,228,457,125,457,125,
    0,115,  0,  0,  0,  0,  0,  0,488,502,502,394,507,  0,530,  0,544,547,
    0,  0,303,303,228,  0,150,228,549,559,  0,150,228,239,571,421,239,125
};

static const unsigned char ag_key_ends[] = {
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 79,67,65,76,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 72,73,76,69,0, 47,0, 
42,0, 88,0, 61,0, 61,0, 68,0, 82,65,89,0, 71,73,78,0, 86,0, 
66,76,69,0, 77,80,0, 83,69,0, 68,0, 82,0, 78,67,84,73,79,78,0, 
70,0, 65,76,0, 71,0, 79,68,0, 79,84,0, 82,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 82,85,67,84,0, 69,78,0, 78,84,73,76,0, 
72,73,76,69,0, 79,82,0, 82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 
82,0, 78,67,84,73,79,78,0, 70,0, 79,67,65,76,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 72,73,76,69,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
84,82,85,67,84,0, 72,73,76,69,0, 42,0, 61,0, 61,0, 88,0, 
79,84,0, 82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 83,69,0, 
68,0, 82,0, 78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 78,84,73,76,0, 
72,73,76,69,0, 82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 78,68,0, 
82,0, 78,67,84,73,79,78,0, 70,0, 79,67,65,76,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 72,73,76,69,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
84,82,85,67,84,0, 78,84,73,76,0, 72,73,76,69,0, 88,0, 79,84,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 78,68,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
84,82,85,67,84,0, 72,73,76,69,0, 61,0, 82,82,65,89,0, 
69,71,73,78,0, 85,77,80,0, 82,0, 78,67,84,73,79,78,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 
78,84,73,76,0, 72,73,76,69,0, 42,0, 61,0, 78,68,0, 89,0, 86,0, 
79,68,0, 82,0, 69,78,0, 79,82,0, 42,0, 61,0, 79,68,0, 82,0, 
69,78,0, 79,82,0, 42,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 
82,0, 69,78,0, 79,82,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 
79,84,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 79,84,0, 61,0, 
78,68,0, 89,0, 86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 61,0, 
89,0, 79,0, 82,0, 69,78,0, 79,82,0, 72,69,78,0, 79,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 83,69,0, 68,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
84,82,85,67,84,0, 78,84,73,76,0, 72,73,76,69,0, 82,82,65,89,0, 
69,71,73,78,0, 85,77,80,0, 82,0, 78,67,84,73,79,78,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 
72,73,76,69,0, 42,0, 88,0, 61,0, 78,68,0, 71,73,78,0, 86,0, 
79,68,0, 82,0, 69,78,0, 79,82,0, 88,0, 89,0, 79,0, 82,0, 
69,78,0, 79,82,0, 42,0, 61,0, 61,0, 78,68,0, 89,0, 86,0, 
79,68,0, 82,0, 69,78,0, 79,82,0, 89,0, 79,0, 69,71,73,78,0, 
79,0, 82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 78,68,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 79,67,65,76,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 72,73,76,69,0, 78,68,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
   10,155,155,155,155,155,155,155,155,154,103,154,154,154,155,155,155,155,
  155,155,155,155,155,155,155,155,155,155,155,155,155,155,154,155,126,155,
  155,155,155,151,188,187,207,190,182,204,194,208,121,156,156,156,156,156,
  156,156,120,120,155,166,200,198,202,155,155,157,157,157,157,107,157,158,
  158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,
  158,170,148,169,155,158,155,157,157,157,157,107,157,158,158,158,158,158,
  158,158,158,158,158,158,158,158,158,158,158,158,158,158,158,185,155,184,
  155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
  155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
  155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
  155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
  155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
  155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
  155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
  155,155,155,155
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (dci_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
189,186,183,181,180,178,176,175,174,173,172,168,166,158,157,154,107,103,99,
  94,10,0,1,159,160,
208,207,204,202,200,198,194,190,188,187,185,184,182,170,169,166,158,157,156,
  155,154,151,148,126,121,120,107,103,0,101,102,
208,207,204,202,200,198,194,190,188,187,185,184,182,170,169,166,158,157,156,
  155,154,151,148,126,121,120,107,103,98,0,96,97,
154,103,99,94,0,1,
189,186,183,181,180,178,176,175,174,173,172,168,166,158,157,107,10,0,7,8,38,
208,207,204,202,200,198,194,190,188,187,185,184,182,170,169,166,158,157,156,
  155,154,151,148,126,121,120,107,0,
103,0,
208,207,204,202,200,198,194,190,188,187,185,184,182,170,169,166,158,157,156,
  155,154,151,148,126,121,120,107,103,0,
98,0,
158,157,154,107,103,99,94,0,1,159,160,
158,157,107,0,4,39,171,
189,186,183,181,180,178,176,175,174,172,168,166,158,157,107,10,0,9,
208,207,204,203,202,201,200,199,198,194,191,190,188,187,185,184,182,170,169,
  166,164,158,157,156,154,121,120,107,103,99,94,0,1,159,160,
182,166,0,28,48,
189,186,183,181,180,178,176,175,174,172,168,166,158,157,107,10,0,4,11,12,13,
  14,16,19,20,24,26,28,29,30,31,32,33,36,37,40,41,42,43,45,47,49,50,53,
  171,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
158,157,107,0,4,171,
189,186,183,181,180,179,178,176,175,174,172,168,167,166,161,158,157,154,107,
  103,99,94,10,0,1,159,160,
158,157,154,107,103,99,94,0,1,159,160,
185,158,157,154,107,103,99,94,0,1,159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
158,157,154,107,103,99,94,0,1,159,160,
189,186,183,181,180,179,178,176,175,174,173,172,168,166,158,157,154,107,103,
  99,94,0,1,159,160,
189,186,183,181,180,178,176,175,174,172,168,167,166,158,157,154,107,103,99,
  94,0,1,159,160,
158,157,107,0,4,171,
185,158,157,107,0,4,51,171,
158,157,154,107,103,99,94,0,1,159,160,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,17,20,
  54,57,59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,
  196,197,215,
158,157,107,0,4,171,
214,204,194,190,188,166,158,157,156,154,151,126,123,121,120,107,103,99,94,0,
  1,159,160,
189,186,183,181,180,179,178,176,175,174,172,168,166,158,157,107,0,9,
158,157,154,107,103,99,94,0,1,159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
166,0,28,
158,157,107,0,4,171,
182,166,0,28,48,
182,166,0,28,48,
214,204,194,190,188,166,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,17,
  20,28,54,57,59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,
  171,195,196,197,215,
194,170,164,0,21,34,72,
189,186,183,181,180,178,176,175,174,172,168,167,166,158,157,107,0,9,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,17,20,
  54,57,59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,
  196,197,215,
158,157,107,0,4,20,171,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,17,20,
  54,57,59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,
  196,197,215,
189,186,183,181,180,178,176,175,174,172,168,166,158,157,107,0,4,11,12,13,14,
  16,19,20,24,26,28,29,30,31,32,33,36,37,40,41,42,43,45,47,49,50,53,171,
188,0,54,
158,157,154,107,103,99,94,0,1,159,160,
158,157,107,0,4,39,171,
185,158,157,107,0,4,39,51,171,
208,207,204,202,200,198,194,190,188,187,185,184,182,170,169,166,158,157,156,
  155,154,151,149,148,144,143,142,141,140,139,138,137,136,135,134,126,121,
  120,107,0,127,
156,121,120,0,116,
156,121,120,0,110,116,
208,207,204,202,200,198,194,190,188,187,185,184,182,170,169,166,158,157,156,
  155,154,149,148,144,143,142,141,140,139,138,137,136,135,134,126,121,120,
  107,0,130,131,132,133,145,146,147,152,
213,212,211,210,209,208,207,206,205,204,203,202,201,200,199,198,191,190,187,
  182,177,169,166,165,163,162,154,126,103,99,94,0,1,159,160,
194,107,0,
107,0,
157,156,121,120,107,0,124,
213,212,211,210,209,208,207,206,205,204,203,202,201,200,199,198,194,191,190,
  187,182,177,169,166,165,163,162,156,154,121,120,107,103,99,94,0,
213,212,211,210,209,208,207,206,205,204,203,202,201,200,199,198,194,191,190,
  187,182,177,169,166,165,163,162,156,154,121,120,107,103,99,94,0,116,
213,212,211,210,209,208,207,206,205,204,203,202,201,200,199,198,191,190,187,
  182,177,169,166,165,163,162,154,103,99,94,0,1,159,160,
213,212,211,210,209,208,207,206,205,204,203,202,201,200,199,198,191,190,187,
  182,177,169,166,165,163,162,154,103,99,94,0,1,159,160,
213,212,211,210,209,208,207,206,205,204,203,202,201,200,199,198,191,190,187,
  182,177,169,166,165,163,162,154,103,99,94,0,1,159,160,
188,0,54,
126,0,5,215,
214,204,194,193,192,190,188,187,158,157,156,154,151,126,123,121,120,107,103,
  99,94,0,1,159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
194,170,0,34,72,
214,204,194,193,192,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,
  17,20,54,57,59,61,63,64,65,66,68,69,70,71,75,82,92,106,109,111,112,113,
  117,171,195,196,197,215,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
191,0,67,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,20,54,
  61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,196,197,
  215,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,20,54,
  61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,196,197,
  215,
213,212,211,210,209,208,207,0,62,85,86,87,88,89,90,91,
206,205,204,203,202,201,200,199,198,190,0,58,60,64,76,77,78,79,80,81,82,83,
  84,
189,186,183,181,180,179,178,176,175,174,172,168,166,158,157,107,0,4,11,12,
  13,14,16,19,20,24,26,28,29,30,31,32,33,36,37,40,41,42,43,45,46,47,49,50,
  53,171,
170,0,34,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,17,20,
  54,57,59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,
  196,197,215,
158,157,107,0,4,171,
166,0,28,
158,157,154,107,103,99,94,0,1,159,160,
158,157,107,0,4,171,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,17,20,
  54,57,59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,
  196,197,215,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,17,20,
  54,57,59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,
  196,197,215,
189,186,183,181,180,178,176,175,174,172,168,167,166,158,157,107,0,4,11,12,
  13,14,16,19,20,24,26,27,28,29,30,31,32,33,36,37,40,41,42,43,45,47,49,50,
  53,171,
177,0,44,
194,170,164,0,21,34,72,
162,0,18,
161,0,15,
158,157,107,0,4,39,171,
184,182,0,48,52,
182,0,48,
158,157,107,0,4,39,171,
208,207,204,202,200,198,194,190,188,187,185,184,182,170,169,166,158,157,156,
  155,154,151,149,148,144,143,142,141,140,139,138,137,136,135,134,126,121,
  120,107,0,128,130,131,132,133,145,146,147,
156,121,120,0,110,116,
156,121,0,
157,156,121,120,107,0,150,
156,121,0,
156,121,0,
157,156,121,120,107,0,150,
151,0,
156,121,120,0,110,116,
204,190,156,121,120,0,108,114,
204,190,156,121,120,0,108,114,
214,204,194,190,188,187,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,17,
  20,54,57,59,61,63,64,65,66,68,69,73,74,75,82,92,106,109,111,112,113,117,
  171,195,196,197,215,
187,154,103,99,94,0,1,159,160,
187,0,55,
187,154,103,99,94,0,1,159,160,
187,0,55,
187,0,55,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,20,54,
  61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,196,197,
  215,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,20,54,
  61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,196,197,
  215,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,20,54,
  59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,196,
  197,215,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,20,54,
  57,59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,196,
  197,215,
189,186,183,181,180,179,178,176,175,174,172,168,167,166,161,158,157,154,107,
  103,99,94,10,0,1,159,160,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,17,20,
  54,57,59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,
  196,197,215,
169,0,35,
166,0,28,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,17,20,
  54,57,59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,
  196,197,215,
189,186,183,181,180,178,176,175,174,172,168,166,158,157,154,107,103,99,94,0,
  1,159,160,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,17,20,
  54,57,59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,
  196,197,215,
189,186,183,181,180,178,176,175,174,172,168,166,158,157,154,107,103,99,94,0,
  1,159,160,
189,186,183,181,180,178,176,175,174,172,168,166,158,157,107,0,4,12,13,14,16,
  19,20,24,26,28,29,30,31,32,33,36,37,40,41,42,43,45,47,49,50,53,171,
189,186,183,181,180,178,176,175,174,172,168,166,158,157,154,107,103,99,94,0,
  1,159,160,
189,186,183,181,180,178,176,175,174,172,168,166,158,157,107,0,4,12,13,14,16,
  19,20,24,26,28,29,30,31,32,33,36,37,40,41,42,43,45,47,49,50,53,171,
187,182,0,48,55,
166,158,157,154,107,103,99,94,0,1,159,160,
158,157,107,0,4,39,171,
184,182,0,48,52,
156,121,120,0,115,
156,121,120,0,115,
182,0,48,
187,0,55,
213,212,211,210,209,208,207,206,205,204,203,202,201,200,199,198,194,191,190,
  188,187,182,180,177,169,166,165,163,162,158,157,156,154,151,126,123,121,
  120,107,103,99,94,0,1,159,160,
194,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,20,54,66,68,69,75,
  106,109,111,112,113,117,171,195,196,197,215,
194,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,20,54,66,68,69,75,
  106,109,111,112,113,117,171,195,196,197,215,
213,212,211,210,209,208,207,0,62,85,86,87,88,89,90,91,
206,205,204,190,0,60,64,82,83,84,
169,0,35,
213,212,211,210,209,208,207,206,205,204,203,202,201,200,199,198,194,191,190,
  187,182,177,170,169,166,165,164,163,162,154,103,99,94,0,1,159,160,
166,0,28,
165,163,0,22,25,
180,0,45,
182,0,48,
158,157,107,0,4,39,171,
156,121,120,0,116,
156,121,120,0,116,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,17,20,
  54,57,59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,
  196,197,215,
166,0,28,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,17,20,
  54,57,59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,
  196,197,215,
163,0,23,
189,186,183,181,180,179,178,176,175,174,173,172,168,166,158,157,107,0,8,38,
  56,
182,0,48,
214,204,194,190,188,158,157,156,154,151,126,123,121,120,107,103,99,94,0,1,
  159,160,
214,204,194,190,188,158,157,156,151,126,123,121,120,107,0,2,3,4,5,6,17,20,
  54,57,59,61,63,64,65,66,68,69,75,82,92,106,109,111,112,113,117,171,195,
  196,197,215,
189,186,183,181,180,179,178,176,175,174,172,168,166,158,157,107,0,9,
179,0,46,
162,0,18,
189,186,183,181,180,178,176,175,174,172,168,166,158,157,107,0,4,11,12,13,14,
  16,19,20,24,26,28,29,30,31,32,33,36,37,40,41,42,43,45,47,49,50,53,171,
189,186,183,181,180,178,176,175,174,172,168,166,158,157,107,0,4,12,13,14,16,
  19,20,24,26,28,29,30,31,32,33,36,37,40,41,42,43,45,47,49,50,53,171,

};


static unsigned const char ag_astt[3469] = {
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,1,8,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,8,7,1,1,9,9,1,1,5,3,4,4,4,4,4,4,4,4,4,1,4,4,4,4,4,4,4,
  7,0,1,1,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,9,9,9,
  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,5,5,1,5,1,1,1,7,1,
  1,3,2,2,2,7,2,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,10,10,10,1,10,10,10,1,1,1,7,1,1,3,1,1,7,2,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,2,2,2,7,2,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,2,1,1,1,1,1,1,
  1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,2,1,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,5,5,1,
  5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,
  7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,2,2,2,7,1,1,1,2,2,2,7,1,1,1,5,5,1,5,1,1,1,
  7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,2,1,1,1,1,1,1,1,1,1,2,1,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,
  1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,5,1,5,1,1,1,7,1,1,3,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,
  7,1,1,3,1,7,3,2,2,2,7,1,1,1,1,7,3,1,1,1,7,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,
  2,7,2,2,1,2,2,1,1,2,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  7,1,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,
  2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,
  1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,1,2,1,2,
  2,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,7,1,5,5,1,5,1,1,1,7,1,1,3,2,2,2,7,2,1,
  1,1,2,2,2,7,2,1,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,7,1,1,1,1,1,1,1,1,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,
  1,5,2,2,2,2,2,5,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,4,5,5,5,5,5,5,5,5,5,5,10,
  5,10,2,4,5,5,5,7,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,4,5,5,5,5,5,5,5,5,5,5,10,
  5,10,10,4,5,5,5,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,
  7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,
  3,1,4,1,1,5,2,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,
  1,1,3,1,1,4,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,
  1,1,1,2,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,
  1,1,7,1,1,3,1,5,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,1,1,2,2,1,1,1,2,
  1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,1,1,
  2,2,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,1,2,2,2,2,2,2,2,
  1,1,1,1,1,1,1,1,1,1,5,1,1,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  2,2,7,2,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,2,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,2,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,
  1,1,1,1,1,1,1,2,2,2,7,2,1,1,7,2,5,5,1,5,1,1,1,7,1,1,3,2,2,2,7,2,1,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,
  2,1,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,
  1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,
  7,2,2,2,2,1,1,1,1,2,1,1,2,2,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,7,2,1,1,1,7,
  1,1,1,1,7,1,1,5,1,2,2,2,7,2,1,1,1,1,7,1,1,1,4,1,2,2,2,7,2,1,1,10,10,10,10,
  10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,1,1,2,2,2,2,2,2,2,2,
  2,2,2,2,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,2,1,2,2,7,2,2,2,2,2,7,2,2,2,5,2,
  2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,8,7,2,1,1,1,8,8,8,7,2,1,1,1,1,1,
  1,4,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,2,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,5,1,1,1,1,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,1,1,7,2,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,
  1,2,2,1,1,2,2,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,
  2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,1,1,2,2,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,2,
  1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,
  1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,
  1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,1,1,1,
  1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,1,7,2,1,7,2,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,2,
  2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,
  1,1,2,1,2,2,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,7,1,1,5,5,5,1,5,1,1,1,7,1,
  1,3,2,2,2,7,2,1,1,1,1,7,1,1,2,2,2,7,1,2,2,2,7,1,1,5,1,1,7,2,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,
  1,3,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,4,1,2,2,2,2,2,2,2,1,1,1,1,4,1,2,2,2,2,1,7,1,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,1,7,2,1,4,7,1,1,1,7,1,1,
  4,1,2,2,2,4,2,1,1,10,10,10,4,2,10,10,10,4,2,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,
  2,2,1,2,2,2,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,7,2,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,
  2,2,2,1,1,1,1,1,1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,7,1,4,4,4,4,4,4,
  4,4,4,4,1,4,4,4,4,4,4,7,1,1,1,1,4,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,
  1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,2,1,1,2,
  2,1,1,1,1,1,1,1,1,1,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,7,2,1,7,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,4,2,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,
  1,2,1,1,1,1,1,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,3,4,3,1,2,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
298,298,1,2,300,298,
27,27,27,27,27,27,27,27,27,9,27,27,27,27,27,27,27,4,0,11,10,
101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,101,
  101,101,101,101,101,101,101,101,103,
104,6,
96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,
  96,96,96,98,
99,8,
299,299,3,299,3,1,2,9,3,3,313,
105,105,105,10,44,13,12,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,11,14,
299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,
  299,299,106,106,106,3,106,106,106,3,1,2,12,3,3,311,
15,17,13,28,16,
18,19,20,21,22,33,31,32,23,26,29,17,105,105,105,1,14,63,3,3,3,44,43,42,39,
  13,40,18,13,38,37,36,35,34,13,40,43,42,41,30,28,27,25,24,12,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,15,3,3,322,
105,105,105,16,45,12,
299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,3,299,3,
  1,2,299,17,3,3,306,
299,299,3,299,3,1,2,18,3,3,329,
299,299,299,3,299,3,1,2,19,3,3,326,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,20,3,3,323,
299,299,3,299,3,1,2,21,3,3,321,
299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,3,299,3,1,2,
  22,3,3,320,
299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,3,299,3,1,2,23,
  3,3,314,
105,105,105,24,45,12,
46,105,105,105,25,48,47,12,
299,299,3,299,3,1,2,26,3,3,312,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,27,71,72,62,137,74,36,67,
  68,74,73,73,73,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
105,105,105,28,34,12,
299,299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,29,3,3,
  308,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,30,75,
299,299,3,299,3,1,2,31,3,3,316,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,32,3,3,315,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,33,3,3,318,
17,34,25,
105,105,105,35,76,12,
15,17,36,23,77,
15,17,37,22,78,
65,66,51,69,64,17,105,105,125,52,49,133,131,125,105,38,71,72,62,137,74,79,
  67,20,68,74,73,73,73,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,
  59,53,
80,82,84,39,85,83,81,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,40,86,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,41,71,72,62,137,74,87,67,
  68,74,73,73,73,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
105,105,105,42,63,88,12,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,43,71,72,62,137,74,89,67,
  68,74,73,73,73,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
18,19,20,21,22,33,31,32,23,26,29,17,105,105,105,44,63,6,6,90,44,43,42,39,13,
  40,18,13,38,37,36,35,34,13,40,43,42,41,30,28,27,25,24,12,
64,45,91,
299,299,3,299,3,1,2,46,3,3,325,
105,105,105,47,44,92,12,
46,105,105,105,48,44,93,94,12,
140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
  140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,140,
  140,140,49,95,
130,130,130,114,130,
96,96,96,51,112,96,
102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,
  102,102,98,97,157,156,155,154,153,152,151,150,149,148,147,102,102,102,
  102,52,102,102,102,101,100,99,102,102,
299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,
  299,299,299,299,299,299,299,3,299,3,1,2,53,3,3,355,
103,104,54,
105,107,
136,135,135,135,136,124,134,
123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,115,123,123,
  123,123,123,123,123,123,123,123,132,123,132,129,115,123,123,123,57,
122,122,122,122,122,122,122,122,122,122,122,122,122,122,122,122,113,122,122,
  122,122,122,122,122,122,122,122,126,122,126,126,113,122,122,122,58,126,
299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,
  299,299,299,299,299,299,299,3,3,1,2,59,3,3,337,
299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,
  299,299,299,299,299,299,299,3,3,1,2,60,3,3,336,
299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,
  299,299,299,299,299,299,299,3,3,1,2,61,3,3,335,
64,63,106,
49,73,138,53,
299,299,299,299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,
  64,3,3,328,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,65,3,3,354,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,66,3,3,344,
80,82,59,83,81,
65,66,51,107,109,69,64,105,105,125,52,49,133,131,125,105,68,71,72,62,137,74,
  111,67,68,74,73,73,73,72,71,70,58,70,110,108,63,92,93,55,54,58,50,57,56,
  12,61,60,59,53,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,69,3,3,330,
112,55,113,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,71,71,72,62,137,74,67,68,
  54,54,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,72,71,72,62,137,74,67,68,
  53,53,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
114,115,116,117,118,119,120,48,121,85,86,87,88,89,90,91,
122,123,66,125,126,127,128,129,130,69,46,131,124,81,75,76,77,78,79,80,82,83,
  84,
18,19,20,21,22,132,33,31,32,23,26,29,17,105,105,105,75,63,3,3,3,44,43,42,39,
  13,40,18,13,38,37,36,35,34,13,40,43,42,41,30,33,28,27,25,24,12,
82,76,133,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,77,71,72,62,137,74,37,67,
  68,74,73,73,73,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
105,105,105,78,35,12,
17,79,21,
299,299,3,299,3,1,2,80,3,3,334,
105,105,105,81,65,12,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,82,3,3,310,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,83,71,72,62,137,74,134,67,
  68,74,73,73,73,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,84,3,3,304,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,85,71,72,62,137,74,135,67,
  68,74,73,73,73,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
18,19,20,21,22,33,31,32,23,26,29,136,17,105,105,105,86,63,3,3,3,44,43,42,39,
  13,40,137,18,13,38,37,36,35,34,13,40,43,42,41,30,28,27,25,24,12,
138,87,32,
80,82,84,88,139,83,81,
140,89,141,
142,5,143,
105,105,105,91,44,144,12,
145,15,92,16,146,
15,41,16,
105,105,105,94,44,147,12,
141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,141,
  141,141,141,98,97,157,156,155,154,153,152,151,150,149,148,147,139,141,
  141,141,95,141,141,141,141,101,100,99,141,
96,96,96,120,121,96,
161,161,97,
167,166,166,166,167,98,164,
163,163,159,
162,162,158,
167,166,166,166,167,146,165,
168,102,
96,96,96,111,110,96,
148,149,149,149,149,104,109,149,
148,149,149,149,149,105,108,149,
65,66,51,69,64,67,105,105,125,52,49,133,131,125,105,106,71,72,62,137,74,69,
  67,68,74,73,73,73,72,71,70,58,70,151,150,63,92,93,55,54,58,50,57,56,12,
  61,60,59,53,
299,3,3,1,2,107,3,3,333,
152,108,153,
299,3,3,1,2,109,3,3,332,
152,110,154,
152,111,57,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,112,3,3,331,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,113,71,72,62,137,74,67,68,
  56,56,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,114,3,3,353,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,115,3,3,352,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,116,3,3,351,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,117,3,3,350,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,118,3,3,349,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,119,3,3,348,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,120,3,3,347,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,121,71,72,62,137,74,67,68,
  51,51,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,122,3,3,346,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,123,3,3,345,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,124,71,72,62,137,74,67,68,
  155,155,155,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,125,3,3,343,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,126,3,3,342,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,127,3,3,341,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,128,3,3,340,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,129,3,3,339,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,130,3,3,338,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,131,71,72,62,137,74,67,68,
  156,73,73,73,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,3,299,3,
  1,2,299,132,3,3,319,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,133,71,72,62,137,74,157,67,
  68,74,73,73,73,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
158,134,64,
17,135,17,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,136,3,3,307,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,137,71,72,62,137,74,159,67,
  68,74,73,73,73,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
299,299,299,299,299,299,299,299,299,299,299,299,299,299,3,299,3,1,2,138,3,3,
  317,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,139,71,72,62,137,74,160,67,
  68,74,73,73,73,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
299,299,299,299,299,299,299,299,299,299,299,299,299,299,3,299,3,1,2,140,3,3,
  302,
18,19,20,21,22,33,31,32,23,26,29,17,105,105,105,141,63,8,11,44,43,42,39,13,
  40,18,13,38,37,36,35,34,13,40,43,42,41,30,28,27,25,24,12,
299,299,299,299,299,299,299,299,299,299,299,299,299,299,3,299,3,1,2,142,3,3,
  301,
18,19,20,21,22,33,31,32,23,26,29,17,105,105,105,143,63,7,10,44,43,42,39,13,
  40,18,13,38,37,36,35,34,13,40,43,42,41,30,28,27,25,24,12,
152,15,144,16,161,
299,299,299,3,299,3,1,2,145,3,3,324,
105,105,105,146,44,162,12,
145,15,147,16,163,
127,127,127,148,164,
127,127,127,149,165,
15,68,166,
152,151,66,
299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,
  299,299,299,299,299,299,299,299,299,299,299,299,299,3,299,299,299,299,
  299,299,3,1,2,152,3,3,327,
51,64,105,105,125,52,49,133,131,125,105,153,71,72,62,137,74,67,68,62,58,62,
  63,55,54,58,50,57,56,12,61,60,59,53,
51,64,105,105,125,52,49,133,131,125,105,154,71,72,62,137,74,67,68,61,58,61,
  63,55,54,58,50,57,56,12,61,60,59,53,
114,115,116,117,118,119,120,49,121,85,86,87,88,89,90,91,
122,123,66,69,47,124,81,82,83,84,
158,157,167,
299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,299,
  299,299,299,299,299,299,299,299,299,299,3,3,1,2,158,3,3,309,
17,159,16,
168,14,160,170,169,
22,161,171,
15,40,16,
105,105,105,39,44,172,12,
128,128,128,119,128,
128,128,128,118,128,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,166,71,72,62,137,74,70,67,
  68,74,73,73,73,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
17,167,24,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,168,3,3,305,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,169,71,72,62,137,74,15,67,
  68,74,73,73,73,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
173,170,174,
27,27,27,27,27,27,27,27,27,27,9,27,27,27,27,27,27,171,175,10,176,
15,38,16,
299,299,299,299,299,299,299,299,3,299,299,299,299,299,299,3,1,2,173,3,3,303,
65,66,51,69,64,105,105,125,52,49,133,131,125,105,174,71,72,62,137,74,177,67,
  68,74,73,73,73,72,71,70,58,70,63,92,93,55,54,58,50,57,56,12,61,60,59,53,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,175,178,
132,176,42,
140,177,179,
18,19,20,21,22,33,31,32,23,26,29,17,105,105,105,43,63,3,3,3,44,43,42,39,13,
  40,18,13,38,37,36,35,34,13,40,43,42,41,30,28,27,25,24,12,
18,19,20,21,22,33,31,32,23,26,29,17,105,105,105,179,63,9,12,44,43,42,39,13,
  40,18,13,38,37,36,35,34,13,40,43,42,41,30,28,27,25,24,12,

};


static const unsigned short ag_sbt[] = {
     0,  25,  56,  88,  94, 115, 143, 145, 174, 176, 187, 194, 212, 247,
   252, 297, 319, 325, 352, 363, 375, 397, 408, 433, 457, 463, 471, 482,
   528, 534, 557, 575, 586, 608, 630, 633, 639, 644, 649, 697, 704, 722,
   768, 775, 821, 865, 868, 879, 886, 895, 936, 941, 947, 994,1029,1032,
  1034,1041,1077,1114,1148,1182,1216,1219,1223,1248,1270,1292,1297,1347,
  1369,1372,1415,1458,1474,1497,1543,1546,1592,1598,1601,1612,1618,1640,
  1686,1708,1754,1800,1803,1810,1813,1816,1823,1828,1831,1838,1886,1892,
  1895,1902,1905,1908,1915,1917,1923,1931,1939,1988,1997,2000,2009,2012,
  2015,2037,2080,2102,2124,2146,2168,2190,2212,2234,2277,2299,2321,2365,
  2387,2409,2431,2453,2475,2497,2542,2569,2615,2618,2621,2643,2689,2712,
  2758,2781,2824,2847,2890,2895,2907,2914,2919,2924,2929,2932,2935,2981,
  3015,3049,3065,3075,3078,3115,3118,3123,3126,3129,3136,3141,3146,3192,
  3195,3217,3263,3266,3287,3290,3312,3358,3376,3379,3382,3426,3469
};


static const unsigned short ag_sbe[] = {
    21,  53,  85,  92, 111, 142, 144, 173, 175, 183, 190, 210, 243, 249,
   268, 315, 322, 348, 359, 371, 393, 404, 429, 453, 460, 467, 478, 496,
   531, 553, 573, 582, 604, 626, 631, 636, 641, 646, 664, 700, 720, 736,
   771, 789, 836, 866, 875, 882, 890, 934, 939, 944, 985,1025,1031,1033,
  1039,1076,1112,1144,1178,1212,1217,1220,1244,1266,1288,1294,1313,1365,
  1370,1386,1429,1465,1484,1513,1544,1560,1595,1599,1608,1615,1636,1654,
  1704,1722,1770,1801,1806,1811,1814,1819,1825,1829,1834,1877,1889,1894,
  1900,1904,1907,1913,1916,1920,1928,1936,1954,1993,1998,2005,2010,2013,
  2033,2051,2098,2120,2142,2164,2186,2208,2230,2248,2295,2317,2335,2383,
  2405,2427,2449,2471,2493,2511,2565,2583,2616,2619,2639,2657,2708,2726,
  2777,2796,2843,2862,2892,2903,2910,2916,2922,2927,2930,2933,2977,2992,
  3026,3056,3069,3076,3111,3116,3120,3124,3127,3132,3139,3144,3160,3193,
  3213,3231,3264,3283,3288,3308,3326,3374,3377,3380,3397,3441,3469
};


static const unsigned char ag_fl[] = {
  2,3,0,2,1,1,2,4,4,9,4,4,9,1,0,2,5,4,1,1,2,3,2,2,6,2,1,0,3,1,1,1,3,3,2,
  3,2,3,6,5,5,3,8,2,1,3,1,3,1,3,1,3,1,2,2,1,3,3,1,1,1,4,4,1,4,3,4,0,1,1,
  3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,
  1,2,1,3,3,3,2,2,1,1,1,0,1,2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,2,1,1,1,2,3,
  0,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2
};

static const unsigned char ag_ptt[] = {
    0,  7,  9,  9, 11, 11, 12, 12, 12, 12, 13, 13, 13, 13, 22, 22, 24, 24,
   24, 24, 24, 24, 24, 24, 24, 24, 24,  8,  8, 26, 16, 19, 14, 29, 31, 31,
   32, 32, 36, 36, 36, 36, 37, 56, 39, 39, 17, 17, 57, 57, 59, 59, 61, 61,
   61, 63, 63, 66, 66, 66, 66, 66, 66, 20, 20, 20, 69, 73, 73, 74, 74, 68,
   68, 68, 68, 58, 58, 58, 58, 58, 58, 60, 60, 60, 60, 62, 62, 62, 62, 62,
   62, 62, 65, 65,  1, 96, 96, 97, 97,  1,101,101,102,102,  1,171,171,196,
  196,196,106,106,106,109,109,109,114,114,108,108,110,110,195,195,195,111,
  111,115,115,112,112,113,113,117,117,124,124, 75, 75,215,127,127,128,128,
  130,130,130,131,131,131,131,131,131,131,131,131,131,131,132,132,132,145,
  146,147,133,133,150,150,197,152,152,116, 93, 93, 95, 95, 95, 95, 95, 95,
   95, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95, 95,
   95, 95, 95, 95,100,100,100,100,100,100,100,100,100,100,100,100,100,100,
  100,100,100,100,100,100,100,100,100,100,100,100,100,104,104,104,105,105,
  105,105,105,105,118,118,119,119,119,122,122,125,125,129,129,129,129,129,
  129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
  129,129,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,153,
  153,153,153,153,153,153,153,153,153,159,159,160,160, 15, 18, 23, 21, 25,
   28, 27, 30, 35, 34,  4, 33, 38, 40, 41, 42, 44, 43, 46, 45, 47, 48, 49,
   52, 51, 50, 55, 54, 53, 64, 67, 70, 71, 72,  2,  3,  6, 76, 77, 78, 79,
   80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: VRO(AG_WRAP_9 *, ag_rp_1(PCB_POINTER, VW(0, AG_WRAP_9 *), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 2: VNO AG_WRAP_6(ag_rp_2(PCB_POINTER)); break;
    case 3: VRO(AG_WRAP_6 *, ag_rp_3(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 4: VRO(AG_WRAP_6 *, ag_rp_4(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 5: VRO(AG_WRAP_6 *, ag_rp_5(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 6: VNO AG_WRAP_6(ag_rp_6(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 7: VNO AG_WRAP_6(ag_rp_7(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *), VW(4, AG_WRAP_6 *), VW(6, AG_WRAP_6 *), VW(8, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); VWD(6, AG_WRAP_6 *); VWD(8, AG_WRAP_6 *); break;
    case 8: VRO(AG_WRAP_6 *, ag_rp_8(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 9: VNO AG_WRAP_6(ag_rp_9(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 10: VNO AG_WRAP_6(ag_rp_10(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *), VW(4, AG_WRAP_6 *), VW(6, AG_WRAP_6 *), VW(8, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); VWD(6, AG_WRAP_6 *); VWD(8, AG_WRAP_6 *); break;
    case 11: VRO(AG_WRAP_6 *, ag_rp_11(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 12: VNO AG_WRAP_6(ag_rp_12(PCB_POINTER)); break;
    case 13: VNO AG_WRAP_6(ag_rp_13(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 14: VNO AG_WRAP_6(ag_rp_14(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 15: VRO(AG_WRAP_6 *, ag_rp_15(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 16: VNO AG_WRAP_6(ag_rp_16(PCB_POINTER)); break;
    case 17: VNO AG_WRAP_6(ag_rp_17(PCB_POINTER)); break;
    case 18: VNO AG_WRAP_6(ag_rp_18(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 19: VNO AG_WRAP_6(ag_rp_19(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_6 *); break;
    case 20: VNO AG_WRAP_9(ag_rp_20(PCB_POINTER)); break;
    case 21: VNO AG_WRAP_9(ag_rp_21(PCB_POINTER, VW(1, AG_WRAP_9 *))); 
            VWD(1, AG_WRAP_9 *); break;
    case 22: VNO AG_WRAP_6(ag_rp_22(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 23: VNO AG_WRAP_6(ag_rp_23(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 24: VNO AG_WRAP_6(ag_rp_24(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 25: VRO(AG_WRAP_6 *, ag_rp_25(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 26: VNO AG_WRAP_6(ag_rp_26(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 27: VRO(AG_WRAP_6 *, ag_rp_27(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 28: VNO AG_WRAP_6(ag_rp_28(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_9 *), VW(5, AG_WRAP_9 *))); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_9 *); VWD(5, AG_WRAP_9 *); break;
    case 29: VNO AG_WRAP_6(ag_rp_29(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_9 *))); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_9 *); break;
    case 30: VNO AG_WRAP_6(ag_rp_30(PCB_POINTER, VW(2, AG_WRAP_9 *), VW(4, AG_WRAP_9 *))); 
            VWD(2, AG_WRAP_9 *); VWD(4, AG_WRAP_9 *); break;
    case 31: VNO AG_WRAP_6(ag_rp_31(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(2, AG_WRAP_9 *))); 
            VWD(1, AG_WRAP_4 *); VWD(2, AG_WRAP_9 *); break;
    case 32: VNO AG_WRAP_6(ag_rp_32(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_9 *), VW(6, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_9 *); VWD(6, AG_WRAP_4 *); break;
    case 33: VRO(AG_WRAP_9 *, ag_rp_33(PCB_POINTER)); break;
    case 34: VRO(AG_WRAP_4 *, ag_rp_34(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 35: VRO(AG_WRAP_9 *, ag_rp_35(PCB_POINTER, VW(0, AG_WRAP_9 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 36: VRO(AG_WRAP_6 *, ag_rp_36(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 37: VRO(AG_WRAP_6 *, ag_rp_37(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 38: VRO(AG_WRAP_6 *, ag_rp_38(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 39: VNO AG_WRAP_6(ag_rp_39(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 40: VNO AG_WRAP_6(ag_rp_40(PCB_POINTER, V(0,(Opcode *)), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 41: VRO(AG_WRAP_6 *, ag_rp_41(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 42: VNO AG_WRAP_6(ag_rp_42(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 43: VRO(AG_WRAP_8 *, ag_rp_43(PCB_POINTER, VW(0, AG_WRAP_8 *))); break;
    case 44: VRO(AG_WRAP_6 *, ag_rp_44(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 45: VNO AG_WRAP_6(ag_rp_45(PCB_POINTER, VW(3, AG_WRAP_6 *))); 
            VWD(3, AG_WRAP_6 *); break;
    case 46: VNO AG_WRAP_6(ag_rp_46(PCB_POINTER, VW(3, AG_WRAP_6 *))); 
            VWD(3, AG_WRAP_6 *); break;
    case 47: VRO(AG_WRAP_4 *, ag_rp_47(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 48: VRO(AG_WRAP_6 *, ag_rp_48(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 49: VRO(AG_WRAP_6 *, ag_rp_49(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 50: VRO(AG_WRAP_4 *, ag_rp_50(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_7 *))); 
            VWD(2, AG_WRAP_7 *); break;
    case 51: VNO AG_WRAP_7(ag_rp_51(PCB_POINTER)); break;
    case 52: VRO(AG_WRAP_6 *, ag_rp_52(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 53: VRO(AG_WRAP_7 *, ag_rp_53(PCB_POINTER, VW(0, AG_WRAP_7 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 54: VNO AG_WRAP_8(ag_rp_54(PCB_POINTER, V(0,(long *)))); break;
    case 55: VNO AG_WRAP_8(ag_rp_55(PCB_POINTER, V(0,(double *)))); break;
    case 56: VNO AG_WRAP_8(ag_rp_56(PCB_POINTER, V(0,(int *)))); break;
    case 57: V(0,(Opcode *)) = ag_rp_57(PCB_POINTER); break;
    case 58: V(0,(Opcode *)) = ag_rp_58(PCB_POINTER); break;
    case 59: V(0,(Opcode *)) = ag_rp_59(PCB_POINTER); break;
    case 60: V(0,(Opcode *)) = ag_rp_60(PCB_POINTER); break;
    case 61: V(0,(Opcode *)) = ag_rp_61(PCB_POINTER); break;
    case 62: V(0,(Opcode *)) = ag_rp_62(PCB_POINTER); break;
    case 63: V(0,(Opcode *)) = ag_rp_63(PCB_POINTER); break;
    case 64: V(0,(Opcode *)) = ag_rp_64(PCB_POINTER); break;
    case 65: V(0,(Opcode *)) = ag_rp_65(PCB_POINTER); break;
    case 66: V(0,(Opcode *)) = ag_rp_66(PCB_POINTER); break;
    case 67: V(0,(Opcode *)) = ag_rp_67(PCB_POINTER); break;
    case 68: V(0,(Opcode *)) = ag_rp_68(PCB_POINTER); break;
    case 69: V(0,(Opcode *)) = ag_rp_69(PCB_POINTER); break;
    case 70: V(0,(Opcode *)) = ag_rp_70(PCB_POINTER); break;
    case 71: V(0,(Opcode *)) = ag_rp_71(PCB_POINTER); break;
    case 72: V(0,(Opcode *)) = ag_rp_72(PCB_POINTER); break;
    case 73: V(0,(Opcode *)) = ag_rp_73(PCB_POINTER); break;
    case 74: V(0,(Opcode *)) = ag_rp_74(PCB_POINTER); break;
    case 75: V(0,(Opcode *)) = ag_rp_75(PCB_POINTER); break;
    case 76: VNO AG_WRAP_4(ag_rp_76(PCB_POINTER, V(0,(int *)))); break;
    case 77: VRO(AG_WRAP_4 *, ag_rp_77(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 78: V(0,(double *)) = ag_rp_78(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 79: V(0,(double *)) = ag_rp_79(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 80: V(0,(double *)) = ag_rp_80(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 81: V(0,(double *)) = ag_rp_81(PCB_POINTER, V(0,(double *))); break;
    case 82: V(0,(double *)) = ag_rp_82(PCB_POINTER, V(1,(double *))); break;
    case 83: V(0,(double *)) = ag_rp_83(PCB_POINTER, V(0,(long *))); break;
    case 84: V(0,(double *)) = ag_rp_84(PCB_POINTER, V(0,(long *))); break;
    case 85: V(0,(double *)) = ag_rp_85(PCB_POINTER, V(0,(long *))); break;
    case 86: V(0,(long *)) = ag_rp_86(PCB_POINTER, V(1,(long *))); break;
    case 87: V(0,(long *)) = ag_rp_87(PCB_POINTER, V(1,(long *))); break;
    case 88: V(0,(double *)) = ag_rp_88(PCB_POINTER, V(0,(int *))); break;
    case 89: V(0,(double *)) = ag_rp_89(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 90: V(0,(long *)) = ag_rp_90(PCB_POINTER, V(0,(int *))); break;
    case 91: V(0,(long *)) = ag_rp_91(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 92: V(0,(long *)) = ag_rp_92(PCB_POINTER, V(0,(int *))); break;
    case 93: V(0,(long *)) = ag_rp_93(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 94: V(0,(long *)) = ag_rp_94(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 95: V(0,(long *)) = ag_rp_95(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 96: V(0,(long *)) = ag_rp_96(PCB_POINTER); break;
    case 97: V(0,(long *)) = ag_rp_97(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 98: V(0,(long *)) = ag_rp_98(PCB_POINTER); break;
    case 99: V(0,(long *)) = ag_rp_99(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 100: V(0,(int *)) = ag_rp_100(PCB_POINTER, V(0,(int *))); break;
    case 101: V(0,(int *)) = ag_rp_101(PCB_POINTER, V(0,(int *))); break;
    case 102: VRO(AG_WRAP_5 *, ag_rp_102(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 103: VRO(AG_WRAP_8 *, ag_rp_103(PCB_POINTER, VW(0, AG_WRAP_8 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 104: VNO AG_WRAP_5(ag_rp_104(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 105: VNO AG_WRAP_4(ag_rp_105(PCB_POINTER)); break;
    case 106: VRO(AG_WRAP_4 *, ag_rp_106(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 107: V(0,(int *)) = ag_rp_107(PCB_POINTER); break;
    case 108: V(0,(int *)) = ag_rp_108(PCB_POINTER); break;
    case 109: V(0,(int *)) = ag_rp_109(PCB_POINTER); break;
    case 110: V(0,(int *)) = ag_rp_110(PCB_POINTER); break;
    case 111: V(0,(int *)) = ag_rp_111(PCB_POINTER); break;
    case 112: V(0,(int *)) = ag_rp_112(PCB_POINTER); break;
    case 113: V(0,(int *)) = ag_rp_113(PCB_POINTER); break;
    case 114: V(0,(int *)) = ag_rp_114(PCB_POINTER); break;
    case 115: V(0,(int *)) = ag_rp_115(PCB_POINTER); break;
    case 116: V(0,(int *)) = ag_rp_116(PCB_POINTER); break;
    case 117: V(0,(int *)) = ag_rp_117(PCB_POINTER); break;
    case 118: V(0,(int *)) = ag_rp_118(PCB_POINTER, V(1,(int *))); break;
    case 119: V(0,(int *)) = ag_rp_119(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 120: V(0,(int *)) = ag_rp_120(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 121: V(0,(int *)) = ag_rp_121(PCB_POINTER, V(1,(int *))); break;
    case 122: V(0,(int *)) = ag_rp_122(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 123: V(0,(int *)) = ag_rp_123(PCB_POINTER, V(0,(int *))); break;
    case 124: V(0,(int *)) = ag_rp_124(PCB_POINTER, V(0,(int *))); break;
    case 125: V(0,(int *)) = ag_rp_125(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES dci_token_names
const char *const dci_token_names[216] = {
  "script",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "script",
  "local statement",
  "statement list",
  "eof",
  "statement",
  "open statement",
  "closed statement",
  "if condition",
  "\"ELSE\"",
  "WHILE",
  "expression",
  "\"DO\"",
  "FOR",
  "lvalue",
  "\":=\"",
  "for increment",
  "\"TO\"",
  "simple statement",
  "\"BY\"",
  "REPEAT",
  "\"UNTIL\"",
  "';'",
  "compound statement",
  "\"RETURN\"",
  "dump statement",
  "print statement",
  "\"ARRAY\"",
  "'['",
  "']'",
  "structure declaration",
  "function definition",
  "\"LOCAL\"",
  "name list",
  "\"REPEAT\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"IF\"",
  "\"THEN\"",
  "\"BEGIN\"",
  "\"END\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "\"STRUCT\"",
  "'{'",
  "'}'",
  "\"FUNCTION\"",
  "'('",
  "')'",
  "function body",
  "simple expression",
  "relational op",
  "term",
  "additive op",
  "unary expression",
  "multiplicative op",
  "factor",
  "'+'",
  "unary op",
  "primary",
  "\"**\"",
  "constant",
  "function call",
  "\"LONG\"",
  "\"DOUBLE\"",
  "'.'",
  "optional arg list",
  "arg list",
  "string",
  "'='",
  "\"<>\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0X\"",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\A\"",
  "\"\\\\B\"",
  "\"\\\\F\"",
  "\"\\\\N\"",
  "\"\\\\R\"",
  "\"\\\\T\"",
  "\"\\\\V\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\X\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "\"ELSE\"",
  "\"DO\"",
  "\"TO\"",
  "\":=\"",
  "\"BY\"",
  "';'",
  "\"UNTIL\"",
  "\"RETURN\"",
  "']'",
  "'['",
  "name",
  "\"ARRAY\"",
  "\"LOCAL\"",
  "\"REPEAT\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"THEN\"",
  "\"IF\"",
  "\"END\"",
  "\"BEGIN\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "'}'",
  "'{'",
  "\"STRUCT\"",
  "')'",
  "'('",
  "\"FUNCTION\"",
  "'+'",
  "\"**\"",
  "\"LONG\"",
  "\"DOUBLE\"",
  "'.'",
  "integer",
  "real",
  "character constant",
  "'='",
  "\"<>\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (dci_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (dci_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_dci(dci_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void dci(dci_pcb_type *PCB_POINTER) {
  init_dci(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


